// Import data from WordNet 1.5
// by roehrig@vsb.cs.uni-frankfurt.de
// $Id: extr.cc,v 1.6 1996/09/16 09:45:47 roemer Exp $


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <fstream.h>

/*
 * define this to connect orphaned words to "something"
 */
/* #define LOST_WORDS */

extern "C" 
{
#include "wn.h"
#include "wnconsts.h"
#include "wnglobals.h"
}


#include <valid.h>
#include <dict.h>
#include <map>

extern "C" {
    extern char *optarg;
    extern int optind;
}

const char ait_syn[]="SYNONYM";
const char ait_ant[]="ANTONYM";
const char ait_isa[]="IS_A";
const char ait_hasa[]="HAS_A";


typedef int (*linefunc)(const char *, void *);
const int dotcount=1000;


// get all non-empty lines of the file

void getlines(const char *filename, linefunc lf, void *li)
{
    FILE *in=fopen(filename, "r");
    ASSERT(in);

    unsigned bufsize=100000;
    char *buf=(char*)malloc(bufsize);
    ASSERT(buf);

    while (!feof(in))
    {
        // read the line
#if 0
        unsigned chars=getdelim(&buf, &bufsize, '\n', in);

#else
        if (!fgets(buf, bufsize, in))
            // EOF or error
            break;
        unsigned chars=strlen(buf);
#endif

        // empty lines are of no interest
        if (chars<=1)
            continue;

        // remove the trailing newline
        buf[chars-1]=0;

        // pass the line
        if (!lf(buf, li))
            break;
    }

    free(buf);
    fclose(in);
}


int addword(const char *t, void *li)
{
    static int count=0;
    if (!(count++%dotcount))
    {
        printf(".");
        fflush(stdout);
    }
    
    Word w(t, true);
    Dict *dptr=(Dict*)li;
    dptr->AddWord(w);
    return 1;
}

int connectorphan (const char *t, void *li)
{
    static int count=0;
    if (!(count++%dotcount))
    {
        printf(".");
        fflush(stdout);
    }
    
    Word w(t, true);
    Dict *dptr=(Dict*)li;
    dptr->AddRelation (w, "IS_A", "something");
    return 1;
}


struct lineinfo_addrels
{
    Dict *dptr;
    int dbase;
};

// In wordnet 1.5 there are words with sense index 0 in different
// synsets! (e.g. both GOAL as game-equipment and GOAL as something you
// want to achieve have sense 0 but are in different synsets).
// This function records for all words the filepos of the first sense #0
// occurence.

static inline bool
is_sense0 (SynsetPtr synset, int idx)
{
    typedef map<string, long, less<string> > msl;
    static msl sensemap;

    if (synset->wnsense[idx] != 0)
        return false;

    string w = Word (synset->words[idx], true);
    msl::iterator pos = sensemap.find (w);
    if (pos != sensemap.end())
        return (*pos).second == synset->hereiam;

    sensemap[w] = synset->hereiam;
    return true;
}

int addrels(const char *buf, void *linf)
{
    static Word emptyword("");

    // ignore comments and other stuff
    if (!isdigit(buf[0]))
        return 1;
    
    // count the numbers of lines processed
    static int count=0;
    if (!(count++%dotcount))
    {
        printf(".");
        fflush(stdout);
    }
#if 0
    printf("count=%i\n", count);
#endif
#if 0
    if (count==1000)
        return 0;
#endif
    
    lineinfo_addrels *const li=(lineinfo_addrels*)linf;
    const int filepos=atoi(buf);
    ASSERT(filepos!=0);

    // read the synset
    SynsetPtr s=read_synset(li->dbase, filepos, "");

    // add synonym relation between words of synset
    Word a(emptyword);
    for (int i=0; i<s->wcount; ++i)
    {
        // we are only interested in sense 0
        if (!is_sense0 (s, i))
            continue;

        Word b(s->words[i], true);
            
        if (a!=emptyword)
            li->dptr->AddRelation(a, ait_syn, b);

        a=b;
    }

    if (a!=emptyword)
    {
        // there is at least one word for the source of the relation
        
        // add other relations
        for (int i=0; i<s->ptrcount; ++i)
        {
            // we are only interested in relations among the same kind of words
            if (s->ppos[i]!=li->dbase)
                continue;
            
#if 0 // what is the difference? for nouns 2352 relations less
            // we are only interested in relations that apply to whole synsets
            if (s->pfrm[i]!=0 || s->pto[i]!=0)
                continue;

            // read the destination synset
            SynsetPtr dest=read_synset(li->dbase, s->ptroff[i], "");
            ASSERT(dest);
#else
            // read the destination synset
            SynsetPtr dest=read_synset(li->dbase, s->ptroff[i], "");
            ASSERT(dest);

            // but don't ignore sense 0 -> sense 0 relations. Most of the
            // ANTONYM relations do not apply to a whole synset.
            if ((s->pfrm[i] != 0 && s->wnsense[s->pfrm[i]-1] != 0) ||
                (s->pto[i]  != 0 && dest->wnsense[s->pto[i]-1] != 0)) {
                free_synset (dest);
                continue;
            }
#endif

            // look for a destination word that is hit by its 0 sense
            Word b(emptyword);
            Word a2(a);
            if (s->pto[i] != 0) {
                int j = s->pto[i] - 1;
                if (is_sense0 (dest, j)) {
                    a2 = Word (s->words[s->pfrm[i]-1], true);
                    b = Word (dest->words[j], true);
                }
            } else {
                for (int j=0; j<dest->wcount; ++j) {
                    if (is_sense0 (dest, j)) {
                        b = Word (dest->words[j], true);
                        break;
                    }
                }
            }

            // add the relation
            if (b!=emptyword)
            {
                const char *rel;
                int swap=0;
                switch (s->ptrtyp[i])
                {
                case ANTPTR :
                    rel=ait_ant;
                    break;
                case HYPERPTR :
                    rel=ait_isa;
                    swap=1;
                    break;
                case HYPOPTR :
                    rel=ait_isa;
                    swap=0;
                    break;
                case ISMEMBERPTR :
                case ISSTUFFPTR :
                case ISPARTPTR :
                    rel=ait_hasa;
                    swap=0;
                    break;
                case HASMEMBERPTR :
                case HASSTUFFPTR :
                case HASPARTPTR :
                    rel=ait_hasa;
                    swap=1;
                    break;
                default :
                    printf("!");
                    rel=0;
                    break;
                }
                if (rel)
                    if (swap)
#if 1 // register relations only in one direction
                        ;
#else
                        li->dptr->AddRelation(a2, rel, b);
#endif
                    else
                        li->dptr->AddRelation(b, rel, a2);
            }
        
            free_synset(dest);
        }
    }
    
    free_synset(s);

    return 1;
}


void
usage (void)
{
    cerr.form ("usage: extr [-v] [-n] -o output file <word file>\n");
    exit (1);
}

main(int argc, char *argv[])
{
    Dict d;

    d.RegisterRelation(ait_syn, Dict::RelSymmetric|Dict::RelReflexive|Dict::RelTransitive);
    d.RegisterRelation(ait_ant, Dict::RelSymmetric);
    d.RegisterRelation(ait_isa, Dict::RelTransitive|Dict::RelAcyclic|Dict::RelReflexive);
    d.RegisterRelation(ait_hasa, Dict::RelTransitive|Dict::RelAcyclic|Dict::RelReflexive);

    int c;
    char *outfilename = NULL;
    int dbase = NOUN;
    while ((c = getopt (argc, argv, "nvo:")) != EOF) {
        switch (c) {
        case 'v':
            dbase = VERB;
            break;
        case 'n':
            dbase = NOUN;
            break;
        case 'o':
            outfilename = optarg;
            break;
        case '?':
        default:
            usage ();
        }
    }
    if (!outfilename || optind >= argc)
        usage ();
    
    VERIFY(!wninit());

#if 0 // Choose whether to check new relations or not
    d.SetQuickMode(true);
#endif

    const char *wordfilename=argv[optind];
    char datafilename[1000];
    sprintf(datafilename, DATAFILE, getenv("WNSEARCHDIR"), partnames[dbase]);
    
    ofstream out(outfilename);
    ASSERT(out.good());

    printf(". = %i\n", dotcount);
    
    // read all the words and add them to the dictionary
    printf("Pass 1: Adding words to the dictionary\n");
    fflush(stdout);
    getlines(wordfilename, addword, (void*)&d);
    printf("\n");

    // add the relations
    printf("Pass 2: Adding relations\n");
    fflush(stdout);
    lineinfo_addrels li;
    li.dptr=&d;
    li.dbase=dbase;
    getlines(datafilename, addrels, (void*)&li);
    printf("\n");

#ifdef LOST_WORDS
    printf ("Pass 2a: Reconnecting orphaned words\n");
    fflush (stdout);
    d.AddWord ("something");
    getlines (wordfilename, connectorphan, (void *)&d);
    printf ("\n");
#endif
    
    printf("Pass 3: Writing output\n");
    fflush(stdout);
    out << d;
    
    return EXIT_SUCCESS;
}

