
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */



#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stream.h>
#include <fstream.h>
#include <strstream.h>

#include "cgparsetree.h" 
#include <Trader.h>


class Application {
private:
    int       argc;
    char      **argv;
    Server    *server;
    ConceptualGraph *LoadFile( char *Name);
public:
    Application( int an_argc, char *an_argv[] );
    ~Application();
    void Run();
};


Application::Application( int an_argc, char *an_argv[] )
{
	argc = an_argc;
	argv = an_argv;
	server = NULL;
}


Application::~Application()
{
	if( server )
		delete server;
}


ConceptualGraph *Application::LoadFile( char *Name )
{
	ifstream file( Name );
	char ch;
	
	if( !file ) {
		printf( "Couldn't open file '%s'\n", Name );
		exit( -1 );
	}
	ostrstream ostr;
	ostr << "@answer-graph=";
	while( !file.eof() ) {
		ch = file.get();
		ostr << ch;
	}
	ostr << ends;
	istrstream istr( ostr.str() );
//	cout << ostr.str() << endl;
	ostr.rdbuf()->freeze (0);

	Parser *parser = new Parser( server, istr );
	parser->Parse();
	delete parser;
	ConceptualGraph *cg;
	server->GetAnswer( &cg );
	return cg;
}

static void
usage (char *appname)
{
	cerr<<form("Usage: %s [-dce|-corba] [-o <output-file>] <input-file>\n",
		   appname);
	exit (1);
}

void Application::Run()
{
	enum { Nothing = 0, Corba = 1, Dce = 2 };
	int wanted = Nothing;
	ostream *ofile = &cout;
	char *OName = NULL;
	int i;

	for (i = 1; i < argc; ++i) {
		if (argv[i][0] != '-')
			break;
		if (!strcmp ("-o", argv[i])) {
			if (++i == argc)
				usage (argv[0]);
			OName = argv[i];
		} else if (!strcmp ("-dce", argv[i])) {
			wanted |= Dce;
		} else if (!strcmp ("-corba", argv[i])) {
			wanted |= Corba;
		} else {
			cerr << form ("invalid option: '%s'\n", argv[i]);
			usage (argv[0]);
		}
	}
	if (i >= argc || wanted == Nothing) {
		usage (argv[0]);
	}
	if (OName) {
		ofile = new ofstream (OName);
		if (!ofile || !*ofile) {
			cerr << form ("Cannot open output file '%s'\n", OName);
			exit (1);
		}
	}
	server = new Server();
	ConceptualGraph *cg = LoadFile( argv[i] );

	if (wanted & Corba) {
		Template2CORBA CORBA_con(cg, *ofile);
		CORBA_con.CreateSpecification();
	}
	if (wanted & Dce) {
		Template2DCE DCE_con(cg, *ofile);
		DCE_con.CreateSpecification();
	}
	if (ofile != &cout)
		delete ofile;
}



int main( int argc, char *argv[] )
{
	Application app( argc, argv );

	app.Run();
}

