/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#include <assert.h>
#include "cgtemplate.h"
#include "util.h"

CGTemplate::CGTemplate()
{
}


CGTemplate::~CGTemplate()
{
}


ConceptualGraph* CGTemplate::CreateNewConceptNode( IDLString cname,
                                                   IDLString iname )
{
    ConceptualGraph *graph;
    ConceptNode *concept;

    SetOfWord word_list;
    if(!(iname == "*NONAME*"))
    {
        word_list.insert(Word (iname, false));
    }
    concept = new ConceptNode(Word(cname, false), word_list);
    graph = new ConceptualGraph(concept, NULL, NULL);
    return(graph); 
}


ConceptualGraph* CGTemplate::CreateNewRelationNode( IDLString rname )
{
    ConceptualGraph *graph;
    RelationNode *relation;

    relation = new RelationNode(Word (rname, false));
    graph = new ConceptualGraph(relation, NULL, NULL);
    return(graph);
}


ConceptualGraph* CGTemplate::GetInstance( ConceptualGraph *cg,
                                          IDLString scope[],
                                          int nr, int maxnr )
{
    ConceptualGraph *graph_1;
    ConceptualGraph *graph_2;
    ConceptualGraph *graph_3;
    ConceptualGraph *result;
    ConceptNode *concept;
    IDLString cname;
    IDLString rname;
    IDLString name;
    IDLString instance;
    SetOfWord instance_list;
    SetOfWord::iterator i;
    int strike = 0;
 
    assert(cg->GetNode()->IsA() == RELATION_NODE);
    result = NULL;
    name = scope[nr];
    gsub(name, "::", "");
    graph_1 = cg;
    graph_2 = graph_1->GetWidth();
    assert(graph_2 != NULL);
    while(!strike)
    {
        concept = (ConceptNode*) graph_2->GetNode();
        cname = concept->GetName();
        if(cname == "MODULE")
        {
            instance_list = concept->GetInstances();
            i = instance_list.begin();
            assert(i != instance_list.end());
            instance = *i;
            if(instance == name)
            {
                if(nr < maxnr)
                {
                    rname = "SCOPE";
                    graph_3 = GetRelation(graph_2, rname);
                    assert(graph_3 != NULL);
                    result = GetInstance(graph_3, scope, ++nr, maxnr);
                    assert(result != NULL);
                }
                else
                    result = graph_1;
                strike = 1;
            }
        }
        if(cname == "INTERFACE")
        {
            instance_list = concept->GetInstances();
            i = instance_list.begin();
            assert(i != instance_list.end());
            instance = *i;
            if(instance == name)
            {
                assert(nr == maxnr);
                result = graph_1;
                strike = 1;
            }
        }
        if(!strike) 
        {
            graph_1 = graph_2;
            graph_2 = graph_2->GetDepth();
            if(graph_2 == NULL) 
            {
                assert(nr < maxnr);
                result = GetInstance(cg, scope, ++nr, maxnr);
                assert(result != NULL);
                strike = 1;
            } 
        }
    }
    return(result);
}

int CGTemplate::DetectConcept( ConceptualGraph *cg, IDLString name,
                               IDLString instance )
{
    ConceptualGraph *cgraph;
    ConceptNode *concept;
    IDLString cname;
    int result = 0;

    assert(cg->GetNode()->IsA() == RELATION_NODE);
    cgraph = cg->GetWidth();
    while(cgraph != NULL)
    {
        concept = (ConceptNode*) cgraph->GetNode();
        cname = concept->GetName();
        if(cname == name && CompareInstances(cgraph, instance))
            result = 1;
        cgraph = cgraph->GetDepth();
    }
    return(result);
}


ConceptualGraph* CGTemplate::GetPureConcept( ConceptualGraph *cg,
                                             IDLString name )
{
    ConceptualGraph *cgraph;
    ConceptualGraph *result;
    ConceptNode *concept;
    IDLString cname;

    assert(cg->GetNode()->IsA() == RELATION_NODE);
    result = NULL;
    cgraph = cg->GetWidth();
    while(cgraph != NULL)
    {
        concept = (ConceptNode*) cgraph->GetNode();
        cname = concept->GetName();
        if(cname == name)
            result = cgraph;
        cgraph = cgraph->GetDepth();
    }
    return(result);
}


ConceptualGraph* CGTemplate::GetConcept( ConceptualGraph *cg, 
                                         IDLString name, 
                                         IDLString instance )
{
    ConceptualGraph *graph;
    ConceptualGraph *result;
    ConceptNode *concept;
    IDLString cname;

    assert(cg->GetNode()->IsA() == RELATION_NODE);
    result = NULL;
    graph = cg->GetWidth();
    while(graph != NULL)
    {
        concept = (ConceptNode*) graph->GetNode();
        cname = concept->GetName();
        if(cname == name && CompareInstances(graph, instance))
            result = graph;
        graph = graph->GetDepth();
    }
    return(result);
}


int CGTemplate::CompareInstances( ConceptualGraph *target,
                                  IDLString name ) 
{
    ConceptualGraph *graph;
    ConceptualGraph *rgraph;
    ConceptNode *concept;
    SetOfWord instance_list;
    IDLString instance;
    IDLString rname;
    SetOfWord::iterator i;
    int result = 0;

    assert(target->GetNode()->IsA() == CONCEPT_NODE); 
    concept = (ConceptNode*) target->GetNode();
    instance_list = concept->GetInstances();
    i = instance_list.begin();
    if(i == instance_list.end()) 
    {
        rname = "ANNOTATIONS";
        graph = GetRelation(target, rname);
        rname = "IDENTIFIER";
        if (graph != NULL)
            rgraph = GetConcept(graph, rname, name);
        if(rgraph != NULL)
            result = 1;
    }	
    while(i != instance_list.end())
    {
        instance = *i;
        if(instance == name)
            result = 1;
        ++i;
    } 
    return(result);
}


ConceptualGraph* CGTemplate::GetRelation( ConceptualGraph *cg, 
                                          IDLString name )
{
    ConceptualGraph *graph;
    ConceptualGraph *result;
    RelationNode *relation;
    IDLString rname;

    assert(cg->GetNode()->IsA() == CONCEPT_NODE);
    result = NULL;
    graph = cg->GetWidth();
    while(graph != NULL)
    {
        relation = (RelationNode*) graph->GetNode();
        rname = relation->GetName();
        if(rname == name)
            result = graph;
        graph = graph->GetDepth();
    }
    return(result);
}


ConceptualGraph* CGTemplate::GetType( ConceptualGraph *cg,
                                      IDLString scope[],
                                      int field, int maxfields )
{
    ConceptualGraph *result;
    ConceptualGraph *graph_1;
    ConceptualGraph *graph_2;
    ConceptNode *concept;
    IDLString name;
    IDLString rname;
    IDLString cname;
    IDLString instance;
    SetOfWord instance_list;
    SetOfWord::iterator i;
    int again = 0;

    assert(cg->GetNode()->IsA() == RELATION_NODE);
    name = scope[field];
    gsub(name, "::", "");
    graph_1 = cg->GetWidth();
    if(graph_1 != NULL)
        again = 1;
    while(again)
    {
        concept = (ConceptNode*) graph_1->GetNode();
        cname = concept->GetName(); 
        if(field < maxfields)
        {
            if(cname == "MODULE" || cname == "INTERFACE")
            {
                instance_list = concept->GetInstances();
                i = instance_list.begin();
                instance = *i;
                if(instance == name)
                {
                    if(cname == "MODULE")
                        rname = "SCOPE";
                    else
                        rname = "TYPES";
                    graph_2 = GetRelation(graph_1, rname);
                    assert(graph_2 != NULL);
                    result = GetType(graph_2, scope, ++field, maxfields);
                    assert(result != NULL);
                    again = 0;
                }	
            }
        }
        else
        {
            if(cname == "TYPE")
            {
                instance_list = concept->GetInstances();
                i = instance_list.begin();
                instance = *i;
                if(instance == name)
                {
                    result = graph_1;
                    again = 0;
                }
            }
        }
        graph_1 = graph_1->GetDepth();
        if(graph_1 == NULL)
            again = 0;
    }
    return(result);
}


ConceptualGraph* CGTemplate::GetMatchingConcept( ConceptualGraph *target,
                                                 IDLString rname,
                                                 IDLString instance )
{
    ConceptualGraph *result;
    ConceptualGraph *rgraph;
    ConceptNode *concept;
    IDLString cname;
    IDLString name;

    assert(target->GetNode()->IsA() == CONCEPT_NODE);
    result = NULL;
    concept = (ConceptNode*) target->GetNode();
    cname = concept->GetName();
    if(cname == "MODULE") 
    {
        name = "SCOPE";
        rgraph = GetRelation(target, name);
        assert(rgraph != NULL);
    }
    else
        rgraph = GetRelation(target, rname);
    if(rgraph != NULL)
    {
        if(rname == "TYPES")
            cname = "TYPE";
        if(rname == "CONSTANTS")
            cname = "CONSTANT";
        if(rname == "OPERATIONS")
            cname = "OPERATION";
        result = GetConcept(rgraph, cname, instance);
    }
    return(result);
}


ConceptualGraph* CGTemplate::GetInsertPosition( ConceptualGraph *target,
                                                IDLString rname )
{
    ConceptualGraph *cgraph;
    ConceptualGraph *rgraph;
    ConceptualGraph *result;
    ConceptNode *concept;
    IDLString cname;
    IDLString name;
 
    assert(target->GetNode()->IsA() == CONCEPT_NODE);
    concept = (ConceptNode*) target->GetNode();
    cname = concept->GetName();
    if(cname == "MODULE")
    {
        name = "SCOPE";
        rgraph = GetRelation(target, name);
        assert(rgraph != NULL);
        cgraph = rgraph->GetWidth();
        assert(cgraph != NULL);
        while(cgraph != NULL) 
        {
            if(cgraph->GetDepth() == NULL)
                result = cgraph;
            cgraph = cgraph->GetDepth();
        }
    }
    if(cname == "INTERFACE")
    {
        rgraph = GetRelation(target, rname);
        if(rgraph != NULL) 
        {
            cgraph = rgraph->GetWidth();
            while(cgraph != NULL) 
            {
                if(cgraph->GetDepth() == NULL)
                    result = cgraph;
                cgraph = cgraph->GetDepth();
            }
        }
        else
            result = NULL;		
    }
    return(result);
}


int CGTemplate::CompareAnnotations( ConceptualGraph *target,
                                    ConceptualGraph *source )
{
    ConceptualGraph *cgraph;
    ConceptualGraph *rgraph;
    ConceptNode *concept;
    SetOfWord instance_list;
    IDLString cname;
    IDLString rname;
    IDLString scope_1;
    IDLString source_name_1;
    IDLString source_form_1;
    IDLString scope_2;
    IDLString source_name_2;
    IDLString source_form_2;
    SetOfWord::iterator i;
    int result = 0;

    scope_1 = "";
    scope_2 = "";
    rname = "ANNOTATIONS";
    rgraph = GetRelation(source, rname);
    assert(rgraph != NULL);
    cgraph = rgraph->GetWidth();
    while(cgraph != NULL)
    {
        concept = (ConceptNode*) cgraph->GetNode();
        cname = concept->GetName();
        if(cname == "SCOPE" || cname == "SOURCE_NAME" ||
           cname == "SOURCE_FORM")
    	{
            instance_list = concept->GetInstances();
            i = instance_list.begin();
            if(cname == "SCOPE")
                scope_1 = *i;
            if(cname == "SOURCE_NAME")
                source_name_1 = *i;
            if(cname == "SOURCE_FORM")
                source_form_1 = *i;
    	}
        cgraph = cgraph->GetDepth();
    }
    rgraph = GetRelation(target, rname);
    assert(rgraph != NULL);
    cgraph = rgraph->GetWidth();
    while(cgraph != NULL)
    {
        concept = (ConceptNode*) cgraph->GetNode();
        cname = concept->GetName();
        if(cname == "SCOPE" || cname == "SOURCE_NAME" ||
           cname == "SOURCE_FORM")
        {
            instance_list = concept->GetInstances();
            i = instance_list.begin();
            if(cname == "SCOPE")
                scope_2 = *i;
            if(cname == "SOURCE_NAME")
                source_name_2 = *i;
            if(cname == "SOURCE_FORM")
                source_form_2 = *i;
    	}
        cgraph = cgraph->GetDepth();
    }
    if(scope_1 == scope_2 && source_name_1 == source_name_2 &&
       source_form_1 == source_form_2)
        result = 1;
    return(result);
}


ConceptualGraph* CGTemplate::GetAppendPosition( ConceptualGraph *target,
                                                ConceptualGraph *source,
                                                IDLString rname,
                                                IDLString instance )
{
    ConceptualGraph *cgraph;
    ConceptualGraph *rgraph;
    ConceptualGraph *result;
    IDLString cname;
    IDLString name;
    
    assert(target->GetNode()->IsA() == CONCEPT_NODE); 
    if(rname == "TYPES")
        cname = "TYPE";
    if(rname == "CONSTANTS")
        cname = "CONSTANT";
    if(rname == "OPERATIONS")
        cname = "OPERATION";
    name = "ANNOTATIONS";
    rgraph = GetRelation(target, name);
    assert(rgraph != NULL);
    cgraph = GetConcept(rgraph, cname, instance);
    if(cgraph != NULL)
    {
        if(CompareAnnotations(cgraph, source))
            result = NULL;
        else
            result = GetAppendPosition(cgraph, source, rname, name);
    } 
    else 
    { 
        if(CompareAnnotations(target, source))
            result = NULL;
        else
        {
            cgraph = rgraph->GetWidth();
            assert(cgraph != NULL);
            while(cgraph != NULL)
    	    {
                if(cgraph->GetDepth() == NULL)
                    result = cgraph;
                cgraph = cgraph->GetDepth();
    	    }
        }
    }
    return(result);
}


ConceptualGraph* CGTemplate::CheckDefinition( ConceptualGraph *target, 
                                              ConceptualGraph *source,
                                              IDLString instance,
                                              IDLString rname )
{
    ConceptualGraph *cgraph;
    ConceptualGraph *result;

    assert(target->GetNode()->IsA() == CONCEPT_NODE);
    assert(source->GetNode()->IsA() == CONCEPT_NODE);
    cgraph = GetMatchingConcept(target, rname, instance);
    if(cgraph != NULL)
        result = GetAppendPosition(cgraph, source, rname, instance);
    else
    {
        result = GetInsertPosition(target, rname);
        if(result == NULL)
            result = InsertNewRelation(target, rname);
    }
    return(result);
}


ConceptualGraph* CGTemplate::InsertNewRelation( ConceptualGraph *target, 
                                                IDLString rname )
{
    ConceptualGraph *graph_1;
    ConceptualGraph *graph_2;
    ConceptualGraph *tgraph;
    RelationNode *relation;

    assert(target->GetNode()->IsA() == CONCEPT_NODE);
    relation = new RelationNode(Word (rname, false));
    tgraph = new ConceptualGraph(relation, NULL, NULL);
    graph_1 = target->GetWidth();
    if(graph_1 != NULL)
    {
        while(graph_1 != NULL)
    	{
            if(graph_1->GetDepth() == NULL)
                graph_2 = graph_1;
            graph_1 = graph_1->GetDepth();
     	}
        graph_2->SetDepth(tgraph);
    }
    else
        target->SetWidth(tgraph);
    return(tgraph);
}


void CGTemplate::InsertNewConcept( ConceptualGraph *target,
                                   IDLString cname, int pos )
{
    ConceptualGraph *tgraph;
    ConceptualGraph *graph_1;
    ConceptualGraph *graph_2;
    ConceptNode *concept;
 
    concept = new ConceptNode(Word (cname, false), emptyWordSet);
    concept->SetValue(pos);
    tgraph = new ConceptualGraph(concept, NULL, NULL);
    if(target->GetNode()->IsA() == RELATION_NODE)
        graph_1 = target->GetWidth();
    else
        graph_1 = target;
    if(graph_1 != NULL)
    {
        while(graph_1 != NULL)
    	{
            if(graph_1->GetDepth() == NULL)
                graph_2 = graph_1;
            graph_1 = graph_1->GetDepth();
     	}
        graph_2->SetDepth(tgraph);
    }
    else
        target->SetWidth(tgraph);
}

