/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#ifndef __CGTEMPLATE_H__
#define __CGTEMPLATE_H__

#include <Trader.h>
#include <dict.h>

#include "idlstring.h"

#define MAXN 256

class CGTemplate 
{
public:
    CGTemplate();
    ~CGTemplate();
    ConceptualGraph* GetInstance( ConceptualGraph *cg, IDLString scope[],
                                  int nr, int maxnr );
    ConceptualGraph* GetType( ConceptualGraph *cg, IDLString scope[],
                              int field, int maxfields );
    ConceptualGraph* GetConcept( ConceptualGraph *cg, IDLString name,
                                 IDLString instance );
    ConceptualGraph* GetRelation( ConceptualGraph *cg, IDLString name );
    int CompareInstances( ConceptualGraph *target, IDLString name );
    ConceptualGraph* GetMatchingConcept( ConceptualGraph *target,
                                         IDLString rname, 
                                         IDLString instance );
    ConceptualGraph* GetInsertPosition( ConceptualGraph *target,
                                        IDLString rname );
    int CompareAnnotations( ConceptualGraph *target,
                            ConceptualGraph *source );
    ConceptualGraph* GetAppendPosition( ConceptualGraph *target, 
                                        ConceptualGraph *source,
                                        IDLString rname,
                                        IDLString instance );
    ConceptualGraph* CheckDefinition( ConceptualGraph *target, 
                                      ConceptualGraph *source,
                                      IDLString instance, 
                                      IDLString rname );
    ConceptualGraph* InsertNewRelation( ConceptualGraph *target,
                                        IDLString rname );
	void InsertNewConcept( ConceptualGraph *target, IDLString cname,
                               int pos );
	ConceptualGraph* CreateNewConceptNode( IDLString cname,
                                               IDLString iname );
	ConceptualGraph* CreateNewRelationNode( IDLString rname );
	int DetectConcept( ConceptualGraph *cg, IDLString name,
                           IDLString instance );
	ConceptualGraph* GetPureConcept(ConceptualGraph *cg, IDLString name );
};

	
#endif

