/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#ifndef __CGTRANSFORM_H__
#define __CGTRANSFORM_H__

#include <Trader.h>
#include <dict.h>

#include "cgtemplate.h"
#include "idlstring.h"
#include "printcg.h"

#define MAXN 256


class ConceptualGraph2Template : public CGTemplate 
{
private:
    Dict idict; /* input */
    Dict odict; /* output */
    Dict ddict; /* dependency */

    ConceptualGraph *root;

    IDLString EncodeIfaceName (IDLString name, int pos);
    IDLString DecodeIfaceName (IDLString name, int &pos);
    vector<string> SortIfaceNames (const Dict::vcwp &iflist);

    void LookUpModuleSpecification( ConceptualGraph *cg, IDLString scope, 
                                    IDLString name );
    void LookUpInterfaceSpecification( ConceptualGraph *cg, IDLString scope,
                                       IDLString name );
    void MaintainNameScopeDictionary( IDLString scope, IDLString module, 
                                      IDLString member );
    void MaintainInheritanceDictionary( IDLString scope, IDLString interface, 
                                        IDLString superinterface,
                                        int position);
    void MaintainDependencyDictionary( IDLString scope, IDLString interface, 
                                       IDLString dependant );
    void RangeMembershipStructure( IDLString base );
    void RangeInheritanceStructure( IDLString base );
    void ResolveHierarchy( IDLString sname );
    void CollectGlobalDefinitions( ConceptualGraph *target,
                                   ConceptualGraph *source,
                                   IDLString sname );
    void CollectInheritedDefinitions( ConceptualGraph *target, 
                                      ConceptualGraph *source );
    void InsertDefinition( ConceptualGraph *target, ConceptualGraph *source, 
                           IDLString rname );
    void CheckAndInsertDefinition( ConceptualGraph *target, 
                                   ConceptualGraph *source, IDLString rname );
    void RemoveInterface( ConceptualGraph *node, IDLString dname, 
                          IDLString sname ); 
    void RemoveModule( ConceptualGraph *node, IDLString dname, 
                       IDLString sname ); 
    void CollectScopedNames( ConceptualGraph *target, ConceptualGraph *cg );
    void InsertScopedType( ConceptualGraph *target, IDLString type );
    void DetectGlobalInterfaceTypeUsage( ConceptualGraph *cg,
                                         IDLString interface,
                                         IDLString sinterface, IDLString name,
                                         IDLString scope );
    void DefineInterfaceType( ConceptualGraph *cg, IDLString interface, 
                              IDLString name, IDLString scope );
    int DetectLocalInterfaceTypeUsage( ConceptualGraph *cg,
                                       IDLString interface,
                                       IDLString name );
    int SubstituteInterfaceTypeUsage( ConceptualGraph *cg, IDLString name );
    void ModifyAndExchangeInterfaceType( ConceptualGraph *cg,
                                         SetOfWord *inst, 
                                         IDLString name );
    void ExchangeInterfaceType( ConceptualGraph *cg, IDLString name );
    IDLString ActualizeScope( IDLString scope, IDLString name );
    IDLString GetScopedName( IDLString scope, IDLString name );
    IDLString GetScopedInterfaceName( ConceptualGraph *cg, IDLString scope[], 
                                      int nr, int maxnr, IDLString interface );
    void CreateDependencySubGraph( ConceptualGraph *cg, IDLString base );
    IDLString GetMemberForm(IDLString name);
public:
    ConceptualGraph2Template( ConceptualGraph *cg );
    ~ConceptualGraph2Template();
    void CreateLeanStructure();
    void CreateDependencyGraph();
    void Print( ostream &ostr );
};

	
#endif

