
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stream.h>
#include <fstream.h>
#include <strstream.h>

#include "idlparser.h"


class Application {
private:
    int       argc;
    char      **argv;
    void      LoadFile( char *Name, char *OName = NULL );
public:
    Application( int an_argc, char *an_argv[] );
	~Application();
    void Run();
};


Application::Application( int an_argc, char *an_argv[] )
{
	argc = an_argc;
	argv = an_argv;
}


Application::~Application()
{
}


void Application::LoadFile( char *Name, char *OName )
{
	ifstream ifile( Name );
	ostream *ofile = &cout;
	
	if( !ifile ) {
		cerr << form ("Cannot open input file '%s'\n", Name);
		exit (1);
	}
	if (OName) {
		ofile = new ofstream (OName);
		if (!ofile || !*ofile) {
			cerr << form ("Cannot open output file '%s'\n", OName);
			exit (1);
		}
	}
	IDLParser *idlparser = new IDLParser(ifile, Name, *ofile);
	idlparser->Parse();
	cerr << idlparser->GetErrorFile() << "   " << idlparser->GetErrorMsg()
	     << "   " << idlparser->GetErrorLine() << endl;
	delete idlparser;

	if (ofile != &cout)
		delete (ofile);
}

static void
usage (char *appname)
{
	cerr << form ("Usage: %s [-o <output-file>] <input-file>\n", appname);
	exit( -1 );
}


void Application::Run()
{
	char *oname = NULL;
	int i = 1;

	if( argc < 2 ) {
		usage (argv[0]);
	}
	if (!strcmp ("-o", argv[i])) {
		if (argc < 4)
			usage (argv[0]);
		oname = argv[i+1];
		i += 2;
	}
	LoadFile( argv[i], oname );
}


int main( int argc, char *argv[] )
{
	Application app( argc, argv );

	app.Run();
}

