
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


/************************************************************************/
/* File: idlconverter.cc		Author:	Carsten Kaiser		*/
/* Date: 20.03.1996		       	Version:	3.0		*/
/*						   			*/
/* In diesem File werden die Regeln fuer die Generierung des		*/
/* kanonischen Konzeptgraphen-Templates aus einer Schnittstellen-	*/
/* Spezifikation in CORBA bzw. DCE festgelegt.				*/
/************************************************************************/  


#include "idlconverter.h"
#include "util.h"


/*----------- Methoden der Klasse IDL2ConceptualGraph ------------------*/

IDL2ConceptualGraph::~IDL2ConceptualGraph()
{
    delete cgout;
}


IDL2ConceptualGraph::IDL2ConceptualGraph( ostream &o ) 
{
    cgout = new CG2Stream(o);
}


IDLString IDL2ConceptualGraph::ActualizeScope( IDLString scope,
                                               IDLString root )
{
    IDLString base;

    if(!(root == "*BASE*"))
        base = root + "::";
    else
        base = "";
    if(!(scope == ""))
        prepend(base, scope);
    return(base);
}
 

void IDL2ConceptualGraph::InsertStdExceptParameter( int pos )
{
    IDLString attribute;

    cgout->PrintConcept("OPERATION_PARAMETER", "except");
    cgout->EnterNewLevel(parameter_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    cgout->PrintConcept("POINTER", "ptr");
    cgout->LeaveLevel(annotation_level_list);
    cgout->PrintRelation("CONSTRUCTED_TYPE");
    cgout->PrintConcept("POINTER", 2);
    cgout->EnterNewLevel(declarator_level); 
    cgout->PrintRelation("TEMPLATE_TYPE");
    cgout->EnterNewLevel(template_type_level);
    cgout->PrintPureConcept("SEQUENCE");
    cgout->PrintConcept("ELEMENT_TYPE", "std_Except");
    cgout->LeaveLevel(template_type_level);
    cgout->LeaveLevel(declarator_level);
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("PARAMETER_POSITION", pos);
    cgout->PrintRelation("ATTRIBUTES");
    cgout->PrintConcept("PARAMETER_ATTRIBUTES");
    attribute = "out";
    cgout->PrintInstance(attribute);
    attribute = "ptr";
    cgout->PrintLastInstance(attribute);
    cgout->LeaveLevel(parameter_level);
}


void IDL2ConceptualGraph::CreateStdExceptType()
{
    cgout->PrintConcept("TYPE", "completion_status");
    cgout->EnterNewLevel(type_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    cgout->PrintConcept("SOURCE_NAME", "*BASE*");
    cgout->PrintConcept("SOURCE_FORM", "SPECIFICATION");
    cgout->PrintConcept("EXCEPTION", "std");
    cgout->LeaveLevel(annotation_level_list);
    cgout->PrintRelation("CONSTRUCTED_TYPE");
    cgout->PrintPureConcept("ENUM");
    cgout->EnterNewLevel(constructed_type_level);
    cgout->PrintRelation("MEMBERS");
    cgout->EnterNewLevel(member_level_list);
    cgout->PrintConcept("ENUM_MEMBER", "YES");
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("MEMBER_POSITION", 1);
    cgout->PrintConcept("ENUM_MEMBER", "NO");
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("MEMBER_POSITION", 2);
    cgout->PrintConcept("ENUM_MEMBER", "MAYBE");
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("MEMBER_POSITION", 3);
    cgout->LeaveLevel(type_level); 
    cgout->PrintConcept("TYPE", "std_Except");
    cgout->EnterNewLevel(type_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    cgout->PrintConcept("SOURCE_NAME", "*BASE*");
    cgout->PrintConcept("SOURCE_FORM", "SPECIFICATION");
    cgout->PrintConcept("EXCEPTION", "std");
    cgout->LeaveLevel(annotation_level_list);
    cgout->PrintRelation("CONSTRUCTED_TYPE");
    cgout->PrintConcept("STRUCT", "std_Except");
    cgout->EnterNewLevel(constructed_type_level);
    cgout->PrintRelation("MEMBERS");
    cgout->EnterNewLevel(member_level_list);
    cgout->PrintConcept("STRUCT_MEMBER", "ident");
    cgout->EnterNewLevel(member_level);
    cgout->PrintRelation("TEMPLATE_TYPE");
    cgout->EnterNewLevel(template_type_level);
    cgout->PrintPureConcept("STRING");
    cgout->LeaveLevel(template_type_level);
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("MEMBER_POSITION", 1);
    cgout->LeaveLevel(member_level);
    cgout->PrintConcept("STRUCT_MEMBER", "minor");
    cgout->EnterNewLevel(member_level);
    cgout->PrintRelation("TYPE");
    cgout->PrintConcept("MEMBER_TYPE", "unsigned long");
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("MEMBER_POSITION", 2);
    cgout->LeaveLevel(member_level);
    cgout->PrintConcept("STRUCT_MEMBER", "completed");
    cgout->EnterNewLevel(member_level);
    cgout->PrintRelation("TYPE");
    cgout->PrintConcept("MEMBER_TYPE", "completion_status");
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("MEMBER_POSITION", 3);
    cgout->LeaveLevel(type_level);
}

/*-------------------------------------------------------------------*/



/*-------- Methoden der Klasse CORBA2ConceptualGraph ----------------*/

CORBA2ConceptualGraph::~CORBA2ConceptualGraph()
{
}


void CORBA2ConceptualGraph::CreateConceptualGraph( IDLParseNode *node )
{
    IDLString scope;
    IDLString root;
    IDLString form;

    scope = "";
    root = "*BASE*";
    form = "SPECIFICATION";
    cgout->PrintRoot(); 
    CreateStdExceptType();
    CreateInterfaceGraph(node->GetBranch(1), scope, root, form);
}


void CORBA2ConceptualGraph::CreateInterfaceGraph( IDLParseNode *node,
                                                  IDLString scope, 
                                                  IDLString root,
                                                  IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNodeType ntype;
    IDLParseNodeType type;
    IDLString name;
    IDLString tname;
    IDLString tscope;
    IDLString tform;

    type = node->GetType(); 
    switch(type) 
    {
    case module:
	name = *node->GetString();
	cgout->PrintConcept("MODULE", name);
	tscope = ActualizeScope(scope, name);
	tform = "MODULE";
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();
	cgout->EnterNewLevel(module_level);
	cgout->PrintRelation("ANNOTATIONS");
 	cgout->EnterNewLevel(annotation_level_list);
	if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
	cgout->PrintConcept("SOURCE_NAME", root);
	cgout->PrintConcept("SOURCE_FORM", form);
	cgout->LeaveLevel(annotation_level_list);
	cgout->PrintRelation("SCOPE");
	if(ntype == definition_list) 
        {
	    cgout->EnterNewLevel(definition_level_list);
	    CreateInterfaceGraph(hnode_1, tscope, name, tform);
	    cgout->LeaveLevel(definition_level_list);
        }
	else
            CreateInterfaceGraph(hnode_1, tscope, name, tform);
	cgout->LeaveLevel(module_level);
	break;
    case definition_list:
	CreateInterfaceGraph(node->GetBranch(1), scope, root, form);
	hnode_1 = node->GetBranch(2);
	ntype = hnode_1->GetType();
	while(ntype == definition_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    CreateInterfaceGraph(hnode_2, scope, root, form);
	    hnode_1 = hnode_1->GetBranch(2);
	    ntype = hnode_1->GetType();
        }
	CreateInterfaceGraph(hnode_1, scope, root, form);
	break;
    case forward_dcl:
#if 0
	name = *node->GetString();
	cgout->PrintConcept("FORWARD_DCL", name);
	cgout->PrintRelation("ANNOTATIONS");
 	cgout->EnterNewLevel(annotation_level_list);
	if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
	cgout->PrintConcept("SOURCE_NAME", root);
	cgout->PrintConcept("SOURCE_FORM", form);
	cgout->LeaveLevel(annotation_level_list);
#endif
	break;
     case interface_dcl:
	hnode_1 = node->GetBranch(1);
	name = *hnode_1->GetString();
	cgout->PrintConcept("INTERFACE", name);
	tscope = ActualizeScope(scope, name);
	tform = "INTERFACE";
	cgout->EnterNewLevel(interface_level);
 	cgout->PrintRelation("ANNOTATIONS");
	cgout->EnterNewLevel(annotation_level_list);
 	tname = "CORBA";
 	cgout->PrintConcept("IDL", tname);
	if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
	cgout->PrintConcept("SOURCE_NAME", root);
	cgout->PrintConcept("SOURCE_FORM", form);
	cgout->LeaveLevel(annotation_level_list);
	hnode_2 = hnode_1->GetBranch(1);
	if(hnode_2) 
        {
	    cgout->PrintRelation("SUPERINTERFACES");
  	    cgout->EnterNewLevel(interface_level_list);
	    while(hnode_2)
            {
		ntype = hnode_2->GetType();
            	if(ntype == scoped_name_list)
                    hnode_3 = hnode_2->GetBranch(1);
		else
                    hnode_3 = hnode_2;
		cgout->PrintConcept("INTERFACE");
		CreateTypes(hnode_3, tscope, name, tform);
		hnode_2 = hnode_2->GetBranch(2);
            }
	    cgout->LeaveLevel(interface_level_list);
        }
	CollectConstants(node, tscope, name, tform);
	CollectTypes(node, tscope, name, tform);
	CollectOperations(node, tscope, name, tform);
	cgout->LeaveLevel(interface_level);
	break;
    case const_dcl:
	CreateConstant(node, scope, root, form);
	break;
    case except_dcl:
        ConvertException(node, scope, root, form);
	break;
    case op_dcl:
	ConvertRaisesExpression(node, scope, root, form);
	break;
    case type_dcl:
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();
	if(ntype == type_declarator)
            CreateTypeSpecification(hnode_1, scope, root, form);
	else 
        {
	    cgout->PrintPureConcept("TYPE");
	    cgout->EnterNewLevel(type_level);   
 	    cgout->PrintRelation("ANNOTATIONS");
	    cgout->EnterNewLevel(annotation_level_list);
	    if(!(scope == ""))
                cgout->PrintConcept("SCOPE", scope);
	    cgout->PrintConcept("SOURCE_NAME", root);
	    cgout->PrintConcept("SOURCE_FORM", form);
	    name = *hnode_1->GetString();
	    cgout->PrintConcept("IDENTIFIER", name);
	    cgout->LeaveLevel(annotation_level_list);
	    CreateTypes(hnode_1, scope, root, form);
	    cgout->LeaveLevel(type_level);
        }
	break;
    }
}


void CORBA2ConceptualGraph::CollectConstants( IDLParseNode *node,
                                              IDLString scope, 
                                              IDLString root, IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;  	
    IDLParseNodeType ntype;
    int counter;

    counter = 0;
    hnode_1 = node->GetBranch(2);
    while(hnode_1) 
    {
        ntype = hnode_1->GetType();
        if(ntype == interface_body)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        ntype = hnode_2->GetType();
        if(ntype == const_dcl) 
    	{
            if(!counter) 
	    {
                cgout->PrintRelation("CONSTANTS");
                cgout->EnterNewLevel(constant_level_list);
	    }
            CreateConstant(hnode_2, scope, root, form);
            counter++;
	}
        hnode_1 = hnode_1->GetBranch(2);
    }
    if(counter)
        cgout->LeaveLevel(constant_level_list); 
}


void CORBA2ConceptualGraph::CreateConstant( IDLParseNode *node,
                                            IDLString scope, 
                                            IDLString root, IDLString form )
{
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLString name;
    IDLString expression;

    name = *node->GetString();
    cgout->PrintConcept("CONSTANT", name);
    cgout->EnterNewLevel(constant_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    cgout->LeaveLevel(annotation_level_list);
    hnode_1 = node->GetBranch(1);
    ntype = hnode_1->GetType();
    switch(ntype)
    {
    case template_type:
        CreateTypes(hnode_1, scope, root, form);
        break;
    case scoped_name:
    case base_type:
        cgout->PrintRelation("TYPE");
        cgout->PrintConcept("CONSTANT_TYPE");
        CreateTypes(hnode_1, scope, root, form);
        break;
    }
    cgout->PrintRelation("VALUE");
    expression = GetConstExpression(node->GetBranch(2));
    cgout->PrintConcept("CONSTANT_VALUE", expression);
    cgout->LeaveLevel(constant_level);
}


IDLString CORBA2ConceptualGraph::GetConstExpression( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNodeType type;
    IDLParseNodeType ntype;
    IDLString expression;
    IDLString part_1;
    IDLString part_2;

    type = node->GetType();
    switch(type)
    {
    case const_expr:
	expression = GetConstExpression(node->GetBranch(1)); 
	break;
    case or_exp:  	
    case xor_exp:  	
    case and_exp:  	
    case shift_exp:  	
    case add_exp:  	
    case mult_exp:
	part_1 = GetConstExpression(node->GetBranch(1));
	part_2 = GetConstExpression(node->GetBranch(2));
	expression = part_1 + *node->GetString() + part_2;
	break;  
    case unary_exp:  	
	part_1 = GetConstExpression(node->GetBranch(1));
	part_2 = GetConstExpression(node->GetBranch(2));
	expression = part_1 + part_2;
	break;
    case literal:
    case unary_operator:
	expression = *node->GetString();
	break;
    case primary_exp:
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();  	
	switch(ntype)
        {
        case literal:
        case scoped_name:
            expression = *hnode_1->GetString();
            break;
        case const_expr:
            expression = "(";
            part_1 = GetConstExpression(hnode_1);
            part_2 = ")";
            expression += part_1;
            expression += part_2;
            break;
        }
	break;
    }
    return(expression);
}


int CORBA2ConceptualGraph::ConvertAttributes( IDLParseNode *node,
                                              IDLString scope, 
                                              IDLString root, IDLString form, 
                                              int pos )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString tscope;
    int offset;

    offset = 0;
    hnode_1 = node->GetBranch(2);
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == declarator_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        if(hnode_2)
            ntype = hnode_2->GetType();
        if(ntype == declarator) /* array_declarator wird nicht betrachtet ! */ 
        {
            CreateGetOperation(node, hnode_2, scope, root, form, 
                               pos + offset);
            offset++;
            if(!node->GetString())
            { 
                CreateSetOperation(node, hnode_2, scope, root, form, 
                                   pos + offset);
                offset++;
            }
    	}
        hnode_1 = hnode_1->GetBranch(2);
    }
    return(pos + offset);
}


void CORBA2ConceptualGraph::CreateGetOperation( IDLParseNode *node,
                                                IDLParseNode *dnode, 
                                                IDLString scope,
                                                IDLString root, 
                                                IDLString form, int pos )
{
    IDLParseNode *tnode;
    IDLParseNodeType ntype;
    IDLString name;
    IDLString pname;
    IDLString tscope;

    name = "_get_";
    pname = GetDeclarator(dnode);
    name += pname;
    tnode = node->GetBranch(1);
    tscope = ActualizeScope(scope, name);
    cgout->PrintConcept("OPERATION", name);
    cgout->EnterNewLevel(operation_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    cgout->PrintConcept("ATTRIBUTE", pname);
    if(node->GetString()) 
    {
        cgout->PrintRelation("FEATURES");
        cgout->PrintPureConcept("READONLY");
    }
    cgout->LeaveLevel(annotation_level_list);
    if(tnode)
        ntype = tnode->GetType();
    switch(ntype)
    {
    case template_type:
        CreateTypes(tnode, tscope, name, form);
        break;
    case scoped_name:
    case base_type:
        cgout->PrintRelation("TYPE");
        cgout->PrintConcept("OPERATION_TYPE");
        CreateTypes(tnode, tscope, name, form);
        break;
    }
    cgout->PrintRelation("SIGNATURE");
    cgout->EnterNewLevel(parameter_level_list);
    InsertStdExceptParameter(1);
    cgout->LeaveLevel(parameter_level_list);
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("OPERATION_POSITION", pos);
    cgout->LeaveLevel(operation_level);
}


void CORBA2ConceptualGraph::CreateSetOperation( IDLParseNode *node,
                                                IDLParseNode *dnode,
                                                IDLString scope,
                                                IDLString root, 
                                                IDLString form, int pos )
{
    IDLParseNode *tnode;
    IDLParseNodeType ntype;
    IDLString name;
    IDLString pname;
    IDLString tscope;

    name = "_set_";
    pname = GetDeclarator(dnode);
    name += pname;
    tnode = node->GetBranch(1);
    tscope = ActualizeScope(scope, name);
    cgout->PrintConcept("OPERATION", name);
    cgout->EnterNewLevel(operation_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    cgout->PrintConcept("ATTRIBUTE", pname);
    cgout->LeaveLevel(annotation_level_list);
    cgout->PrintRelation("TYPE");
    cgout->PrintConcept("OPERATION_TYPE", "void");
    cgout->PrintRelation("SIGNATURE");
    cgout->EnterNewLevel(parameter_level_list);
    cgout->PrintConcept("OPERATION_PARAMETER");
    cgout->PrintLastInstance(pname);
    cgout->EnterNewLevel(parameter_level);
    ntype = tnode->GetType();
    switch(ntype)
    {
    case template_type:
        CreateTypes(tnode, tscope, name, form);
        break;
    case scoped_name:
    case base_type:
        cgout->PrintRelation("TYPE");
        cgout->PrintConcept("PARAMETER_TYPE");
        CreateTypes(tnode, tscope, name, form);
        break;
    }
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("PARAMETER_POSITION", 1);
    cgout->PrintRelation("ATTRIBUTES");
    cgout->PrintConcept("PARAMETER_ATTRIBUTES", "in");
    cgout->LeaveLevel(parameter_level);
    InsertStdExceptParameter(2);
    cgout->LeaveLevel(parameter_level_list);
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("OPERATION_POSITION", pos);
    cgout->LeaveLevel(operation_level);
}


void CORBA2ConceptualGraph::CollectTypes( IDLParseNode *node, IDLString scope, 
				       IDLString root, IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    int flag = 0;
    int counter = 0;

    hnode_1 = node->GetBranch(2);
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == interface_body)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        if(hnode_2)
            ntype = hnode_2->GetType();
        if(ntype == type_dcl || ntype == except_dcl || ntype == op_dcl) 
	{
            if(!counter) 
	    {
                switch(ntype)
		{
                case type_dcl:
                case except_dcl:
		    flag = 1;
                    break;
                case op_dcl:
		    flag = LookForRaisesExpression(hnode_2);
		    break;
		}
                if(flag)
		{
                    cgout->PrintRelation("TYPES");
                    cgout->EnterNewLevel(type_level_list);
                    counter++;
		}
	    }
            CreateInterfaceGraph(hnode_2, scope, root, form);
	}
        hnode_1 = hnode_1->GetBranch(2);
    } 
    if(counter)
        cgout->LeaveLevel(type_level_list);
}


void CORBA2ConceptualGraph::CreateTypes( IDLParseNode *node, IDLString scope, 
                                         IDLString root, IDLString form )
{
    IDLParseNodeType type;
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;  
    IDLString declarator;	
    IDLString name;
    IDLString tscope;    
    int counter;
    int pos;
    
    type = node->GetType();
    switch(type) 
    {
    case union_type:
	cgout->PrintRelation("CONSTRUCTED_TYPE");
	name = *node->GetString();
	cgout->PrintConcept("UNION", name);
        cgout->EnterNewLevel(constructed_type_level);
	tscope = ActualizeScope(scope, name);
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
            ntype = hnode_1->GetType();
	switch(ntype) 
        {
        case scoped_name:
        case base_type:
            cgout->PrintRelation("DISCRIMINATOR_TYPE");
            cgout->PrintConcept("UNION_DISCRIMINATOR_TYPE");
            /*
             * XXX fehlt hier ein break ???
             */
        case enum_type:
	       CreateTypes(hnode_1, tscope, name, form);
	       break;
        }
	cgout->PrintRelation("CASES");
	cgout->EnterNewLevel(case_level_list);
	hnode_1 = node->GetBranch(2);
	if(hnode_1)
            ntype = hnode_1->GetType();
	if(ntype == union_body) 
        {
	    while(ntype == union_body)
            {
		hnode_2 = hnode_1->GetBranch(1);
		CreateTypes(hnode_2, tscope, name, form);
		hnode_1 = hnode_1->GetBranch(2);
		ntype = hnode_1->GetType();
            }
	    CreateTypes(hnode_1, tscope, name, form);
        }
	else
            CreateTypes(hnode_1, tscope, name, form);
	cgout->LeaveLevel(constructed_type_level);
	break;
    case struct_type:
        cgout->PrintRelation("CONSTRUCTED_TYPE");
	name = *node->GetString();
	cgout->PrintConcept("STRUCT", name);
        cgout->EnterNewLevel(constructed_type_level);
	tscope = ActualizeScope(scope, name);
	cgout->PrintRelation("MEMBERS");
	pos = 1;
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();
	assert(ntype == member_list || ntype == member_declarator);
	cgout->EnterNewLevel(member_level_list);
	while(ntype == member_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    pos = CreateMemberSpecification(hnode_2, tscope, name, form, pos);
	    pos++;
	    hnode_1 = hnode_1->GetBranch(2);
	    ntype = hnode_1->GetType();
        }
	pos = CreateMemberSpecification(hnode_1, tscope, name, form, pos);
	cgout->LeaveLevel(constructed_type_level);
	break;	
    case enum_type:
	cgout->PrintRelation("CONSTRUCTED_TYPE");
	name = *node->GetString();
	cgout->PrintConcept("ENUM", name);
        cgout->EnterNewLevel(constructed_type_level);
	cgout->PrintRelation("MEMBERS");
	cgout->EnterNewLevel(member_level_list);
	pos = 1;
	hnode_1 = node->GetBranch(1);
	while(hnode_1)
        {
	    ntype = hnode_1->GetType();
	    if(ntype == declarator_list)
                hnode_2 = hnode_1->GetBranch(1);
	    else
                hnode_2 = hnode_1;
	    declarator = GetDeclarator(hnode_2);
	    cgout->PrintConcept("ENUM_MEMBER", declarator);
	    cgout->PrintRelation("POSITION");
	    cgout->PrintConcept("MEMBER_POSITION", pos);
	    pos++;
	    hnode_1 = hnode_1->GetBranch(2);
        }
	cgout->LeaveLevel(constructed_type_level);
	break;
    case case_type:
	counter = 0;
	cgout->PrintConcept("CASE_LABEL");
	hnode_1 = node->GetBranch(1);
	if(hnode_1->GetBranch(1))
            declarator = GetConstExpression(hnode_1->GetBranch(1));
	else
            declarator = *hnode_1->GetString(); 
	hnode_1 = node->GetBranch(2);
	if(hnode_1)
            ntype = hnode_1->GetType();
	if(ntype != element_declarator)
            cgout->PrintInstance(declarator);
	else
            cgout->PrintLastInstance(declarator);
	while(ntype != element_declarator)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    if(hnode_2->GetBranch(1))
                declarator = GetConstExpression(hnode_2->GetBranch(1));
	    else
                declarator = *hnode_2->GetString(); 
 	    hnode_1 = hnode_1->GetBranch(2);
 	    if(hnode_1)
                ntype = hnode_1->GetType();
	    if(ntype != element_declarator)
                cgout->PrintInstance(declarator);
	    else
                cgout->PrintLastInstance(declarator);
        }
	cgout->PrintRelation("MEMBERS");
	pos = 1;
	pos = CreateMemberSpecification(hnode_1, scope, root, form, pos); 
	break;
    case op_type:
	hnode_1 = node->GetBranch(1);
	if(hnode_1) 
        {
	    ntype = hnode_1->GetType();
	    switch(ntype)
            {
            case scoped_name:
            case base_type:  
                cgout->PrintRelation("TYPE");
                cgout->PrintConcept("OPERATION_TYPE");
            case template_type:
                CreateTypes(hnode_1, scope, root, form);
                break;
            }
        }
	else 
        {
	    cgout->PrintRelation("TYPE");
	    declarator = *node->GetString();
	    cgout->PrintConcept("OPERATION_TYPE", declarator);
        }
	break;
    case base_type:
	if(!(*node->GetString() == "octet"))
            declarator = *node->GetString();
	else
            declarator = "byte";
	cgout->PrintLastInstance(declarator);
	break;	
    case scoped_name:
	name = *node->GetString();
	cgout->PrintLastInstance(name);
	break;
    case template_type:
	cgout->PrintRelation("TEMPLATE_TYPE");
	cgout->EnterNewLevel(template_type_level);
	name = *node->GetString();
	if(name == "sequence")
            cgout->PrintPureConcept("SEQUENCE");
	else
            cgout->PrintPureConcept("STRING");
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
        {
	    ntype = hnode_1->GetType();
	    switch(ntype)
            {
            case base_type:
            case scoped_name:
                cgout->PrintConcept("ELEMENT_TYPE");
                CreateTypes(hnode_1, scope, root, form);
                break;
            case template_type:
                declarator = GetTemplateTypeDeclarator(hnode_1);
                cgout->PrintConcept("ELEMENT_TYPE", declarator);
                CreateTypes(hnode_1, scope, root, form);
                break;
            }
	    if(ntype == const_expr) 
            {
		declarator = GetConstExpression(hnode_1);
		cgout->PrintConcept("TEMPLATE_BOUND", declarator);
            }
	    else 
            {
	        hnode_2 = node->GetBranch(2);
	        if(hnode_2) 
                {
		    declarator = GetConstExpression(hnode_2);
		    cgout->PrintConcept("TEMPLATE_BOUND", declarator);
                }
            }
        }
	cgout->LeaveLevel(template_type_level);
	break;
    }
}

	
void CORBA2ConceptualGraph::CreateRecursiveType( IDLParseNode *node,
                                                 IDLString scope,
                                                 IDLString root,
                                                 IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNodeType ntype;
    IDLString name;

    hnode_1 = node->GetBranch(1);
    if(hnode_1) 
    {
        ntype = hnode_1->GetType();
        if(ntype == scoped_name)
	{
            name = *hnode_1->GetString();
            if(name == root)
            {
                cgout->PrintRelation("RECURSIVE_TYPE");
                cgout->PrintConcept("TYPE", name);
	    }
            else CreateTypes(node, scope, root, form);
	}
        else CreateTypes(node, scope, root, form);
    }
    else CreateTypes(node, scope, root, form);
}


void CORBA2ConceptualGraph::CollectOperations( IDLParseNode *node,
                                               IDLString scope, 
                                               IDLString root, IDLString form )
{
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    int counter;  
 
    counter = 1;
    hnode_1 = node->GetBranch(2);
    while(hnode_1) 
    {
        ntype = hnode_1->GetType();
        if(ntype == interface_body)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        if(hnode_2)
            ntype = hnode_2->GetType();
        if(ntype == op_dcl) 
	{
            if(counter == 1) 
	    {
                cgout->PrintRelation("OPERATIONS");
                cgout->EnterNewLevel(operation_level_list);
	    }	
            CreateOperation(hnode_2, scope, root, form, counter);
            counter++;
	}
        hnode_1 = hnode_1->GetBranch(2);
    } 
    hnode_1 = node->GetBranch(2);
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == interface_body)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        if(hnode_2)
            ntype = hnode_2->GetType();
        if(ntype == attr_dcl) 
   	{
            if(counter == 1) 
	    {
                cgout->PrintRelation("OPERATIONS");
                cgout->EnterNewLevel(operation_level_list);
	    }	
            counter = ConvertAttributes(hnode_2, scope, root, form, counter);
            counter++;
    	}
        hnode_1 = hnode_1->GetBranch(2);
    } 
    if(counter > 1)
        cgout->LeaveLevel(operation_level_list);
}


int CORBA2ConceptualGraph::LookForRaisesExpression( IDLParseNode *node )
{
    IDLParseNode *hnode;
    int flag = 0;
    
    hnode = node->GetBranch(4);
    if(hnode)
        flag = 1;
    return(flag);
}


int CORBA2ConceptualGraph::LookForContextExpression( IDLParseNode *node )
{
    IDLParseNode *hnode;
    int flag = 0;

    hnode = node->GetBranch(5);
    if(hnode)
        flag = 1;
    return(flag);
}


void CORBA2ConceptualGraph::ConvertContextExpression( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString literal;
    int counter = 1;

    hnode_1 = node->GetBranch(5);
    assert(hnode_1);
    cgout->PrintPureConcept("CONTEXT");
    cgout->PrintRelation("MEMBERS");
    cgout->EnterNewLevel(member_level_list);
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == literal_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        if(hnode_2) 
	{
            literal = *hnode_2->GetString();
            cgout->PrintConcept("CONTEXT_MEMBER", literal);
            cgout->EnterNewLevel(member_level);
            cgout->PrintRelation("POSITION");
            cgout->PrintConcept("MEMBER_POSITION", counter);
            cgout->LeaveLevel(member_level);
            counter++;
	}
        hnode_1 = hnode_1->GetBranch(2);
    }
    cgout->LeaveLevel(member_level_list);
}


void CORBA2ConceptualGraph::ConvertRaisesExpression( IDLParseNode *node,
                                                     IDLString scope, 
                                                     IDLString root,
                                                     IDLString form )
{
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;  
    IDLParseNode *hnode_3;  
    IDLString name;
    IDLString tname;
    IDLString disc;
    int counter;

    name = *node->GetString();
    tname = name;
    hnode_1 = node->GetBranch(4);
    if(hnode_1)
    {
        tname += "_Except"; 
        cgout->PrintConcept("TYPE", tname);
        cgout->EnterNewLevel(type_level);
        cgout->PrintRelation("ANNOTATIONS");
        cgout->EnterNewLevel(annotation_level_list);
        if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
        cgout->PrintConcept("SOURCE_NAME", root);
        cgout->PrintConcept("SOURCE_FORM", form);
        cgout->PrintConcept("RAISES_EXPRESSION", name);
        cgout->LeaveLevel(annotation_level_list);
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        cgout->PrintConcept("UNION", tname);
        cgout->EnterNewLevel(constructed_type_level);
        cgout->PrintRelation("DISCRIMINATOR_TYPE");
        cgout->PrintConcept("UNION_DISCRIMINATOR_TYPE", "long");
        cgout->PrintRelation("CASES");
        cgout->EnterNewLevel(case_level_list);
        disc = "";
        cgout->PrintConcept("CASE_LABEL", disc, 1);
        cgout->PrintRelation("MEMBERS");
        cgout->PrintConcept("UNION_MEMBER", "exc1");
        cgout->EnterNewLevel(member_level);
        cgout->PrintRelation("TYPE");
        cgout->PrintConcept("MEMBER_TYPE", "std_Except");
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept("MEMBER_POSITION", 1);
        cgout->LeaveLevel(member_level);
        counter = 2;
        while(hnode_1)
	{
            ntype = hnode_1->GetType();
            if(ntype == scoped_name_list)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            name = *hnode_2->GetString();
            name += "_Except";
            disc = "";
            cgout->PrintConcept("CASE_LABEL", disc, counter);
            cgout->PrintRelation("MEMBERS");
            disc = "exc";
            cgout->PrintConcept("UNION_MEMBER", disc, counter);
            cgout->EnterNewLevel(member_level);
            cgout->PrintRelation("TYPE");
            cgout->PrintConcept("MEMBER_TYPE", name);
            cgout->PrintRelation("POSITION");
            cgout->PrintConcept("MEMBER_POSITION", 1);
            cgout->LeaveLevel(member_level);
            counter++;
            hnode_1 = hnode_1->GetBranch(2);
        }
        cgout->LeaveLevel(type_level);
    }
}


void CORBA2ConceptualGraph::CreateOperation( IDLParseNode *node,
                                             IDLString scope, 
                                             IDLString root, IDLString form, 
                                             int position )
{
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;  
    IDLString tname;
    IDLString name;
    IDLString tscope;
    IDLString attribute;
    int pos;

    name = *node->GetString();
    cgout->PrintConcept("OPERATION", name);
    tscope = ActualizeScope(scope, name);
    cgout->EnterNewLevel(operation_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    if(LookForContextExpression(node))
        ConvertContextExpression(node); /* Zwischenloesung !!!! */
    cgout->LeaveLevel(annotation_level_list);
    CreateTypes(node->GetBranch(2), tscope, name, form);
    if(node->GetBranch(1)) 
    {
        cgout->PrintRelation("ATTRIBUTES");
        cgout->PrintConcept("OPERATION_ATTRIBUTES", "maybe");
    }
    cgout->PrintRelation("SIGNATURE");
    cgout->EnterNewLevel(parameter_level_list);
    pos = 1;
    hnode_1 = node->GetBranch(5);
    if(hnode_1) 
    {
        ntype = hnode_1->GetType();
	/*** CONTEXT_EXPRESSIONS integrieren ***/
    }
    hnode_1 = node->GetBranch(3);
    while(hnode_1) 
    {
        ntype = hnode_1->GetType();
        if(ntype == param_declarator_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        pos = CreateMemberSpecification(hnode_2, tscope, name, form, pos);
        pos++;
        hnode_1 = hnode_1->GetBranch(2);
    }
    hnode_1 = node->GetBranch(4);
    if(hnode_1) 
    {
        tname = name;
        tname += "_Except";
        cgout->PrintConcept("OPERATION_PARAMETER", "except");
        cgout->EnterNewLevel(parameter_level);
        cgout->PrintRelation("ANNOTATIONS");
        cgout->EnterNewLevel(annotation_level_list);
        cgout->PrintConcept("POINTER", "ptr");
        cgout->LeaveLevel(annotation_level_list);
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        cgout->PrintConcept("POINTER", 2);
        cgout->EnterNewLevel(declarator_level);
        cgout->PrintRelation("TEMPLATE_TYPE");
        cgout->EnterNewLevel(template_type_level);
        cgout->PrintPureConcept("SEQUENCE");
        cgout->PrintConcept("ELEMENT_TYPE", tname);
        cgout->LeaveLevel(template_type_level);
        cgout->LeaveLevel(declarator_level);
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept("PARAMETER_POSITION", pos);
        cgout->PrintRelation("ATTRIBUTES");
        cgout->PrintConcept("PARAMETER_ATTRIBUTES");
        attribute = "out";
        cgout->PrintInstance(attribute);
        attribute = "ptr";
        cgout->PrintLastInstance(attribute);
        cgout->LeaveLevel(parameter_level);
    }
    else
        InsertStdExceptParameter(pos);
    cgout->LeaveLevel(parameter_level_list);
    cgout->PrintRelation("POSITION");
    cgout->PrintConcept("OPERATION_POSITION", position);
    cgout->LeaveLevel(operation_level);
}


void CORBA2ConceptualGraph::ConvertException( IDLParseNode *node,
                                              IDLString scope, 
                                              IDLString root, IDLString form )
{
    IDLParseNodeType type;
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;  	
    IDLString name;
    IDLString tname;
    IDLString tscope;
    int pos;

    name = *node->GetString();
    tname = name;
    tname += "_Except"; 
    tscope = ActualizeScope(scope, name);
    cgout->PrintConcept("TYPE", tname);
    cgout->EnterNewLevel(type_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    cgout->PrintConcept("EXCEPTION", name);
    cgout->LeaveLevel(annotation_level_list);
    hnode_1 = node->GetBranch(1);
    if(hnode_1) 
    {
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        cgout->PrintConcept("STRUCT", name);
        cgout->EnterNewLevel(constructed_type_level);
        cgout->PrintRelation("MEMBERS");
        cgout->EnterNewLevel(member_level_list);	
        pos = 1;
        while(hnode_1)
	{
            ntype = hnode_1->GetType();
            if(ntype == member_list)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            pos = CreateMemberSpecification(hnode_2, tscope, name, form, pos);
            pos++;
            hnode_1 = hnode_1->GetBranch(2);
   	}
        cgout->LeaveLevel(constructed_type_level);
    }
    else
    {
        cgout->PrintRelation("TYPE");
        cgout->PrintConcept("TYPE_TYPE", "long");
    }
    cgout->LeaveLevel(type_level);
}


IDLString
CORBA2ConceptualGraph::GetTemplateTypeDeclarator( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString decl; 

    decl = *node->GetString();
    hnode_1 = node->GetBranch(1);
    if(hnode_1) 
    {
        decl += "<";
        ntype = hnode_1->GetType();
        switch(ntype) 
        {
        case const_expr:
	    decl += GetConstExpression(hnode_1);
	    break;
        case base_type:
        case scoped_name:
	    decl += *hnode_1->GetString();
	    hnode_2 = node->GetBranch(2);
	    if(hnode_2) 
            {
		decl += ",";
		decl += GetConstExpression(hnode_2);
            }
	    break;
        case template_type:
	    decl += GetTemplateTypeDeclarator(hnode_1);
   	    break;
	}
        decl += ">";
    }
    return(decl);
}


IDLString CORBA2ConceptualGraph::GetDeclarator( IDLParseNode *node )
{
    IDLParseNodeType type;
    IDLString name;

    type = node->GetType();
    if(type == declarator)
        name = *node->GetString();
    else
        name = GetArrayDeclarator(node);
    return(name);
}

 
IDLString CORBA2ConceptualGraph::GetArrayDeclarator( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString declarator;

    declarator = *node->GetString();
    hnode_1 = node->GetBranch(1);
    if(hnode_1)
        ntype = hnode_1->GetType();
    while(ntype == array_bound_list) 
    {
        hnode_2 = hnode_1->GetBranch(1);
        declarator += GetArrayBound(hnode_2);
        hnode_1 = hnode_1->GetBranch(2);
        ntype = hnode_1->GetType();
    }
    declarator += GetArrayBound(hnode_1);
    return(declarator);
}


IDLString CORBA2ConceptualGraph::GetArrayBound( IDLParseNode *node )
{
    IDLParseNode *hnode;
    IDLString bound;
 
    hnode = node->GetBranch(1);
    bound = "[";
    bound += GetConstExpression(hnode);
    bound += "]";
    return(bound);
}


void CORBA2ConceptualGraph::CreateArraySubGraph( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString bound;
    int counter;

    counter = 1;
    cgout->PrintRelation("DIMENSIONS");
    cgout->EnterNewLevel(bound_level_list);
    hnode_1 = node->GetBranch(1);
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == array_bound_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        bound = GetConstExpression(hnode_2->GetBranch(1));
        cgout->PrintConcept("DIMENSION", bound);
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept("DIMENSION_POSITION", counter);
        counter++;  
        hnode_1 = hnode_1->GetBranch(2);
    }
    cgout->LeaveLevel(bound_level_list);
}


int CORBA2ConceptualGraph::CountArrayDimensions( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    int dim = 0;

    hnode_1 = node;
    while(hnode_1)
    {
        dim++;
        hnode_1 = hnode_1->GetBranch(2);
    }
    assert(dim); 
    return(dim);
}


void CORBA2ConceptualGraph::CreateTypeSpecification( IDLParseNode *node,
                                                     IDLString scope, 
                                                     IDLString root,
                                                     IDLString form ) 
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNodeType ntype;
    IDLParseNodeType dtype;
    IDLString tscope;
    IDLString declarator;
    int dim;

    hnode_1 = node->GetBranch(2);
    hnode_3 = node->GetBranch(1);
    assert(hnode_3);
    dtype = hnode_3->GetType();
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == declarator_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        declarator = *hnode_2->GetString();
        cgout->PrintConcept("TYPE", declarator);
        cgout->EnterNewLevel(type_level);
        cgout->PrintRelation("ANNOTATIONS");
        cgout->EnterNewLevel(annotation_level_list);
        if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
        cgout->PrintConcept("SOURCE_NAME", root);
        cgout->PrintConcept("SOURCE_FORM", form);
        ntype = hnode_2->GetType();
        if(ntype == array_declarator) 
	{
            declarator = GetDeclarator(hnode_2);
            cgout->PrintConcept("FIXED_ARRAY", declarator);
            cgout->LeaveLevel(annotation_level_list);
            dim = CountArrayDimensions(hnode_2->GetBranch(1));
            cgout->PrintRelation("CONSTRUCTED_TYPE");
            cgout->PrintConcept("ARRAY", dim);
            cgout->EnterNewLevel(declarator_level);
            CreateArraySubGraph(hnode_2);
	}
        else
            cgout->LeaveLevel(annotation_level_list);
        switch(dtype) 
	{
        case template_type:
	    CreateRecursiveType(hnode_3, scope, root, form);
	    break;
        case scoped_name:
        case base_type:
	    cgout->PrintRelation("TYPE");
	    if(ntype == array_declarator)
                cgout->PrintConcept("ARRAY_TYPE");
	    else
                cgout->PrintConcept("TYPE_TYPE");
            /* XXX fehlt hier nicht ein break ?? */
        case union_type:
        case struct_type:
        case enum_type:
	    CreateTypes(hnode_3, scope, root, form);
	    break;
	}
#if 0
        if(ntype == array_declarator)
            cgout->LeaveLevel(declarator_level);
#endif
        cgout->LeaveLevel(type_level);
        hnode_1 = hnode_1->GetBranch(2);
    }
} 


int CORBA2ConceptualGraph::CreateMemberSpecification( IDLParseNode *node,
                                                      IDLString scope, 
                                                      IDLString root,
                                                      IDLString form, int pos )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNodeType ntype;
    IDLParseNodeType type;
    IDLParseNodeType dtype;
    CGLevelType level;
    IDLString declarator;
    char *concept;
    char *ext;
    IDLString tmp;
    int offset = -1;
    int dim;
 
    type = node->GetType();
    switch(type) 
    {
    case member_declarator:
	concept = "STRUCT_MEMBER";
	ext = "MEMBER";
	level = member_level;
	break;
    case element_declarator:
	concept = "UNION_MEMBER";
	ext = "MEMBER";
	level = member_level;
	break;
    case param_declarator:
	concept = "OPERATION_PARAMETER";
	ext = "PARAMETER";
	level = parameter_level;
	break;
    }
    hnode_3 = node->GetBranch(1);
    if(hnode_3)
        dtype = hnode_3->GetType();
    hnode_1 = node->GetBranch(2);
    while(hnode_1)
    {
        offset++;
        ntype = hnode_1->GetType();
        if(ntype == declarator_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        declarator = *hnode_2->GetString();
        cgout->PrintConcept(concept, declarator);
        cgout->EnterNewLevel(level);
        ntype = hnode_2->GetType();
        if(ntype == array_declarator) 
	{
            cgout->PrintRelation("ANNOTATIONS");
            cgout->EnterNewLevel(annotation_level_list);
            declarator = GetDeclarator(hnode_2);
            cgout->PrintConcept("FIXED_ARRAY", declarator);
            cgout->LeaveLevel(annotation_level_list);
            dim = CountArrayDimensions(hnode_2->GetBranch(1));
            cgout->PrintRelation("CONSTRUCTED_TYPE");
            cgout->PrintConcept("ARRAY", dim);
            cgout->EnterNewLevel(declarator_level);
            CreateArraySubGraph(hnode_2);
	}
        switch(dtype) 
	{
        case template_type:
	    CreateRecursiveType(hnode_3, scope, root, form);
	    break;
        case scoped_name:
        case base_type:
	    cgout->PrintRelation("TYPE");
	    tmp = ext;
	    tmp += "_TYPE";
	    if(ntype == array_declarator)
                cgout->PrintConcept("ARRAY_TYPE");
	    else
                cgout->PrintConcept(tmp);
            /* XXX fehlt hier nicht ein break ?? */
        case union_type:
        case struct_type:
        case enum_type:
	    CreateTypes(hnode_3, scope, root, form);
	    break;
	}
        if(ntype == array_declarator)
            cgout->LeaveLevel(declarator_level);
        cgout->PrintRelation("POSITION");
        tmp = ext;
        tmp += "_POSITION";
        cgout->PrintConcept(tmp, pos + offset);
        if(node->GetString())
	{
            cgout->PrintRelation("ATTRIBUTES");
            tmp = ext;
            tmp += "_ATTRIBUTES";
            cgout->PrintConcept(tmp, *node->GetString());
        }
        cgout->LeaveLevel(level);
        hnode_1 = hnode_1->GetBranch(2);
    }
    return(pos + offset);
}

/*------------------------------------------------------------------------*/


/*--------- Methoden der Klasse DCE2ConceptualGraph ----------------------*/

DCE2ConceptualGraph::~DCE2ConceptualGraph()
{
}


void DCE2ConceptualGraph::CreateConceptualGraph( IDLParseNode *node )
{
    IDLParseNode *hnode;
    IDLString scope;
    IDLString root;
    IDLString form;
 
    scope = "";
    root = "*BASE*";
    form = "SPECIFICATION";
    cgout->PrintRoot();
    hnode = node->GetBranch(1);
    if(hnode)
        CollectInterfaceGraph(hnode, scope, root, form);
}
 

IDLString DCE2ConceptualGraph::CreateInterfaceHeader( IDLParseNode *node,
                                                      IDLString scope, 
                                                      IDLString root,
                                                      IDLString form )
{
    IDLString name;
    IDLString sname;

    name = *node->GetString();
    cgout->PrintConcept("INTERFACE", name);
    cgout->EnterNewLevel(interface_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    sname = "DCE";
    cgout->PrintConcept("IDL", sname);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    cgout->LeaveLevel(annotation_level_list);
    return(name); 
}


void DCE2ConceptualGraph::CollectInterfaceGraph( IDLParseNode *node,
                                                 IDLString scope, 
                                                 IDLString root,
                                                 IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType type;
    IDLString name;
    IDLString tform;
    IDLString tscope;
    int flag;

    type = node->GetType();
    switch(type)  
    {
    case interface_list:
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
        {
	    name = CreateInterfaceHeader(hnode_1->GetBranch(1), scope, root, form);	
	    tscope = ActualizeScope(scope, name);
	    tform = "INTERFACE";	
 	    CollectInterfaceAttributes(hnode_1->GetBranch(1), tscope, name, tform);
	    hnode_2 = node->GetBranch(2);
	    flag = CollectConstants(hnode_1, hnode_2, tscope, name, tform, 0);
	    if(flag)
                cgout->LeaveLevel(constant_level_list);
	    flag = CollectTypes(hnode_1, hnode_2, tscope, name, tform, 0);
	    if(flag)
                cgout->LeaveLevel(type_level_list);
	    CreateOperations(hnode_1, tscope, name, tform, 0);
	    cgout->LeaveLevel(interface_level);
        }
	break;
    case interface_dcl:
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
        {
	    name = CreateInterfaceHeader(hnode_1, scope, root, form);	
	    tscope = ActualizeScope(scope, name);
	    tform = "INTERFACE";
	    CollectInterfaceAttributes(hnode_1, tscope, name, tform); 
	    CollectConstants(node, tscope, name, tform);
	    CollectTypes(node, tscope, name, tform);
	    CreateOperations(node, tscope, name, tform, 0);
	    cgout->LeaveLevel(interface_level);
        }
	break;
    }
}
 

void DCE2ConceptualGraph::CollectInterfaceAttributes( IDLParseNode *node,
                                                      IDLString scope,
                                                      IDLString root,
                                                      IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNodeType ntype;
    IDLString name;
    IDLString ptr_default;
	
    hnode_1 = node->GetBranch(1);
    if(hnode_1)
    {
        cgout->PrintRelation("ATTRIBUTES");
        cgout->EnterNewLevel(attribute_level_list);
        while(hnode_1)
	{
            ntype = hnode_1->GetType();
            if(ntype == attribute_list)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            if(hnode_2) 
	    {
                name = *hnode_2->GetString();
                if(!(name == "ptr_default"))
                    CreateInterfaceAttributes(hnode_2, scope, root, form);
                else 
		{
                    hnode_3 = hnode_2->GetBranch(1);
                    ptr_default = *hnode_3->GetString();
                    if(ptr_default == "ptr")
                        mode = 1;
                    else
                        mode = 2;
		}
	    }
            hnode_1 = hnode_1->GetBranch(2);
        }
        cgout->LeaveLevel(attribute_level_list);
    }
}


void DCE2ConceptualGraph::CreateInterfaceAttributes( IDLParseNode *node, 
                                                     IDLString scope,
                                                     IDLString root,
                                                     IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString name;
    IDLString attribute;
    char *concept;

    name = *node->GetString();
    hnode_1 = node->GetBranch(1);
    if(hnode_1) 
    {
        if(name == "uuid") 
        {
            concept = "INTERFACE_UUID";
            attribute = GetAttribute(hnode_1, scope, root, form);
            cgout->PrintConcept(concept, attribute);
        }
        if(name == "version") 
        {
            concept = "INTERFACE_VERSION";
            attribute = GetAttribute(hnode_1, scope, root, form);
            cgout->PrintConcept(concept, attribute);
        }
        if(name == "endpoint") 
        {
            concept = "INTERFACE_SERVER_ENDPOINTS";
            cgout->PrintConcept(concept);
            ntype = hnode_1->GetType();
            while(ntype == endpoint_list)
	    {
                hnode_2 = hnode_1->GetBranch(1);
                attribute = GetAttribute(hnode_2, scope, root, form);
                cgout->PrintInstance(attribute);
                hnode_1 = hnode_1->GetBranch(2);
                if(hnode_1)
                    ntype = hnode_1->GetType();
	    }
            attribute = GetAttribute(hnode_1, scope, root, form);
            cgout->PrintLastInstance(attribute);
        }
    }
    else 
    {
        concept = "INTERFACE_ATTRIBUTE";
        cgout->PrintConcept(concept, *node->GetString());
    }
}


void DCE2ConceptualGraph::CollectConstants( IDLParseNode *node,
                                            IDLString scope,
                                            IDLString root, IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    int flag = 0;

    hnode_1 = node->GetBranch(2);
    if(hnode_1)
    {
        while(hnode_1)
	{
            ntype = hnode_1->GetType();
            if(ntype == interface_body)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            if(hnode_2)
                ntype = hnode_2->GetType();
            if(ntype == const_dcl) 
	    {
                if(!flag)
		{
                    cgout->PrintRelation("CONSTANTS"); 
                    cgout->EnterNewLevel(constant_level_list);
		}	
                CreateConstants(hnode_2, scope, root, form);
                flag++;
	    }
            hnode_1 = hnode_1->GetBranch(2);
        }
        if(flag)
            cgout->LeaveLevel(constant_level_list);
    }
}


int DCE2ConceptualGraph::CollectConstants( IDLParseNode *node_A,
                                           IDLParseNode *node_B, 
                                           IDLString scope,
                                           IDLString root, IDLString form,
                                           int flag )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    int offset = 0;

    offset += flag;
    hnode_1 = node_A->GetBranch(2);
    if(hnode_1)
    {
        while(hnode_1)
        {
            ntype = hnode_1->GetType();
            if(ntype == interface_body)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            if(hnode_2)
                ntype = hnode_2->GetType();
            if(ntype == const_dcl) 
	    {
                if(!offset) 
		{
                    cgout->PrintRelation("CONSTANTS");	
                    cgout->EnterNewLevel(constant_level_list);
		}	
                CreateConstants(hnode_2, scope, root, form);
                offset++;
	    }
            hnode_1 = hnode_1->GetBranch(2);
        }
    }
    if(node_B)
    {
        ntype = node_B->GetType();
        if(ntype == importfile_list) 
        {
            hnode_1 = node_B->GetBranch(1);
            hnode_2 = node_B->GetBranch(2);
            offset = CollectConstants(hnode_1->GetBranch(1), hnode_2, 
                                      scope, root, form, offset);
        }
        else 
        {
            hnode_1 = node_B->GetBranch(1);
            hnode_2 = NULL;	
            offset = CollectConstants(hnode_1, hnode_2, scope, 
                                      root, form, offset);
        }
    }
    return(offset);
}


void DCE2ConceptualGraph::CreateConstants( IDLParseNode *node,
                                           IDLString scope, 
                                           IDLString root, IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLString name;
    IDLString expression;

    name = *node->GetString();	
    cgout->PrintConcept("CONSTANT", name);
    cgout->EnterNewLevel(constant_level);
    cgout->PrintRelation("ANNOTATIONS");
    cgout->EnterNewLevel(annotation_level_list);
    if(!(scope == ""))
        cgout->PrintConcept("SCOPE", scope);
    cgout->PrintConcept("SOURCE_NAME", root);
    cgout->PrintConcept("SOURCE_FORM", form);
    cgout->LeaveLevel(annotation_level_list);
    hnode_1 = node->GetBranch(1);
    if(!(*hnode_1->GetString() == "char*"))
    { 	
        cgout->PrintRelation("TYPE");
        cgout->PrintConcept("CONSTANT_TYPE");
        CreateTypes(hnode_1, scope, root, form);
    }
    else
    {
        cgout->PrintRelation("TEMPLATE_TYPE");
        cgout->EnterNewLevel(template_type_level);
        cgout->PrintPureConcept("STRING");
        cgout->LeaveLevel(template_type_level);
    } 
    cgout->PrintRelation("VALUE");
    hnode_2 = node->GetBranch(2);
    expression = GetConstExpression(hnode_2);
    cgout->PrintConcept("CONSTANT_VALUE", expression);
    cgout->LeaveLevel(constant_level);
}


IDLString DCE2ConceptualGraph::GetConstExpression( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNodeType type;
    IDLParseNodeType ntype;
    IDLString expression;
    IDLString part_1;
    IDLString part_2;

    type = node->GetType();
    switch(type)
    {
    case const_expr:
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
            expression = GetConstExpression(hnode_1); 
	else
            expression = *node->GetString();
	break;
    case conditional_exp:
        hnode_1 = node->GetBranch(1);
	part_1 = GetConstExpression(hnode_1);
	part_2 = *node->GetString();
        hnode_1 = node->GetBranch(2);
	expression = part_1 + part_2;
	part_1 = GetConstExpression(hnode_1);
	part_1 += " : ";
	expression += part_1;
	hnode_1 = node->GetBranch(3);
	part_2 = GetConstExpression(hnode_1);
	expression += part_2;
	break;  	
    case logical_or_exp:
    case logical_and_exp:
    case inclusive_or_exp:  	
    case exclusive_or_exp:  	
    case and_exp:
    case equality_exp:
    case relational_exp:  	
    case shift_exp:  	
    case add_exp:  	
    case mult_exp:
	part_1 = GetConstExpression(node->GetBranch(1));
	part_2 = GetConstExpression(node->GetBranch(2));
	expression = part_1 + *node->GetString() + part_2;
	break;  
    case unary_exp:  	
	part_1 = *node->GetString();
	part_2 = GetConstExpression(node->GetBranch(1));
	expression = part_1 + part_2;
	break;
    case array_bound:
    case literal:
    case primary_exp:
	expression = *node->GetString();
	break;
    }
    return(expression);
}


void DCE2ConceptualGraph::CollectTypes( IDLParseNode *node, IDLString scope, 
                                        IDLString root, IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    int flag = 2;

    cgout->PrintRelation("TYPES");
    cgout->EnterNewLevel(type_level_list);
    CreateStdExceptType();
    hnode_1 = node->GetBranch(2);
    if(hnode_1) 
    {
        while(hnode_1) 
        {
            ntype = hnode_1->GetType();
            if(ntype == interface_body)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            if(hnode_2) ntype = hnode_2->GetType();
            if(ntype == type_dcl) 
	    {
                if(!flag) 
		{
                    cgout->PrintRelation("TYPES");
                    cgout->EnterNewLevel(type_level_list);
		}
                CreateTypeSpecification(hnode_2, scope, root, form);
                flag++;
	    }
            hnode_1 = hnode_1->GetBranch(2);
        }
    }
    if(flag)
        cgout->LeaveLevel(type_level_list); 
}


int DCE2ConceptualGraph::CollectTypes( IDLParseNode *node_A,
                                       IDLParseNode *node_B, 
                                       IDLString scope, IDLString root, 
                                       IDLString form, int flag )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    int offset = 0;

    offset += flag;
    if(!offset)
    {
        cgout->PrintRelation("TYPES");
        cgout->EnterNewLevel(type_level_list);
        CreateStdExceptType();
        offset = 2;
    }
    hnode_1 = node_A->GetBranch(2);
    if(hnode_1) 
    {
        while(hnode_1) 
        {
            ntype = hnode_1->GetType();
            if(ntype == interface_body)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            if(hnode_2)
                ntype = hnode_2->GetType();
            if(ntype == type_dcl) 
	    {
                if(!offset) 
		{
                    cgout->PrintRelation("TYPES");
                    cgout->EnterNewLevel(type_level_list);
		}
                CreateTypeSpecification(hnode_2, scope, root, form);
                offset++;
	    }
            hnode_1 = hnode_1->GetBranch(2);
        }
    }
    if(node_B)
    {
        ntype = node_B->GetType();
        if(ntype == importfile_list) 
        {
            hnode_1 = node_B->GetBranch(1);
            hnode_2 = node_B->GetBranch(2);
            CollectTypes(hnode_1->GetBranch(1), hnode_2, scope, root,
                         form, offset); 
        }
        else 
        {
            hnode_1 = node_B->GetBranch(1);
            hnode_2 = NULL;	
            CollectTypes(hnode_1, hnode_2, scope, root, form, offset);
        }
    }
    return 0; /* XXX return was missing, assume 0 ... */
}


void DCE2ConceptualGraph::CreateRecursiveType( IDLParseNode *node )
{
    IDLString name;

    cgout->PrintRelation("RECURSIVE_TYPE");
    name = *node->GetString();
    cgout->PrintConcept("TYPE", name);
}

	
void DCE2ConceptualGraph::CreateTypes( IDLParseNode *node, IDLString scope, 
                                       IDLString root, IDLString form )
{
    IDLParseNodeType ntype;
    IDLParseNodeType type;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;  	
    IDLString troot;
    IDLString name;
    IDLString tscope;
    IDLString expression;
    int counter;
    int pos;
    
    type = node->GetType();
    switch(type) 
    {
    case union_type:
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        if(node->GetBranch(1)) 
        {
   	    hnode_1 = node->GetBranch(2);
	    hnode_1 = hnode_1->GetBranch(2);
	    hnode_2 = node->GetBranch(3);
	    if(hnode_2) 
            {
	        cgout->PrintConcept("UNION");
	        if(hnode_1) 
                {
		    name = *hnode_1->GetString();
		    cgout->PrintInstance(name);
		    tscope = ActualizeScope(scope, name);
		    troot = name;
		    name = *hnode_2->GetString();
		    cgout->PrintLastInstance(name);
                }
	        else 
                {
		    name = *hnode_1->GetString();
		    cgout->PrintLastInstance(name);
		    tscope = ActualizeScope(scope, name);
		    troot = name;
                }
            }
	    else 
            {
	        if(hnode_1) 
                {
		    name = *hnode_1->GetString();
		    cgout->PrintConcept("UNION", name);
		    tscope = ActualizeScope(scope, name);
		    troot = name;
                }
	        else 
                { 
        	    cgout->PrintPureConcept("UNION");
		    tscope = scope;
		    troot = root;
                }
            }
            cgout->EnterNewLevel(constructed_type_level);
	    CreateTypes(node->GetBranch(2), tscope, troot, form);
	    cgout->PrintRelation("CASES");
	    CreateTypes(node->GetBranch(1), tscope, troot, form);
	    cgout->LeaveLevel(constructed_type_level);
        }
        else 
        {
	    name = *node->GetString();
	    cgout->PrintConcept("UNION", name);
        }			
        break;
    case union_switch:
        name = *node->GetString();
        hnode_1 = node->GetBranch(1);		
        if(hnode_1)
            ntype = hnode_1->GetType();
        if(ntype == base_type || ntype == scoped_name) 
        {
            cgout->PrintRelation("DISCRIMINATOR_TYPE");
	    cgout->PrintConcept("UNION_DISCRIMINATOR_TYPE");
	    CreateTypes(hnode_1, scope, root, form);
	    cgout->PrintRelation("ANNOTATIONS");
	    cgout->EnterNewLevel(annotation_level_list);
	    cgout->PrintConcept("UNION_DISCRIMINATOR_NAME", name);
	    cgout->LeaveLevel(annotation_level_list);
        }
        else
        {
 	    CreateTypes(hnode_1, scope, root, form);
	    cgout->PrintRelation("ANNOTATIONS");
	    cgout->EnterNewLevel(annotation_level_list);
	    cgout->PrintConcept("UNION_DISCRIMINATOR_NAME", name);
	    cgout->LeaveLevel(annotation_level_list);
        }
        break;
    case union_body:
        cgout->EnterNewLevel(case_level_list);
        hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();
	while(ntype == case_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
 	    CreateTypes(hnode_2, scope, root, form);
  	    hnode_1 = hnode_1->GetBranch(2);
            ntype = hnode_1->GetType();		
        }
	CreateTypes(hnode_1, scope, root, form);
        hnode_1 = node->GetBranch(2);
        if(hnode_1)
            CreateTypes(hnode_1, scope, root, form);
	cgout->LeaveLevel(case_level_list);
        break;
    case case_type:
        cgout->PrintConcept("CASE_LABEL");
        hnode_1 = node->GetBranch(2);
        if(hnode_1) 
        {
	    ntype = hnode_1->GetType();
	    while(ntype == case_label_list) 
            {
		hnode_2 = hnode_1->GetBranch(1);
	    	expression = GetConstExpression(hnode_2->GetBranch(1));
	    	cgout->PrintInstance(expression);
	    	hnode_1 = hnode_1->GetBranch(2);
		if(hnode_1)
                    ntype = hnode_1->GetType();
            }
	    expression = GetConstExpression(hnode_1->GetBranch(1));
	    cgout->PrintLastInstance(expression);
        }
        else 
        {
	    expression = "default";
	    cgout->PrintLastInstance(expression);
        }
        pos = 1;
        hnode_1 = node->GetBranch(1);
        if(hnode_1) 
        {
            cgout->PrintRelation("MEMBERS");
	    pos = CreateMemberSpecification(hnode_1, scope, root, form,
                                            pos, "UNION");
        }
        break;
    case union_type_n_e:
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        hnode_1 = node->GetBranch(2);
        if(hnode_1) 
        {
	    name = *hnode_1->GetString();
            cgout->PrintConcept("UNION", name);
	    tscope = ActualizeScope(scope, name);
	    troot = name;
        }
        else 
        { 
            cgout->PrintConcept("UNION");
	    tscope = scope;
	    troot = root;
        }
        cgout->EnterNewLevel(constructed_type_level);
        cgout->PrintRelation("CASES");
	cgout->EnterNewLevel(case_level_list);
        CreateTypes(node->GetBranch(1), tscope, troot, form);
	cgout->LeaveLevel(constructed_type_level);
        break;
    case case_type_n_e:
        cgout->PrintConcept("CASE_LABEL");
        hnode_1 = node->GetBranch(2);
        if(hnode_1)
            CreateTypes(hnode_1, scope, root, form); 
        else
        {
	    expression = "default";
 	    cgout->PrintLastInstance(expression);
        }
        pos = 1;
        hnode_1 = node->GetBranch(1);
        if(hnode_1) 
        {
	    pos = CreateMemberSpecification(hnode_1, scope, root, form,
                                            pos, "UNION");
            cgout->PrintRelation("MEMBERS");
        }
        break;
    case case_label:
        hnode_1 = node->GetBranch(1);
        if(hnode_1)
            ntype = hnode_1->GetType();
        while(ntype == const_expr_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    expression = GetConstExpression(hnode_2);
	    cgout->PrintInstance(expression);
 	    hnode_1 = hnode_1->GetBranch(2);
 	    ntype = hnode_1->GetType();
        }
        expression = GetConstExpression(hnode_1);
        cgout->PrintLastInstance(expression);	
        break;
    case struct_type:
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        hnode_1 = node->GetBranch(1);
        if(hnode_1) 
        {
	    hnode_2 = node->GetBranch(2);
	    if(hnode_2) 
            {
	        name = *hnode_2->GetString();
	    	cgout->PrintConcept("STRUCT", name);
	        tscope = ActualizeScope(scope, name);
	        troot = name;
            }
	    else 
            { 
        	cgout->PrintPureConcept("STRUCT");
	        tscope = scope;
	        troot = root;
            }
 	    cgout->EnterNewLevel(constructed_type_level);
	    cgout->PrintRelation("MEMBERS");
	    cgout->EnterNewLevel(member_level_list);
	    pos = 1;
	    ntype = hnode_1->GetType();
	    while(ntype == member_list)
            {
	        hnode_2 = hnode_1->GetBranch(1);
	        pos = CreateMemberSpecification(hnode_2, tscope, troot,
                                                form, pos, "STRUCT");
	        pos++;
	        hnode_1 = hnode_1->GetBranch(2);
	        if(hnode_1)
                    ntype = hnode_1->GetType();
            }
	    pos = CreateMemberSpecification(hnode_1, tscope, troot, form,
                                            pos, "STRUCT");
	    cgout->LeaveLevel(constructed_type_level); 
        }
        else 
        {
	    name = *node->GetString();
	    cgout->PrintConcept("STRUCT", name);
        }			
        break;
    case enum_type:
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        cgout->PrintPureConcept("ENUM");
        cgout->EnterNewLevel(constructed_type_level);
        cgout->PrintRelation("MEMBERS");
        hnode_1 = node->GetBranch(1);
        pos = 1;
        if(hnode_1)
            ntype = hnode_1->GetType();
        cgout->EnterNewLevel(member_level_list);
        while(ntype == declarator_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    expression = GetDeclarator(hnode_2, scope, root, form);
	    cgout->PrintConcept("ENUM_MEMBER", expression);
	    cgout->PrintRelation("POSITION");
	    cgout->PrintConcept("MEMBER_POSITION", pos);
	    pos++;
	    hnode_1 = hnode_1->GetBranch(2);
	    if(hnode_1)
                ntype = hnode_1->GetType();
        }
	expression = GetDeclarator(hnode_1, scope, root, form);
        cgout->PrintConcept("ENUM_MEMBER", expression);
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept("MEMBER_POSITION", pos);
	cgout->LeaveLevel(constructed_type_level);
        break;
    case pipe_type:
        cgout->PrintRelation("CONSTRUCTED_TYPE");
        cgout->PrintConcept("PIPE");
        hnode_1 = node->GetBranch(2);
        if(hnode_1)
            ntype = hnode_1->GetType();
        while(ntype == declarator_list)
        {
  	    hnode_2 = hnode_1->GetBranch(1);
	    expression = GetDeclarator(hnode_2, scope, root, form);
	    cgout->PrintInstance(expression);
 	    hnode_1 = hnode_1->GetBranch(2);
 	    ntype = hnode_1->GetType();
        }
	expression = GetDeclarator(hnode_1, scope, root, form);
	cgout->PrintLastInstance(expression);
        cgout->EnterNewLevel(constructed_type_level);
        hnode_2 = node->GetBranch(1);
        if(hnode_2)
            ntype = hnode_2->GetType();
        if(ntype == base_type || ntype == scoped_name) 
        {
	    cgout->PrintRelation("TYPE");
	    cgout->PrintConcept("PIPE_TYPE");
        }
        CreateTypes(hnode_2, scope, root, form);
        cgout->LeaveLevel(constructed_type_level);
        break;
    case scoped_name:
    case base_type:
        name = *node->GetString(); 
        cgout->PrintLastInstance(name);
	break;	
    }
}


IDLString DCE2ConceptualGraph::GetArrayDeclarator( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString declarator;

    declarator = *node->GetString();
    hnode_1 = node->GetBranch(1);
    ntype = hnode_1->GetType();
    while(ntype == array_bound_list) 
    {
        hnode_2 = hnode_1->GetBranch(1);
        declarator += GetArrayBound(hnode_2);
        hnode_1 = hnode_1->GetBranch(2);
        ntype = hnode_1->GetType();
    }	
    declarator += GetArrayBound(hnode_1);
    return(declarator);
}


IDLString DCE2ConceptualGraph::GetArrayBound( IDLParseNode *node )
{
    IDLString bound;
 
    bound = "[";
    bound += GetConstExpression(node);
    bound += "]";
    return(bound);
}


void DCE2ConceptualGraph::CreateArraySubGraph( IDLParseNode *node, int status )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString bound;
    int counter;
    string tmp;

    cgout->PrintRelation("DIMENSIONS");
    cgout->EnterNewLevel(bound_level_list);
    hnode_1 = node->GetBranch(1);
    ntype = hnode_1->GetType();
    counter = 1;
    while(ntype == array_bound_list)
    {
        hnode_2 = hnode_1->GetBranch(1);
        bound = GetConstExpression(hnode_2);
        if(contains(bound, ".."))
	{
            tmp = after(bound, "..");
            bound = tmp;
	}
        if(status != counter) 
	{
            cgout->PrintConcept("DIMENSION", bound);
            cgout->PrintRelation("POSITION");
            cgout->PrintConcept("DIMENSION_POSITION", counter - status);
	}
        counter++;  
        hnode_1 = hnode_1->GetBranch(2);
        if(hnode_1)
            ntype = hnode_1->GetType();
    }
    bound = GetConstExpression(hnode_1);
    if(contains(bound, ".."))
    {
        tmp = after(bound, "..");
        bound = tmp;
    }
    if(status != counter) 
    {
        cgout->PrintConcept("DIMENSION", bound);
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept("DIMENSION_POSITION", counter - status);
    }
    cgout->LeaveLevel(bound_level_list);
}


IDLString DCE2ConceptualGraph::GetParenthesisAttribute( IDLParseNode *node, 
                                                        IDLString scope, 
                                                        IDLString root,
                                                        IDLString form )
{
    IDLParseNode *hnode;
    IDLParseNodeType type;
    IDLString attr;
    IDLString part;

    type = node->GetType();
    switch(type) 
    {
    case attribute:
 	attr = "[";
 	part = *node->GetString(); 
 	part += "]";
	break;
    case switch_type_attr:
        attr = "[switch_type(";
        hnode = node->GetBranch(1);
        part = GetTypes(hnode, scope, root, form);
        part += ")]";
        break;
    }
    attr += part;
    return(attr);
}


IDLString DCE2ConceptualGraph::GetTypes( IDLParseNode *node, IDLString scope,
                                         IDLString root, IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLParseNodeType htype;
    IDLString type;
    IDLString part;

    htype = node->GetType();
    switch(htype)
    {
    case union_type:
	type = "union ";
	if(node->GetBranch(1)) 
        {
	    hnode_1 = node->GetBranch(3);
	    if(hnode_1)
                part = GetDeclarator(hnode_1, scope, root, form);
	    part += " ";
	    type += part;
	    part = GetTypes(node->GetBranch(2), scope, root, form);
	    part += " {";
	    type += part;
	    part = GetTypes(node->GetBranch(1), scope, root, form);
	    part += "}";
	    type += part;
        }
	else
            type += *node->GetString();
	break;
    case union_switch:
	type = "switch(";
	part = GetTypes(node->GetBranch(1), scope, root, form);
	part += " ";
	type += part;
	part = *node->GetString();
	part += ") ";
	type += part;
	part = GetDeclarator(node->GetBranch(2), scope, root, form);
	type += part;
	break;
    case union_body: 
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();		
      	while(ntype == case_list)
        {
 	    hnode_2 = hnode_1->GetBranch(1);
 	    part = GetTypes(hnode_2, scope, root, form);
	    part += " ";
	    type += part;
 	    hnode_1 = hnode_1->GetBranch(2);
	    ntype = hnode_1->GetType();
        }
	part = GetTypes(hnode_1, scope, root, form);
	type += part;
	hnode_1 = node->GetBranch(2);
	if(hnode_1) 
        {
	    part = GetTypes(hnode_1, scope, root, form);
	    type += part;
        }
	break;
    case case_type: 
	hnode_1 = node->GetBranch(2);
	if(hnode_1) 
        {
	    ntype = hnode_1->GetType();
	    while(ntype == case_label_list)
            {
	 	hnode_2 = hnode_1->GetBranch(1);
	 	part = "case ";
		type += part;
		part = GetConstExpression(hnode_2);
		part += ":";
		type += part;
	 	hnode_1 = hnode_1->GetBranch(2);
	 	ntype = hnode_1->GetType(); 
            }
	    part = "case ";
	    type += part;
	    part = GetConstExpression(hnode_1);
	    type += part;
        }
	else
            type = "default:";
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
            part = GetTypeSpecification(hnode_1, scope, root, form);
	type += part;
	break;
    case union_type_n_e:
	type = "union ";
	hnode_1 = node->GetBranch(2); 
	if(hnode_1)
            part = GetDeclarator(hnode_1, scope, root, form);
	part += " {";
	type += part;
	part = GetTypes(node->GetBranch(1), scope, root, form);
	part += "}";
	type += part;
	break;
    case case_type_n_e: 
	hnode_1 = node->GetBranch(2);
	if(hnode_1)
            part = GetTypes(hnode_1, scope, root, form);
	else
            part = "[default] ";
	type = part;
	hnode_1 = node->GetBranch(1);
	if(hnode_1)
            part = GetTypeSpecification(hnode_1, scope, root, form);
	type += part;
	break;
    case case_label:
	type = "[case(";
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();
	while(ntype == const_expr_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    part = GetConstExpression(hnode_2);
	    part += ",";
	    type += part;
	    hnode_1 = hnode_1->GetBranch(2);
	    ntype = hnode_1->GetType();
        }
	part = GetConstExpression(hnode_1);
	part += ")] ";
	type += part;
	break;
    case struct_type:
	type = "struct ";
	hnode_1 = node->GetBranch(1);
	if(hnode_1) 
        {
	    hnode_2 = node->GetBranch(2);
	    if(hnode_2)
                part = GetDeclarator(hnode_2, scope, root, form);
	    part += "{";
	    type += part;
	    ntype = hnode_1->GetType();
	    while(ntype == member_list)
            {
	        hnode_2 = hnode_1->GetBranch(1);
	        part = GetTypeSpecification(hnode_2, scope, root, form);
	        part += ",";
		type += part;
	        hnode_1 = hnode_1->GetBranch(2);
	        ntype = hnode_1->GetType();
            }
	    part = GetTypeSpecification(hnode_1, scope, root, form);
	    part += "}";
	    type += part;
        }
	else
            type += *node->GetString();
	break;
    case enum_type:
        type = "enum{";
	hnode_1 = node->GetBranch(1);
	ntype = hnode_1->GetType();
 	while(ntype == declarator_list)
        {
	   hnode_2 = hnode_1->GetBranch(1);
	   part = GetDeclarator(hnode_2, scope, root, form);
	   part += ",";
	   type += part;
 	   hnode_1 = hnode_1->GetBranch(2);
 	   ntype = hnode_1->GetType();
        }
	part = GetDeclarator(hnode_1, scope, root, form);
	part = "}";
	type += part;
	break;
    case pipe_type:
	type = "pipe ";
	hnode_1 = node->GetBranch(1);
	part = GetTypes(hnode_1, scope, root, form);
	type += part;
	hnode_1 = node->GetBranch(2);
	ntype = hnode_1->GetType();
	while(ntype == declarator_list)
        {
	    hnode_2 = hnode_1->GetBranch(1);
	    part = GetDeclarator(hnode_2, scope, root, form);
	    part += ",";
	    type += part;
	    hnode_1 = hnode_1->GetBranch(2);
	    ntype = hnode_1->GetType();
        }
	part = GetDeclarator(hnode_1, scope, root, form);
	part += ",";
	type += part;
	break;
    case scoped_name:
    case base_type:
	type = *node->GetString();
	break;
    }
    return(type);
}


IDLString DCE2ConceptualGraph::GetParameterDeclarator( IDLParseNode *node, 
                                                       IDLString scope, 
                                                       IDLString root,
                                                       IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString declarator;
    IDLString part;

    hnode_1 = node->GetBranch(3);
    ntype = hnode_1->GetType();
    while(ntype == attribute_list)
    {
        hnode_2 = hnode_1->GetBranch(1);
        part = GetParenthesisAttribute(hnode_2, scope, root, form);
        part += ",";
        declarator += part;
        hnode_1 = hnode_1->GetBranch(2);
        ntype = hnode_1->GetType();
    }
    part = GetParenthesisAttribute(hnode_1, scope, root, form);
    part += " ";
    declarator += part;
    hnode_1 = node->GetBranch(1);
    part = GetTypes(hnode_1, scope, root, form);
    part += " ";
    declarator += part;
    hnode_1 = node->GetBranch(2);
    part = GetDeclarator(hnode_1, scope, root, form);
    declarator += part;
    return(declarator);
}


IDLString
DCE2ConceptualGraph::GetFunctionPointerDeclarator( IDLParseNode *node,
                                                   IDLString scope,
                                                   IDLString root,
                                                   IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString declarator;
    IDLString part;

    declarator = *node->GetString();
    hnode_1 = node->GetBranch(1);
    if(hnode_1) 
    {
        declarator += "("; 
        ntype = hnode_1->GetType();
        while(ntype == param_declarator_list) 
        {
            hnode_2 = hnode_1->GetBranch(1);
            part = GetParameterDeclarator(hnode_2, scope, root, form);
            part += ",";
            declarator += part;
            hnode_1 = hnode_1->GetBranch(2);
            ntype = hnode_1->GetType();
        }
        part = GetParameterDeclarator(hnode_1, scope, root, form);
        part += ")";
        declarator += part;
    }
    else
        declarator += "(void)"; 
    return(declarator); 
}


void DCE2ConceptualGraph::CreateFunctionPointerSubGraph( IDLParseNode *node,
                                                         IDLString scope, 
                                                         IDLString root,
                                                         IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    int pos;

    hnode_1 = node->GetBranch(1);
    if(hnode_1) 
    {
        cgout->PrintRelation("SIGNATURE");
        cgout->EnterNewLevel(parameter_level_list);
        pos = 1;
        ntype = hnode_1->GetType();
        while(ntype == param_declarator_list)
        {
            hnode_2 = hnode_1->GetBranch(1);
            pos = CreateMemberSpecification(hnode_2, scope, root,
                                            form, pos, "FUNCTION");
            pos++;
            hnode_1 = hnode_1->GetBranch(2);
            if(hnode_1)
                ntype = hnode_1->GetType();
        }
        pos = CreateMemberSpecification(hnode_1, scope, root, form,
                                        pos, "FUNCTION");
        cgout->LeaveLevel(parameter_level_list);
    }
}


IDLString DCE2ConceptualGraph::GetDeclarator( IDLParseNode *node,
                                              IDLString scope,
                                              IDLString root, IDLString form )
{
    IDLParseNodeType type;
    IDLString dcl;
    
    type = node->GetType();
    switch(type)
    {
    case declarator:
    case ptr_declarator:
	dcl = *node->GetString();
	break;
    case array_declarator:
	dcl = GetArrayDeclarator(node);
	break;
    case function_ptr_dcl:
	dcl = GetFunctionPointerDeclarator(node, scope, root, form);
	break;
    case param_declarator:
	dcl = GetParameterDeclarator(node, scope, root, form);
	break;
    }
    return(dcl);
}


IDLString DCE2ConceptualGraph::GetTypeSpecification( IDLParseNode *node,
                                                     IDLString scope,
                                                     IDLString root,
                                                     IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString specification;
    IDLString part;

    hnode_1 = node->GetBranch(3);
    if(hnode_1)
    {
        ntype = hnode_1->GetType();
        while(ntype == attribute_list)
        {
            hnode_2 = hnode_1->GetBranch(1);
            part = GetParenthesisAttribute(hnode_2, scope, root, form);
            part += ",";
            specification += part;
            hnode_1 = hnode_1->GetBranch(2);
            ntype = hnode_1->GetType();
        }
        part = GetParenthesisAttribute(hnode_1, scope, root, form);
        part += " ";
        specification += part;
    }
    hnode_1 = node->GetBranch(1);
    part = GetTypes(hnode_1, scope, root, form);
    part += " ";
    specification += part;
    hnode_1 = node->GetBranch(2);
    ntype = hnode_1->GetType();
    while(ntype == declarator_list) 
    {
        hnode_2 = hnode_1->GetBranch(1);
        part = GetDeclarator(hnode_2, scope, root, form);
        part += ",";
        specification += part;
        hnode_1 = hnode_1->GetBranch(2);
        ntype = hnode_1->GetType();
    }
    part = GetDeclarator(hnode_1, scope, root, form);
    specification += part;
    return(specification);
}


int DCE2ConceptualGraph::ExamineAttributes( IDLParseNode *node, char *target )
{
    IDLParseNodeType ntype;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLString attribute;
    
    hnode_1 = node;
    if(!hnode_1)
        return(0);
    else
    {
        while(hnode_1)
	{
            ntype = hnode_1->GetType();
            if(ntype == attribute_list)
                hnode_2 = hnode_1->GetBranch(1);
            else
                hnode_2 = hnode_1;
            assert(hnode_2);
            attribute = *hnode_2->GetString();
            if(attribute == target || contains(attribute, target))
                return(1);
            hnode_1 = hnode_1->GetBranch(2);
	}
    }
    /*
     * XXX hier fehlte ein return
     */
    return 0;
}


void DCE2ConceptualGraph::CreateDeclaratorType( IDLParseNode *node,
                                                IDLString scope, 
                                                IDLString root,
                                                IDLString form, 
                                                char *name, char *extension )
{
    IDLParseNodeType ntype;
    IDLString tname;
    IDLString cname;
    
    cname = extension;
    cname += "_TYPE";
    ntype = node->GetType();
    switch(ntype) 
    {
    case scoped_name:
    case base_type:
	cgout->PrintRelation("TYPE");
 	cgout->PrintConcept(cname);	
	CreateTypes(node, scope, root, form);
	break;
    case pipe_type:
    case union_type:
	tname = *node->GetString();
	if(name == "UNION") 
        {
	    if(tname == root)
                CreateRecursiveType(node);
	    else
                CreateTypes(node, scope, root, form);
        }
	else 
        {
	    if(name == "OPERATION" || name == "FUNCTION") 
            {
		cgout->PrintRelation("TYPE");
 		cgout->PrintConcept(cname, tname);
            }
	    else
                CreateTypes(node, scope, root, form);
        }
	break;
    case struct_type:
	tname = *node->GetString();
	if(name == "STRUCT") 
        {
	    if(tname == root)
                CreateRecursiveType(node);
	    else
                CreateTypes(node, scope, root, form);
        }
	else 
        {
	    if(name == "OPERATION" || name == "FUNCTION") 
            {
		cgout->PrintRelation("TYPE");
 		cgout->PrintConcept(cname, tname);
            }
	    else
                CreateTypes(node, scope, root, form);
        }
	break;
    case enum_type:
	assert(name != "OPERATION" && name != "FUNCTION");
	CreateTypes(node, scope, root, form);
	break;
    }
}


void DCE2ConceptualGraph::CreateDeclaratorPosition( char *name, int pos )
{
    if(pos > 0) 
    {
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept(name, "_POSITION", pos);
    }
}


void DCE2ConceptualGraph::CreateAttributes( IDLParseNode *node,
                                            IDLString cname )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString attribute;
    int in_detected = 0;
    int out_detected = 0;
 
    if(node)
    {
        cgout->PrintRelation("ATTRIBUTES");
        cgout->PrintConcept(cname);
        hnode_1 = node;
        ntype = hnode_1->GetType();
        while(ntype == attribute_list)
    	{
            hnode_2 = hnode_1->GetBranch(1);
            attribute = *hnode_2->GetString();
            if(attribute == "in")
                in_detected = 1;
            else if(attribute == "out")
                out_detected = 1; 
            else
                cgout->PrintInstance(attribute);
            hnode_1 = hnode_1->GetBranch(2);
            assert(hnode_1);
            ntype = hnode_1->GetType();
    	}
        attribute = *hnode_1->GetString();
        if(attribute == "in")
            in_detected = 1;
        if(attribute == "out")
            out_detected = 1;
        if(in_detected)
    	{	
            if(out_detected)
	    {
                if(!(attribute == "in") && !(attribute == "out"))
	    	{
                    cgout->PrintInstance(attribute);
                    attribute = "inout";
                    cgout->PrintLastInstance(attribute);
	    	}
                else 
	    	{
                    attribute = "inout";
                    cgout->PrintLastInstance(attribute);
	    	}
	    }
            else
	    {
                if(!(attribute == "in"))
	    	{
                    cgout->PrintInstance(attribute);
                    attribute = "in";
                    cgout->PrintLastInstance(attribute);
	    	}
                else
                    cgout->PrintLastInstance(attribute);
            }
    	}
        else
        {
            if(out_detected)
	    {
                if(!(attribute == "out"))
	    	{
                    cgout->PrintInstance(attribute);
                    attribute = "out";
                    cgout->PrintLastInstance(attribute);
	    	}
                else
                    cgout->PrintLastInstance(attribute);
	    }
            else
                cgout->PrintLastInstance(attribute);
    	}
    }
}


void DCE2ConceptualGraph::CreateTypeSpecification( IDLParseNode *node,
                                                   IDLString scope,
                                                   IDLString root,
                                                   IDLString form )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNode *hnode_4;
    IDLParseNodeType dtype;
    IDLParseNodeType ntype;
    IDLString type_declarator;
    IDLString name;
    IDLString cname;
    int dim;
    int status;

    hnode_1 = node->GetBranch(2);
    hnode_4 = node->GetBranch(3);
    hnode_3 = node->GetBranch(1);
    assert(hnode_3);
    dtype = hnode_3->GetType();
    while(hnode_1)
    {
        ntype = hnode_1->GetType();
        if(ntype == declarator_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        assert(hnode_2);
        type_declarator = *hnode_2->GetString();
        ntype = hnode_2->GetType(); 
        switch(ntype)
	{
        case ptr_declarator:
	    dim = freq(type_declarator, "*");
	    assert(dim == 1);
	    del(type_declarator, "*");
	    status = DetectPointerConversionStatus(node);
	    break;
        case array_declarator:
	    dim = CountArrayDimensions(hnode_2->GetBranch(1));
	    status = DetectArrayConversionStatus(node, hnode_2);
	    break;
        case function_ptr_dcl:
	    dim = freq(type_declarator, "*");
	    del(type_declarator, "*");
	    del(type_declarator, "(");
	    del(type_declarator, ")");
	    status = 0;
	    break;
        case declarator:
	    status = 0;
	    break;
	}
        cgout->PrintConcept("TYPE", type_declarator);
        cgout->EnterNewLevel(type_level);
        cgout->PrintRelation("ANNOTATIONS");
        cgout->EnterNewLevel(annotation_level_list);
        if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
        cgout->PrintConcept("SOURCE_NAME", root);
        cgout->PrintConcept("SOURCE_FORM", form);
        switch(ntype)
        {
        case declarator:
            cgout->LeaveLevel(annotation_level_list);
            if(dtype == scoped_name || dtype == base_type)
            {
                cgout->PrintRelation("TYPE");
                cgout->PrintConcept("TYPE_TYPE");
            }	
            CreateTypes(hnode_3, scope, root, form);
            cname = "TYPE_ATTRIBUTES";
            CreateAttributes(hnode_4, cname);
            break;
        case ptr_declarator:
            switch(status)
            {
            case 0:
                cgout->PrintPureConcept("POINTER");
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("POINTER", dim);
                cgout->EnterNewLevel(declarator_level);
                CreateTypes(hnode_3, scope, root, form);
                cgout->LeaveLevel(declarator_level);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            case 1:
                cgout->PrintConcept("POINTER", "ptr");
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("SEQUENCE");
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintConcept("ELEMENT_TYPE");
                    CreateTypes(hnode_3, scope, root, form);
                }
                else 
                {
                    name = *hnode_3->GetString();
                    cgout->PrintConcept("ELEMENT_TYPE", name);
                    cgout->PrintRelation("ANNOTATIONS");
                    cgout->EnterNewLevel(annotation_level_list);
                    if(dtype == union_type)
                        cgout->PrintConcept("UNION", name);
                    else
                        cgout->PrintConcept("STRUCT", name);
                    cgout->LeaveLevel(annotation_level_list);
                }
                cgout->LeaveLevel(template_type_level);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            case 2:
                cgout->PrintConcept("POINTER", "ref");
                cgout->LeaveLevel(annotation_level_list);
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintRelation("TYPE");
                    cgout->PrintConcept("TYPE_TYPE");
                }	
                CreateTypes(hnode_3, scope, root, form);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            case 3:
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("STRING");
                cgout->LeaveLevel(template_type_level);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            }
            break;	
        case array_declarator:
            switch(status)
            {
            case 0:
                name = GetArrayDeclarator(hnode_2);
                cgout->PrintConcept("FIXED_ARRAY", name);
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("ARRAY", dim);
                cgout->EnterNewLevel(declarator_level);
                CreateArraySubGraph(hnode_2, status);
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintRelation("TYPE");
                    cgout->PrintConcept("ARRAY_TYPE");
                }	
                CreateTypes(hnode_3, scope, root, form);
                cgout->LeaveLevel(declarator_level);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            case 1:
                name = GetArrayDeclarator(hnode_2);
                cgout->PrintConcept("VARYING_ARRAY", name);
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("STRUCT", type_declarator);
                cgout->EnterNewLevel(constructed_type_level);
                cgout->PrintRelation("MEMBERS");
                cgout->EnterNewLevel(member_level_list);
                cgout->PrintConcept("STRUCT_MEMBER", "lower");
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("STRING");
                cgout->LeaveLevel(template_type_level);
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 1);
                cgout->LeaveLevel(member_level);
                cgout->PrintConcept("STRUCT_MEMBER", "uppermax");
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("STRING");
                cgout->LeaveLevel(template_type_level);
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 2);
                cgout->LeaveLevel(member_level);
                cgout->PrintConcept("STRUCT_MEMBER", "upper");
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("TYPE");
                cgout->PrintConcept("MEMBER_TYPE", "boolean");
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 3);
                cgout->LeaveLevel(member_level);
                name = declarator + "_arr";
                cgout->PrintConcept("STRUCT_MEMBER", name);
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("ARRAY", dim);
                cgout->EnterNewLevel(declarator_level);
                CreateArraySubGraph(hnode_2, status);
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintRelation("TYPE");
                    cgout->PrintConcept("ARRAY_TYPE");
                }	
                CreateTypes(hnode_3, scope, root, form);
                cgout->LeaveLevel(declarator_level);
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 4);
                cgout->LeaveLevel(constructed_type_level);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            case 2:
                name = GetArrayDeclarator(hnode_2);
                cgout->PrintConcept("CONFORMANT_ARRAY", name);
                cgout->LeaveLevel(annotation_level_list);
                if(dim > 1) 
                {
                    cgout->PrintRelation("CONSTRUCTED_TYPE");
                    cgout->PrintConcept("ARRAY", dim - 1);
                    cgout->EnterNewLevel(declarator_level);
                    CreateArraySubGraph(hnode_2, 1);
                }
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("SEQUENCE");
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintConcept("ELEMENT_TYPE");
                    CreateTypes(hnode_3, scope, root, form);
                }
                else 
                {
                    name = *hnode_3->GetString();
                    cgout->PrintConcept("ELEMENT_TYPE", name);
                    cgout->PrintRelation("ANNOTATIONS");
                    cgout->EnterNewLevel(annotation_level_list);
                    if(dtype == union_type)
                        cgout->PrintConcept("UNION", name);
                    else
                        cgout->PrintConcept("STRUCT", name);
                    cgout->LeaveLevel(annotation_level_list);
                }	
                cgout->LeaveLevel(template_type_level);
                if(dim > 1)
                    cgout->LeaveLevel(declarator_level);
                cname = "TYPE_ATTRIBUTES";
                CreateAttributes(hnode_4, cname);
                break;
            }
            break;
        case function_ptr_dcl:
            name = GetFunctionPointerDeclarator(hnode_2, scope, root, form);
            cgout->PrintConcept("FUNCTION_POINTER", name);
            cgout->LeaveLevel(annotation_level_list);
            cgout->PrintRelation("CONSTRUCTED_TYPE");
            cgout->PrintConcept("FUNCTION_POINTER", dim);
            cgout->EnterNewLevel(declarator_level);
            CreateFunctionPointerSubGraph(hnode_2, scope, root, form);	
            assert(dtype == scoped_name || dtype == base_type);
            cgout->PrintRelation("TYPE");
            cgout->PrintConcept("FUNCTION_TYPE");
            CreateTypes(hnode_3, scope, root, form);
            cgout->LeaveLevel(declarator_level);
            cname = "TYPE_ATTRIBUTES";
            CreateAttributes(hnode_4, cname);
            break;
        }
        cgout->LeaveLevel(type_level);
        hnode_1 = hnode_1->GetBranch(2);
    }
}


int DCE2ConceptualGraph::DetectPointerConversionStatus( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNodeType ntype;
    IDLString ptype;
    int actual_mode = 0;
    int status = 0; 
    
    actual_mode = DetectPointerMode(node->GetBranch(3));
    hnode_1 = node->GetBranch(1);
    assert(hnode_1); 
    ntype = hnode_1->GetType();
    switch(ntype)
    {
    case scoped_name:
    case base_type:
	ptype = *hnode_1->GetString();
	hnode_2 = node->GetBranch(3);
	if(ptype == "char")
        {
	    if(ExamineAttributes(hnode_2, "string"))
                status = 3;
	    else
                status = actual_mode;
        }
	else
            status = actual_mode;
	break;
    case pipe_type:
	status = 0;
	break;
    case union_type:
    case struct_type:
	if(!hnode_1->GetBranch(1))
            status = actual_mode;
	else
            status = 0;
	break;
    case enum_type:
	if(actual_mode == 1)
            status = 1;
	else
            status = 0;
	break;
    }
    return(status);
}


int DCE2ConceptualGraph::DetectPointerMode( IDLParseNode *node )
{
    int tmode = 0;
 
    switch(mode)
    {
    case 0:
	if(ExamineAttributes(node, "ptr"))
            tmode = 1; 
	else if(ExamineAttributes(node, "ref"))
            tmode = 2;
#if 0
        else
            assert(0);
#else
        /*
         * XXX default for pointers in parameter lists is "ref"
         */
        tmode = 2;
#endif
	break;
    case 1:
	if(ExamineAttributes(node, "ref"))
            tmode = 2;
	else
            tmode = 1;
	break;
    case 2:
	if(ExamineAttributes(node, "ptr"))
            tmode = 1; 
	else
            tmode = 2;
	break;
    }
    return(tmode);
}


int DCE2ConceptualGraph::CountArrayDimensions( IDLParseNode *node )
{
    IDLParseNode *hnode_1;
    int dim = 0;

    hnode_1 = node;
    while(hnode_1)
    {
        dim++;
        hnode_1 = hnode_1->GetBranch(2);
    }
    assert(dim); 
    return(dim);
}


int DCE2ConceptualGraph::DetectArrayConversionStatus( IDLParseNode *node_1,
                                                      IDLParseNode *node_2 )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNodeType ntype;
    IDLParseNodeType dtype;
    IDLString bound;
    int status = 0;
 
    hnode_3 = node_1->GetBranch(1);
    assert(hnode_3);
    dtype = hnode_3->GetType();
    hnode_1 = node_1->GetBranch(3);
    hnode_2 = node_2->GetBranch(1);
    assert(hnode_2);
    ntype = hnode_2->GetType();
    if(ntype == array_bound_list) 
    {
        hnode_2 = hnode_2->GetBranch(1);
        assert(hnode_2);
    }
    bound = *hnode_2->GetString();
    if(bound == "[]" || contains(bound, "*")) 
    {
        /* conformant array */
        if(ExamineAttributes(hnode_1, "size_is(") ||
           ExamineAttributes(hnode_1, "max_is("))
            status = 2;
        else {
            cerr << "warning: missing `size_is' or `max_is' for "
                 << "unbounded array `" << *node_2->GetString() << "'" << endl;
            status = 2;
        }
    }
    else
    {
        if(ExamineAttributes(hnode_1, "first_is(") ||
           ExamineAttributes(hnode_1, "last_is(") ||
           ExamineAttributes(hnode_1, "length_is("))
	{
            /* varying array */
            switch(dtype)
	    {
            case base_type:
            case scoped_name:
                /* XXX Was soll denn das ? */
		if(0)
                    status = 1;
		else
                    status = 0;
		break;
            case enum_type:
            case pipe_type:
		status = 0;
		break;
            case struct_type:
            case union_type:
                /* XXX Was soll denn das ? */
		if(hnode_3->GetBranch(1))
                    status = 0;
		else if(0)
                    status = 1;
                else
                    status = 0;
		break;
	    }
	}
        /* else fixed array */
    }
    return(status);
}


int DCE2ConceptualGraph::CreateMemberSpecification( IDLParseNode *node,
                                                    IDLString scope,
                                                    IDLString root,
                                                    IDLString form,
                                                    int pos, char* cname )
{
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLParseNode *hnode_3;
    IDLParseNode *hnode_4;
    IDLParseNodeType type;
    IDLParseNodeType dtype;
    IDLParseNodeType ntype;
    CGLevelType level;
    IDLString member_declarator;
    IDLString name;
    IDLString cname_1;
    IDLString cname_2;
    char *cname_3;
    char* extension;
    int offset = -1;
    int dim;
    int status;
    int actual_mode;

    type = node->GetType();
    if(type == field_declarator) 
    {
        extension = "MEMBER";
        level = member_level;
    }
    else
    {
        extension = "PARAMETER";
        level = parameter_level;
    }
    cname_1 = cname;
    cname_1 += "_";
    cname_1 += extension;
    hnode_1 = node->GetBranch(2);
    hnode_4 = node->GetBranch(3);
    hnode_3 = node->GetBranch(1);
    if(hnode_3)
        dtype = hnode_3->GetType();
    while(hnode_1) 	
    {
        offset++;
        ntype = hnode_1->GetType();
        if(ntype == declarator_list)
            hnode_2 = hnode_1->GetBranch(1);
        else
            hnode_2 = hnode_1;
        assert(hnode_2);
        member_declarator = *hnode_2->GetString();
        ntype = hnode_2->GetType(); 
        switch(ntype)
	{
        case ptr_declarator:
	    dim = freq(member_declarator, "*");
	    if(level == parameter_level)
            {
		if(ExamineAttributes(hnode_4, "out"))
                {
		    actual_mode = DetectPointerMode(hnode_4);
		    if(dim == 1 && actual_mode == 2)
                        status = 2; 
		    else if(dim == 2)
                        status = 1;
                    /*
                     * XXX stimmt das ?
                     */
                    else
                        status = 0;
                }
		else
                    status = DetectPointerConversionStatus(node);
            }		
	    else
                status = DetectPointerConversionStatus(node);
	    gsub(member_declarator, "*", "");
	    assert(dim <= 2);
	    break;
        case array_declarator:
	    dim = CountArrayDimensions(hnode_2->GetBranch(1));
	    status = DetectArrayConversionStatus(node, hnode_2);
	    break;
        case function_ptr_dcl:
	    dim = freq(member_declarator, "*");
	    del(member_declarator, "*");
	    del(member_declarator, "(");
	    del(member_declarator, ")");
	    status = 0;
	    break;
        case declarator:
	    status = 0;
	    break;
	}
        cgout->PrintConcept(cname_1, member_declarator);
        cgout->EnterNewLevel(level);
        switch(ntype)
	{
        case declarator:
     	    CreateDeclaratorType(hnode_3, scope, root, form, cname, extension);
     	    CreateDeclaratorPosition(extension, pos + offset);
//	    cname_2 = cname_1 + "_ATTRIBUTES";
	    cname_2 = string(extension) + "_ATTRIBUTES";
	    CreateAttributes(hnode_4, cname_2);
	    break;
        case ptr_declarator:
	    switch(status)
            {
            case 0:
                cgout->PrintRelation("ANNOTATIONS");
                cgout->EnterNewLevel(annotation_level_list);
                cgout->PrintPureConcept("POINTER");
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("POINTER", dim);
                cgout->EnterNewLevel(declarator_level);
                CreateDeclaratorType(hnode_3, scope, root, form, cname,
                                     extension);
                cgout->LeaveLevel(declarator_level);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            case 1:
                cgout->PrintRelation("ANNOTATIONS");
                cgout->EnterNewLevel(annotation_level_list);
                cgout->PrintConcept("POINTER", "ptr");
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("SEQUENCE");
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintConcept("ELEMENT_TYPE");
                    CreateTypes(hnode_3, scope, root, form);
                }
                else 
                {
                    name = *hnode_3->GetString();
                    cgout->PrintConcept("ELEMENT_TYPE", name);
                    cgout->PrintRelation("ANNOTATIONS");
                    cgout->EnterNewLevel(annotation_level_list);
                    if(dtype == union_type)
                        cgout->PrintConcept("UNION", name);
                    else
                        cgout->PrintConcept("STRUCT", name);
                    cgout->LeaveLevel(annotation_level_list);
                }	
                cgout->LeaveLevel(template_type_level);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            case 2:
                cgout->PrintRelation("ANNOTATIONS");
                cgout->EnterNewLevel(annotation_level_list);
                cgout->PrintConcept("POINTER", "ref");
                cgout->LeaveLevel(annotation_level_list);
                CreateDeclaratorType(hnode_3, scope, root, form, cname,
                                     extension);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            case 3:
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("STRING");
                cgout->LeaveLevel(template_type_level);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            }
	    break;	
        case array_declarator:
	    switch(status)
            {
            case 0:
                cgout->PrintRelation("ANNOTATIONS");
                cgout->EnterNewLevel(annotation_level_list);
                name = GetArrayDeclarator(hnode_2);
                cgout->PrintConcept("FIXED_ARRAY", name);
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("ARRAY", dim);
                cgout->EnterNewLevel(declarator_level);
                CreateArraySubGraph(hnode_2, status);
                cname_3 = "ARRAY";
                CreateDeclaratorType(hnode_3, scope, root, form,
                                     cname, cname_3);
                cgout->LeaveLevel(declarator_level);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            case 1:
                cgout->PrintRelation("ANNOTATIONS");
                cgout->EnterNewLevel(annotation_level_list);
                name = GetArrayDeclarator(hnode_2);
                cgout->PrintConcept("VARYING_ARRAY", name);
                cgout->LeaveLevel(annotation_level_list);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("STRUCT", member_declarator);
                cgout->EnterNewLevel(constructed_type_level);
                cgout->PrintRelation("MEMBERS");
                cgout->EnterNewLevel(member_level_list);
                cgout->PrintConcept("STRUCT_MEMBER", "lower");
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("STRING");
                cgout->LeaveLevel(template_type_level);
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 1);
                cgout->LeaveLevel(member_level);
                cgout->PrintConcept("STRUCT_MEMBER", "uppermax");
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("STRING");
                cgout->LeaveLevel(template_type_level);
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 2);
                cgout->LeaveLevel(member_level);
                cgout->PrintConcept("STRUCT_MEMBER", "upper");
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("TYPE");
                cgout->PrintConcept("MEMBER_TYPE", "boolean");
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 3);
                cgout->LeaveLevel(member_level);
                name = declarator + "_arr";
                cgout->PrintConcept("STRUCT_MEMBER", name);
                cgout->EnterNewLevel(member_level);
                cgout->PrintRelation("CONSTRUCTED_TYPE");
                cgout->PrintConcept("ARRAY", dim);
                cgout->EnterNewLevel(declarator_level);
                CreateArraySubGraph(hnode_2, status);
                cname_3 = "ARRAY";
                CreateDeclaratorType(hnode_3, scope, root, form,
                                     cname, cname_3);
                cgout->LeaveLevel(declarator_level);
                cgout->PrintRelation("POSITION");
                cgout->PrintConcept("MEMBER_POSITION", 4);
                cgout->LeaveLevel(constructed_type_level);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            case 2:
                cgout->PrintRelation("ANNOTATIONS");
                cgout->EnterNewLevel(annotation_level_list);
                name = GetArrayDeclarator(hnode_2);
                cgout->PrintConcept("CONFORMANT_ARRAY", name);
                cgout->LeaveLevel(annotation_level_list);
                if(dim > 1) 
                {
                    cgout->PrintRelation("CONSTRUCTED_TYPE");
                    cgout->PrintConcept("ARRAY", dim - 1);
                    cgout->EnterNewLevel(declarator_level);
                    CreateArraySubGraph(hnode_2, 1);
                }
                cgout->PrintRelation("TEMPLATE_TYPE");
                cgout->EnterNewLevel(template_type_level);
                cgout->PrintPureConcept("SEQUENCE");
                if(dtype == scoped_name || dtype == base_type)
                {
                    cgout->PrintConcept("ELEMENT_TYPE");
                    CreateTypes(hnode_3, scope, root, form);
                }
                else 
                {
                    name = *hnode_3->GetString();
                    cgout->PrintConcept("ELEMENT_TYPE", name);
                    cgout->PrintRelation("ANNOTATIONS");
                    cgout->EnterNewLevel(annotation_level_list);
                    if(dtype == union_type)
                        cgout->PrintConcept("UNION", name);
                    else
                        cgout->PrintConcept("STRUCT", name);
                    cgout->LeaveLevel(annotation_level_list);
                }	
                cgout->LeaveLevel(template_type_level);
                if(dim > 1)
                    cgout->LeaveLevel(declarator_level);
                CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
                cname_2 = string(extension) + "_ATTRIBUTES";
                CreateAttributes(hnode_4, cname_2);
                break;
            }
	    break;
        case function_ptr_dcl:
	    cgout->PrintRelation("ANNOTATIONS");
     	    cgout->EnterNewLevel(annotation_level_list);
	    name = GetFunctionPointerDeclarator(hnode_2, scope, root, form);
	    cgout->PrintConcept("FUNCTION_POINTER", name);
	    cgout->LeaveLevel(annotation_level_list);
	    cgout->PrintRelation("CONSTRUCTED_TYPE");
	    cgout->PrintConcept("FUNCTION_POINTER", dim);
	    cgout->EnterNewLevel(declarator_level);
	    CreateFunctionPointerSubGraph(hnode_2, scope, root, form);	
     	    assert(dtype == scoped_name || dtype == base_type);
     	    cgout->PrintRelation("TYPE");
     	    cgout->PrintConcept("FUNCTION_TYPE");
	    CreateTypes(hnode_3, scope, root, form);
	    cgout->LeaveLevel(declarator_level);
     	    CreateDeclaratorPosition(extension, pos + offset);
//                cname_2 = cname_1 + "_ATTRIBUTES";
	    cname_2 = string(extension) + "_ATTRIBUTES";
	    CreateAttributes(hnode_4, cname_2);
	    break;
	}
        cgout->LeaveLevel(level);
        hnode_1 = hnode_1->GetBranch(2);
    }
    return(pos + offset);
}


IDLString DCE2ConceptualGraph::GetAttribute( IDLParseNode *node,
                                             IDLString scope, 
                                             IDLString root, IDLString form )
{
    IDLParseNode *hnode;
    IDLParseNodeType type;
    IDLString attr;
    IDLString part;

    type = node->GetType();
    switch(type) 
    {
    case endpoint:
    case attr_part:
    case attribute:
 	attr = *node->GetString(); 
	break;
    case switch_type_attr:
        attr = "switch_type(";
        hnode = node->GetBranch(1);
        part = GetTypes(hnode, scope, root, form);
        part += ")";
 	attr += part;
        break;
    }
    return(attr);
}


void DCE2ConceptualGraph::CreateOperations( IDLParseNode *node,
                                            IDLString scope, 
                                            IDLString root, IDLString form, 
                                            int position )
{
    IDLParseNodeType ntype;
    IDLParseNodeType type;
    IDLParseNode *hnode_1;
    IDLParseNode *hnode_2;
    IDLString name;
    IDLString tscope; 
    IDLString attribute; 
    int counter;
    int pos;

    type = node->GetType();
    switch(type) 
    {
    case interface_dcl:
	counter = 1;
	hnode_1 = node->GetBranch(2);
	if(hnode_1) 
        {
	    while(hnode_1) 
            {
		hnode_2 = hnode_1->GetBranch(1);
		if(hnode_2)
                    ntype = hnode_2->GetType();
		if(ntype == op_dcl) 
                {
		    if(counter == 1)
                    {
			cgout->PrintRelation("OPERATIONS");
		    	cgout->EnterNewLevel(operation_level_list);
                    }	
		    CreateOperations(hnode_2, scope, root, form, counter);
		    counter++;
                }
		hnode_1 = hnode_1->GetBranch(2);
            }
        }
	if(counter > 1)
            cgout->LeaveLevel(operation_level_list);
	break;
    case op_dcl:
	name = *node->GetString();
	cgout->PrintConcept("OPERATION", name);
	tscope = ActualizeScope(scope, name);
	cgout->EnterNewLevel(operation_level);
        cgout->PrintRelation("ANNOTATIONS");
        cgout->EnterNewLevel(annotation_level_list);
        if(!(scope == ""))
            cgout->PrintConcept("SCOPE", scope);
        cgout->PrintConcept("SOURCE_NAME", root);
	cgout->PrintConcept("SOURCE_FORM", form);
	cgout->LeaveLevel(annotation_level_list);
	cgout->PrintRelation("TYPE");
 	cgout->PrintConcept("OPERATION_TYPE");
	hnode_2 = node->GetBranch(1);
	CreateTypes(hnode_2, tscope, name, form);
	hnode_1 = node->GetBranch(3);
	if(hnode_1) 
        {
	    cgout->PrintRelation("ATTRIBUTES");
	    ntype = hnode_1->GetType();
	    while(ntype == attribute_list)
            {
	        hnode_2 = hnode_1->GetBranch(1);
	        attribute = GetAttribute(hnode_2, tscope, name, form);
	        attribute += ", ";
	        hnode_1 = hnode_1->GetBranch(2);
	        if(hnode_1)
                    ntype = hnode_1->GetType();
            }
	    attribute += GetAttribute(hnode_1, tscope, name, form);
	    cgout->PrintConcept("OPERATION_ATTRIBUTES", attribute);
        }
	cgout->PrintRelation("SIGNATURE");
	cgout->EnterNewLevel(parameter_level_list);
	hnode_1 = node->GetBranch(2);
	if(hnode_1) 
        {
	    pos = 1;
	    ntype = hnode_1->GetType();
	    while(ntype == param_declarator_list)
            {
	        hnode_2 = hnode_1->GetBranch(1);
	        pos = CreateMemberSpecification(hnode_2, tscope, name,
                                                form, pos, "OPERATION");
	        pos++;
	        hnode_1 = hnode_1->GetBranch(2);
	        if(hnode_1)
                    ntype = hnode_1->GetType();
            }
	    pos = CreateMemberSpecification(hnode_1, tscope, name, form,
                                            pos, "OPERATION");
	    pos++;
	    InsertStdExceptParameter(pos);
        }
        else
            InsertStdExceptParameter(1); 
	cgout->LeaveLevel(parameter_level_list);
        cgout->PrintRelation("POSITION");
        cgout->PrintConcept("OPERATION_POSITION", position);
	cgout->LeaveLevel(operation_level);
        break;
    }
}






