
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#ifndef __IDLCONVERTER_H__
#define __IDLCONVERTER_H__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strstream.h>

#include "idlstring.h"
#include "idlparsenode.h"
#include "printcg.h" 
#include <Trader.h>


class IDL2ConceptualGraph 
{
protected:
    CG2Stream *cgout;
public:
    IDL2ConceptualGraph( ostream &o );
    ~IDL2ConceptualGraph();
    IDLString ActualizeScope( IDLString scope, IDLString root );
    void CreateStdExceptType();
    void InsertStdExceptParameter( int pos );
};


class CORBA2ConceptualGraph : public IDL2ConceptualGraph
{
private:
    void CreateInterfaceGraph( IDLParseNode *node, IDLString scope, 
                               IDLString root, IDLString form );
    void CollectConstants( IDLParseNode *node, IDLString scope, 
                           IDLString root, IDLString form );
    void CreateConstant( IDLParseNode *node, IDLString scope, 
                         IDLString root, IDLString form );
    IDLString GetConstExpression( IDLParseNode *node );
    int ConvertAttributes( IDLParseNode *node, IDLString scope, 
                           IDLString root, IDLString form, int pos );
    void CreateSetOperation( IDLParseNode *hnode, IDLParseNode *tnode, 
                             IDLString scope, IDLString root, 
                             IDLString form, int pos );
    void CreateGetOperation( IDLParseNode *hnode, IDLParseNode *tnode, 
                             IDLString scope, IDLString root, 
                             IDLString form, int pos );
    void CollectTypes( IDLParseNode *node, IDLString scope, 
                       IDLString root, IDLString form );
    void CreateTypes( IDLParseNode *node, IDLString scope, 
                      IDLString root, IDLString form );
    void CreateRecursiveType( IDLParseNode *node, IDLString scope, 
                              IDLString root, IDLString form );
    void CollectOperations( IDLParseNode *node, IDLString scope, 
                            IDLString root, IDLString form );
    int LookForContextExpression( IDLParseNode *node );
    void ConvertContextExpression( IDLParseNode *node );
    int LookForRaisesExpression( IDLParseNode *node );
    void ConvertRaisesExpression( IDLParseNode *node, IDLString scope,
                                  IDLString root, IDLString form );
    void CreateOperation( IDLParseNode *node, IDLString scope, 
                          IDLString root, IDLString form,
                          int position );
    void ConvertException( IDLParseNode *node, IDLString scope, 
                           IDLString root, IDLString form );
    IDLString GetTemplateTypeDeclarator( IDLParseNode *node );
    IDLString GetDeclarator( IDLParseNode *node );
    IDLString GetArrayDeclarator( IDLParseNode *node );
    IDLString GetArrayBound( IDLParseNode *node );
    void CreateArraySubGraph( IDLParseNode *node );
    void CreateTypeSpecification( IDLParseNode *node, IDLString scope, 
                                  IDLString root, IDLString form );
    int CreateMemberSpecification( IDLParseNode *node, IDLString scope, 
                                   IDLString root, IDLString form,
                                   int pos );
    int CountArrayDimensions( IDLParseNode *node );
public:
    CORBA2ConceptualGraph( ostream &o ) : IDL2ConceptualGraph(o) {};
    ~CORBA2ConceptualGraph();
    void CreateConceptualGraph( IDLParseNode *node );
};


class DCE2ConceptualGraph : public IDL2ConceptualGraph
{ 
private:
    int mode;
    void CollectInterfaceGraph( IDLParseNode *node, IDLString scope, 
                                IDLString root, IDLString form );
    IDLString CreateInterfaceHeader( IDLParseNode *node, IDLString scope, 
                                     IDLString root, IDLString form );
    void CollectInterfaceAttributes( IDLParseNode *node, IDLString scope, 
                                     IDLString root, IDLString form );
    void CreateInterfaceAttributes( IDLParseNode *node, IDLString scope, 
                                    IDLString root, IDLString form );
    void CollectConstants( IDLParseNode *node, IDLString scope, 
                           IDLString root, IDLString form );
    int CollectConstants( IDLParseNode *node_A, IDLParseNode *node_B, 
                          IDLString scope, IDLString root, 
                          IDLString form, int flag );
    void CreateConstants( IDLParseNode *node, IDLString scope, 
                          IDLString root, IDLString form );
    void CollectTypes( IDLParseNode *node, IDLString scope, 
                       IDLString root, IDLString form ); 
    int CollectTypes( IDLParseNode *node_A, IDLParseNode *node_B, 
                      IDLString scope, IDLString root, IDLString form, 
                      int flag );
    void CreateTypes( IDLParseNode *node, IDLString scope, 
                      IDLString root, IDLString form );
    void CreateRecursiveType( IDLParseNode *node );
    void CreateTypeSpecification( IDLParseNode *node, IDLString scope, 
                                  IDLString root, IDLString form );
    void CreateOperations( IDLParseNode *node, IDLString scope, 
                           IDLString root, IDLString form, int position );
    int CreateMemberSpecification( IDLParseNode *node, IDLString scope, 
                                   IDLString root, IDLString form,
                                   int pos, char *name );
    void CreateDeclaratorType( IDLParseNode *node, IDLString scope, 
                               IDLString root, IDLString form,
                               char* name, char* extension );
    void CreateDeclaratorPosition( char *name, int pos );
    IDLString GetTypeDeclarator( IDLParseNode *node, IDLString scope, 
                                 IDLString root, IDLString form );
    void CreateArraySubGraph( IDLParseNode *node, int status );
    void CreateFunctionPointerSubGraph( IDLParseNode *node, IDLString scope, 
                                        IDLString root, IDLString form );
    int ExamineAttributes( IDLParseNode *node,  char *target );
    void CreateAttributes( IDLParseNode *node, IDLString cname );
    IDLString GetArrayDeclarator( IDLParseNode *node );
    IDLString GetArrayBound( IDLParseNode *node );
    IDLString GetConstExpression( IDLParseNode *node );
    IDLString GetAttribute( IDLParseNode *node, IDLString scope, 
                            IDLString root, IDLString form );
    IDLString GetParenthesisAttribute( IDLParseNode *node, IDLString scope, 
                                       IDLString root, IDLString form );
    IDLString GetTypes( IDLParseNode *node, IDLString scope, 
                        IDLString root, IDLString form );
    IDLString GetParameterDeclarator( IDLParseNode *node, IDLString scope, 
                                      IDLString root, IDLString form );
    IDLString GetFunctionPointerDeclarator( IDLParseNode *node,
                                            IDLString scope, 
                                            IDLString root, IDLString form );
    IDLString GetDeclarator( IDLParseNode *node, IDLString scope, 
                             IDLString root, IDLString form );
    IDLString GetTypeSpecification( IDLParseNode *node, IDLString scope, 
                                    IDLString root, IDLString form );
    int DetectPointerConversionStatus(  IDLParseNode *node );
    int DetectArrayConversionStatus(  IDLParseNode *node_1,
                                      IDLParseNode *node_2 );
    int DetectPointerMode( IDLParseNode *node );
    int CountArrayDimensions( IDLParseNode *node );
public:
    DCE2ConceptualGraph( ostream &o ) : IDL2ConceptualGraph(o) {mode = 0;};
    ~DCE2ConceptualGraph();
    void CreateConceptualGraph( IDLParseNode *node );
};

#endif
