
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

#include "idlparsenode.h"


IDLParseNode::IDLParseNode( IDLParseNodeType aType, IDLParseNode *one,
                            IDLParseNode *two, 
                            IDLParseNode *three, IDLParseNode *four,
                            IDLParseNode *five )
{
    type = aType;
    str = NULL;
    b1 = one;
    b2 = two;
    b3 = three;
    b4 = four;
    b5 = five;
}


IDLParseNode::~IDLParseNode()
{
    if( str ) delete str;
    if( b1 ) delete b1;
    if( b2 ) delete b2;
    if( b3 ) delete b3;
    if( b4 ) delete b4;
    if( b5 ) delete b5;
}


void IDLParseNode::SetString( string *aStr )
{
    if( str ) delete str;
    str = aStr;
}


string *IDLParseNode::GetString()
{
    return str;
}


IDLParseNodeType IDLParseNode::GetType()
{
    return type;
}


IDLParseNode *IDLParseNode::GetBranch( int i )
{
    switch( i ) {
    case 1:
        return b1;
    case 2:
        return b2;
    case 3:
        return b3;
    case 4:
        return b4;
    case 5:
        return b5;
    default:
        assert( 0 );
    }
}


void IDLParseNode::SetBranch( int i, IDLParseNode *node )
{
    switch( i ) {
    case 1:
        if( b1 ) delete b1;
        b1 = node;
        break;
    case 2:
        if( b2 ) delete b2;
        b2 = node;
        break;
    case 3:
        if( b3 ) delete b3;
        b3 = node;
        break;
    case 4:
        if( b4 ) delete b4;
        b4 = node;
        break;
    case 5:
        if( b5 ) delete b5;
        b5 = node;
        break;
    default:
        assert( 0 );
    }
}

