
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

#ifndef __IDLPARSENODE_H__
#define __IDLPARSENODE_H__

#include <assert.h>
#include <fstream.h>
#include "stl.h"
#include "idlstring.h"

#define MAX_NODE_SIZE 255

typedef enum 
{
    specification,
    module,
    forward_dcl,
    interface_dcl,
    interface_header,
    interface_body,
    scoped_name_list,
    scoped_name,
    const_dcl,
    const_expr,
    or_exp,
    xor_exp,
    and_exp,
    shift_exp,
    add_exp,
    mult_exp,
    unary_exp,
    unary_operator,
    primary_exp,
    literal,
    literal_list,
    type_dcl,
    type_declarator,
    element_declarator,
    member_declarator,
    declarator_list,
    declarator,
    base_type,
    struct_type,
    member_list,
    union_type,
    union_type_n_e,
    template_type,
    union_body,
    case_type,
    case_label,
    enum_type,
    array_bound,
    array_bound_list,
    attr_dcl,
    except_dcl,
    op_dcl,
    op_type,
    param_declarator_list,
    param_declarator,
    param_attribute,
    definition_list,
    attribute,
    array_declarator,
    attribute_list,
    ptr_declarator,
    importfile_list,
    importfile,
    interface_list,
    interface_attribute,
    attr_part,
    endpoint_list,
    endpoint,
    conditional_exp,
    logical_or_exp,
    logical_and_exp,
    inclusive_or_exp,
    exclusive_or_exp,
    equality_exp,
    relational_exp,
    field_declarator,
    union_switch,
    case_list,
    case_type_n_e,
    case_label_list,
    const_expr_list,
    switch_type_attr,
    pipe_type,
    attr_var,
    function_ptr_dcl
} IDLParseNodeType;


class IDLParseNode
{
 private:
    IDLParseNodeType type;
    string		*str;
    IDLParseNode	*b1;
    IDLParseNode	*b2;
    IDLParseNode	*b3;
    IDLParseNode	*b4;
    IDLParseNode	*b5;
 public:
    IDLParseNode( IDLParseNodeType aType, IDLParseNode *one = NULL,
                  IDLParseNode *two = NULL,
                  IDLParseNode *three = NULL, IDLParseNode *four = NULL,
                  IDLParseNode *five = NULL );
    ~IDLParseNode();
    IDLParseNodeType GetType();
    void SetString( string *aStr );
    string *GetString();
    IDLParseNode *GetBranch( int i );
    void SetBranch( int i, IDLParseNode *node );
};

#endif
