
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

#include "idlparser.h"
#undef yyFlexLexer
#define yyFlexLexer yyidlFlexLexer
#include <FlexLexer.h>



extern int idl_line_no;


static yyidlFlexLexer *lexer;

IDLParser *theIDLParser;



int yyidllex()
{
    return lexer->yylex();
}


IDLParser::IDLParser(istream &istr, char *filename, ostream &o) : ostr(o) 
{
    idl_lexFile = filename;
    lexer = new yyidlFlexLexer( &istr );
}


IDLParser::~IDLParser()
{
    delete lexer;
}


void IDLParser::Parse()
{
    theIDLParser = this;
    idl_line_no = 1;
    errorMsg = NULL;
    yyidlparse();
}


void IDLParser::SetError( char *file, char *err, int line )
{
    errorFile = file;
    errorMsg  = err;
    errorLine = line;
}

ostream& IDLParser::GetOutputStream()
{
    return ostr;
}


bool IDLParser::ErrorOccured()
{
    return errorMsg != NULL;
}


char *IDLParser::GetErrorFile()
{
    return errorFile;
}


char *IDLParser::GetErrorMsg()
{
    return errorMsg;
}


int IDLParser::GetErrorLine()
{
    return errorLine;
}

