/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#include <assert.h>
#include "idltransform.h"


CORBA2Template::CORBA2Template( ostream &o, IDLParseNode *node )
    : ostr(o)
{
    CORBA2ConceptualGraph *conv;
    ConceptualGraph2Template *hconv;
    ostrstream ostrstr;
    ConceptualGraph *cg;

    ostrstr << "@answer-graph=";
#if 0
    conv = new CORBA2ConceptualGraph(cout);
    conv->CreateConceptualGraph(node);
    delete conv;
    cout << endl;
    exit(0);
#else
    conv = new CORBA2ConceptualGraph(ostrstr);
    conv->CreateConceptualGraph(node);
    delete conv;
#endif
    ostrstr << ends;
    istrstream istrstr(ostrstr.str());
    cg = ConvertStream2ConceptualGraph(istrstr);
    ostrstr.rdbuf()->freeze(0);
    hconv = new ConceptualGraph2Template(cg);
    hconv->CreateDependencyGraph();
    hconv->CreateLeanStructure();
    hconv->Print(ostr);
    delete hconv;
}


ConceptualGraph* CORBA2Template::ConvertStream2ConceptualGraph( istream &i )
{
    Server *server;
    ConceptualGraph *cg;
    Parser *parser;

    server = new Server();
    parser = new Parser(server, i);
    parser->Parse();
    delete parser;
    server->GetAnswer(&cg); 
    assert(cg != NULL);
    return(cg);
}


CORBA2Template::~CORBA2Template()
{
}



DCE2Template::DCE2Template( ostream &o, IDLParseNode *node )
{
    DCE2ConceptualGraph *conv;

    conv = new DCE2ConceptualGraph(o);
    conv->CreateConceptualGraph(node);
    delete conv;
}


DCE2Template::~DCE2Template()
{
}
