
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version RELATION of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#include "printcg.h"


void CG2Stream::EnterNewLevel( CGLevelType l )
{
    switch(state)
    {
    case INITIAL:
    case CONCEPT:
    case RELATION:
	if(next == NULL)
            next = new CG2Stream(l, strout, depth + 2);
	state = PASS_ON;
	break;
    case PASS_ON:
	if(level == l)
            synonym_counter++;
	next->EnterNewLevel(l);
	break;
    default:
	assert(0);	
	break;
    }
}	
  

void CG2Stream::LeaveLevel( CGLevelType l )
{
    CGLevelType next_level;
    int counter;

    switch(state)
    {
    case PASS_ON:
	if(next != NULL) 
        {
	    next_level = next->GetLevel();
	    counter = next->GetSynonymCounter();
	    if(next_level == l)
            {
		if(!counter)
                {
	            delete next;
	            next = NULL;
	            state = mode + 1;
                }
	        else 
                {
		    next->LeaveLevel(l);
		    next->SetSynonymCounter(counter - 1);
                }
            }
	    else
                next->LeaveLevel(l);
        }
	break;
    default:
	assert(0);
	break;
    }
}

	
void CG2Stream::PrintRelation( const char *relation ) 
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ('";
	strout->PrintWord(sign);
	keyword = relation;
	strout->PrintWord(keyword);
	sign = "')";
	strout->PrintWord(sign);
	strout->PrintWhiteSpace(1);
	state = CONCEPT;
	break;	
    case CONCEPT:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ('";
	strout->PrintWord(sign);
	keyword = relation;
	strout->PrintWord(keyword);
	sign = "')";
	strout->PrintWord(sign);
	strout->PrintWhiteSpace(1);
	break;
    case RELATION:
	if(!mode)
        {
	    sign = ",";
	    strout->PrintLine(sign);
	    strout->PrintWhiteSpace(depth);
        }
	sign = "-> ('";
	strout->PrintWord(sign);
	keyword = relation;
	strout->PrintWord(keyword);
	sign = "')";
	strout->PrintWord(sign);
	strout->PrintWhiteSpace(1);
	state = CONCEPT;
	break;
    case PASS_ON:
	next->PrintRelation(relation);
	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintRoot()
{
    IDLString word;

    word = "['SPECIFICATION'] -> ('DEFINITIONS') ";
    strout->PrintWord(word);
}

	
void CG2Stream::PrintPureConcept( const char *concept )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "']";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "']";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "']";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintPureConcept(concept);
     	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintConcept( const char *concept )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = FIRST_INSTANCE;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{";
	strout->PrintWord(sign);
	state = FIRST_INSTANCE;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{";
	strout->PrintWord(sign);
	state = FIRST_INSTANCE;
 	break;
    case PASS_ON:
 	next->PrintConcept(concept);
     	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintConcept( const char *con1, const char *con2, int value )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = con1;
	strout->PrintWord(keyword);
	keyword = con2;
	strout->PrintWord(keyword);
	sign = "':";
	strout->PrintWord(sign);
	strout->PrintValue(value);
	sign = "]";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = con1;
	strout->PrintWord(keyword);
	keyword = con2;
	strout->PrintWord(keyword);
	sign = "':";
	strout->PrintWord(sign);
	strout->PrintValue(value);
	sign = "]";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = con1;
	strout->PrintWord(keyword);
	keyword = con2;
	strout->PrintWord(keyword);
	sign = "':";
	strout->PrintWord(sign);
	strout->PrintValue(value);
	sign = "]";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintConcept(con1, con2, value);
     	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintConcept( const char *concept, int value )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':";
	strout->PrintWord(sign);
	strout->PrintValue(value);
	sign = "]";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':";
	strout->PrintWord(sign);
	strout->PrintValue(value);
	sign = "]";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':";
	strout->PrintWord(sign);
	strout->PrintValue(value);
	sign = "]";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintConcept(concept, value);
     	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintConcept( int nr, IDLString instance )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	strout->PrintValue(nr);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	strout->PrintValue(nr);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	strout->PrintValue(nr);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintConcept(nr, instance);
	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintConcept( const char *concept, IDLString instance, int nr )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	strout->PrintValue(nr);
	sign = "'}]";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	strout->PrintValue(nr);
	sign = "'}]";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	strout->PrintValue(nr);
	sign = "'}]";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintConcept(concept, instance, nr);
	break;
    default: 
	assert(0);
	break;
    }
}


void CG2Stream::PrintConcept( const char *concept, IDLString instance )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintConcept(concept, instance);
	break;
    default:
	assert(0);
    }
}


void CG2Stream::PrintConcept( const char *concept, const char *instance )
{
    IDLString keyword;
    IDLString sign;

    switch(state)
    {
    case INITIAL:
	sign = "-";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	keyword = instance;
	strout->PrintWord(keyword);
	sign = "'}]";
	strout->PrintWord(sign);
	mode = CONCEPT;
	state = RELATION;
	break;
    case CONCEPT:
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	keyword = instance;
	strout->PrintWord(keyword);
	sign = "'}]";
	strout->PrintWord(sign);
	state = RELATION;
	break;
    case RELATION:
	sign = ",";
	strout->PrintLine(sign);
	strout->PrintWhiteSpace(depth);
	sign = "-> ['";
	strout->PrintWord(sign);
	keyword = concept;
	strout->PrintWord(keyword);
	sign = "':{'";
	strout->PrintWord(sign);
	keyword = instance;
	strout->PrintWord(keyword);
	sign = "'}]";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintConcept(concept, instance);
	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintInstance( IDLString instance )
{
    IDLString sign;
    
    switch(state)
    {
    case FIRST_INSTANCE:
	sign = "'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'";
	strout->PrintWord(sign);
	state = NEXT_INSTANCE;
	break;
    case NEXT_INSTANCE:
	sign = ",";
	strout->PrintWord(sign);
	strout->PrintWhiteSpace(1);
	sign = "'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'";
	strout->PrintWord(sign);
 	break;
    case PASS_ON:
 	next->PrintInstance(instance);
	break;
    default:
	assert(0);
	break;
    }
}


void CG2Stream::PrintLastInstance( IDLString instance )
{
    IDLString sign;

    switch(state)
    {
    case FIRST_INSTANCE:
	sign = "'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
	state = RELATION;
 	break;
    case NEXT_INSTANCE:
	sign = ",";
	strout->PrintWord(sign);
	strout->PrintWhiteSpace(1);
	sign = "'";
	strout->PrintWord(sign);
	strout->PrintWord(instance);
	sign = "'}]";
	strout->PrintWord(sign);
	state = RELATION;
 	break;
    case PASS_ON:
 	next->PrintLastInstance(instance);
	break;
    default:
	assert(0);
	break;
    }
}


