
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

#ifndef __PRINTCG_H__
#define __PRINTCG_H__

#include <fstream.h>
#include <assert.h>
#include "idlstring.h"
#include "print.h"

#define INITIAL 0
#define CONCEPT 1
#define RELATION 2 
#define FIRST_INSTANCE 3 
#define NEXT_INSTANCE 4
#define PASS_ON 5


typedef enum
{
    specification_level,
    module_level,
    definition_level_list,
    interface_level,
    interface_level_list,
    operation_level,
    operation_level_list,
    type_level,
    type_level_list,
    constant_level,
    constant_level_list,
    bound_level_list,
    attribute_level_list,
    member_level_list,
    member_level,
    case_level_list,
    declarator_level,
    annotation_level_list,
    parameter_level_list,
    parameter_level,
    template_type_level,
    constructed_type_level
} CGLevelType;


class CG2Stream
{
private:
    String2Stream *strout;
    CG2Stream *next;
    CGLevelType level;
    int synonym_counter;
    int depth;
    int mode;
    int state;
    CGLevelType GetLevel() { return(level); }
    int GetSynonymCounter() { return(synonym_counter); }
    void SetSynonymCounter( int counter ) { synonym_counter = counter; }
    CG2Stream( CGLevelType l, String2Stream *s, int d )
    {
        strout = s;
        level = l;
        next = NULL;
        depth = d;
        synonym_counter = 0;
        state = INITIAL;
        mode = INITIAL;
    }
public:
    CG2Stream( ostream &o ) 
    {
        IDLString word;
        
        strout = new String2Stream(o);
        level = specification_level;
        next = NULL;
        depth = 1;
        synonym_counter = 0;
        state = INITIAL;
        mode = INITIAL;    
    }
    ~CG2Stream() 
    {
        IDLString sign;
        
        if(next) delete next;
        sign = ".";
        strout->PrintWord(sign); 
        if(level == specification_level) delete strout;
    }
    void PrintRoot(); 
    void PrintRelation( const char *relation );
    void PrintPureConcept( const char *concept );
    void PrintConcept( const char *concept );
    void PrintConcept( const char *concept, const char *instance );
    void PrintConcept( int nr, IDLString instance );
    void PrintConcept( const char *concept, IDLString instance );
    void PrintConcept( const char *concept, IDLString instance, int nr );
    void PrintConcept( const char *con1, const char *con2, int value );
    void PrintConcept( const char *concept, int value );
    void PrintInstance( IDLString instance );
    void PrintLastInstance( IDLString instance );
    void EnterNewLevel( CGLevelType l );
    void LeaveLevel( CGLevelType l );

    void PrintRelation( const string &relation )
    {
        PrintRelation (relation.c_str());
    }
    void PrintPureConcept( const string &concept )
    {
        PrintPureConcept (concept.c_str());
    }
    void PrintConcept( const string &concept )
    {
        PrintConcept (concept.c_str());
    }
    void PrintConcept( const string &concept, const string &instance )
    {
        PrintConcept (concept.c_str(), instance.c_str());
    }
    /* ambigous in egcs 1.x
    void PrintConcept( const string &concept, IDLString instance )
    {
        PrintConcept (concept.c_str(), instance);
    }
    void PrintConcept( const string &concept, IDLString instance, int nr )
    {
        PrintConcept (concept.c_str(), instance, nr);
    }
    */
    void PrintConcept( const string &con1, const string &con2, int value )
    {
        PrintConcept (con1.c_str(), con2.c_str(), value);
    }
    void PrintConcept( const string &concept, int value )
    {
        PrintConcept (concept.c_str(), value);
    }
};

#endif
