/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

#ifndef __PRINTIDL_H__
#define __PRINTIDL_H__

#include <fstream.h>
#include <assert.h>
#include "idlstring.h"
#include "print.h"


#define INITIAL 0
#define IDENTIFIER 1
#define MEMBER 2
#define TYPE 3
#define PARAMETER 4
#define CASE 5
#define TAG 6
#define SWITCH 7
#define EXPRESSION 8
#define FINAL 9
#define INHERITANCE 10
#define ATTRIBUTE 11


typedef enum
{
    root_level,
    interface_level,
    forward_level,
    module_level,
    inheritance_level,
    type_level,
    attribute_level,
    constant_level,
    operation_level,
    exception_level,
    raises_level,
    context_level,
    signature_level,
    struct_level,
    union_level,
    enum_level,
    pipe_level,
    member_level,
    parameter_level,
    switch_level,
    sequence_level,
    string_level,
    case_level,
    case_n_e_level,
    type_dcl_level,
} IDLLevelType;


class DCE2Stream
{
private:
    IDLLevelType level;
    String2Stream *strout;
    int state;
    int synonym_counter;
    int depth;
    DCE2Stream *next;
public:
    DCE2Stream( ostream &o, int d );
    DCE2Stream( String2Stream *s, IDLLevelType l, int d );
    ~DCE2Stream();
    void EnterNewLevel( IDLLevelType newlevel );
    void LeaveLevel( IDLLevelType deletelevel );
    void PrintIdentifier( IDLString identifier );
    void PrintType( IDLString type );
    void PrintAttribute( IDLString attribute );
    void PrintExpression( IDLString expression );
    int GetSynonymCounter() { return(synonym_counter); }
    IDLLevelType GetLevel() { return(level); }
    void SetSynonymCounter( int sc ) { synonym_counter = sc; }
};


class CORBA2Stream
{
private:
    IDLLevelType level;
    String2Stream *strout;
    int state;
    int synonym_counter;
    int depth;
    CORBA2Stream *next;
public:
    CORBA2Stream( ostream &o, int d );
    CORBA2Stream( String2Stream *s, IDLLevelType l, int d );
    ~CORBA2Stream();
    void EnterNewLevel( IDLLevelType newlevel );
    void LeaveLevel( IDLLevelType deletelevel );
    void PrintIdentifier( IDLString identifier );
    void PrintType( IDLString type );
    void PrintAttribute( IDLString attribute );
    void PrintExpression( IDLString expression );
    int GetSynonymCounter() { return(synonym_counter); }
    IDLLevelType GetLevel() { return(level); }
    void SetSynonymCounter( int sc ) { synonym_counter = sc; }
};



#endif
