/* -*- c++ -*- */
/****************************************************************************/
/*	File:	scanner.l	Author:		Carsten Kaiser		    */
/*	Date: 	28.10.95 	Version:	2.0			    */
/*									    */
/*	Dieses File beschreibt die Regeln fuer die lexikalische Analyse     */
/*      einer Schnittstellen-Spezifikation.				    */
/****************************************************************************/

%{
#include <iostream.h>
#include <fstream.h>
#include "stl.h"
#include <stdio.h>

#include "idlparsenode.h"
#include "yacc.tab.h"
#include "util.h"

#define MAX_STR_CONST 255
#define MAX_IMPORT_DEPTH 50

YY_BUFFER_STATE	idl_import_stack[MAX_IMPORT_DEPTH];
char		*idl_import_file_name[MAX_IMPORT_DEPTH];
int 		idl_import_stack_ptr = 0;

char 		idl_string_buf[MAX_STR_CONST];
char 		*idl_string_buf_ptr;
char		*idl_lexFile;
int		idl_line_no = 1;
%}

%option c++
%option prefix="yyidl"
%option debug

/*---  Sowohl von DCE als auch von CORBA benoetigte 'start conditions' -----*/
%x COMMENT
%x STR 
/*--------------------------------------------------------------------------*/


/*--------------------  CORBA 'start conditions' ---------------------------*/
%x ONELINECOMMENT
/*--------------------------------------------------------------------------*/


/*-------------------  DCE 'start conditions' ------------------------------*/
%x ENDPT
%x VERSION_NR
%x IMPORT
/*--------------------------------------------------------------------------*/


/*------------  Gemeinsam benoetigte 'name definitions'  -------------------*/
Int10_Literal		[1-9][0-9]*
Int8_Literal		0[1-7]*
Int16_Literal		(0x|0X)[A-F0-9]*
Char_Literal		"'".*"'"
String_Literal		\"[^\n\"]*\"
Float_Literal1		{Int10_Literal}"."{Int10_Literal}(e|E)("+"|"-")?{Int10_Literal}(double|float)?  
Float_Literal2		{Int10_Literal}(e|E)("+"|"-")?{Int10_Literal}(double|float)?  
Float_Literal3		"."{Int10_Literal} 
Float_Literal4		"."{Int10_Literal}(e|E)("+"|"-")?{Int10_Literal}(double|float)?  
/*--------------------------------------------------------------------------*/


/*------------------  CORBA 'name definitions'  ----------------------------*/
CORBA_Identifier	[a-zA-Z][a-zA-Z0-9_]*
/*--------------------------------------------------------------------------*/


/*-----------------  DCE 'name definitions'  -------------------------------*/
DCE_Identifier		[a-zA-Z_][a-zA-Z0-9_]{0,30}
AsteriskIdentifier	[*]+{DCE_Identifier} 
Version_Nr		{Int10_Literal}|{Int10_Literal}"."[0-9]*
Uuid_Nr			[0-9a-f]{8}"-"([0-9a-f]{4}"-"){3}[0-9a-f]{12}
Digit			[0-9]*
/*--------------------------------------------------------------------------*/


%%

[ \t]					;
[\n]					idl_line_no++;


"//"					BEGIN(ONELINECOMMENT); 
<ONELINECOMMENT>{
		 .			;
		 \n			{
					 idl_line_no++;
					 BEGIN(INITIAL);
					}
		}					

"/*"					BEGIN(COMMENT);
<COMMENT>{
	  [^*\n]*			;
	  "*"+[^*/\n]*			;
	  \n				idl_line_no++;
	  "*"+"/"			BEGIN(INITIAL);
	 }

endpoint				{
					BEGIN(ENDPT);
					return ENDPOINT;
					}
<ENDPT>{
	"("				return PARENTLEFT;
	\"				return DOUBLEQUOTE;
	ncacn_ip_tcp			return NCACN_IP_TCP;
	ncadg_ip_udp			return NCADG_IP_UDP;
	":"				return COLON;
	"["				return SQUAREBRACELEFT;
	"]"				return SQUAREBRACERIGHT;
	{Digit}				{
					 yyidllval.int_literal = new string(YYText());
					 return INTEGER_LITERAL;
					}
	","				return COMMA;
	")"				{
					BEGIN(INITIAL);
					return PARENTRIGHT;
					}
	.				/* error ("Bad Endpoint")*/;
	}

version					{
					BEGIN(VERSION_NR);
					return VERSION;
					}
<VERSION_NR>{
	     "("			return PARENTLEFT;
	     {Version_Nr} 		{
					 yyidllval.float_literal = new string(YYText());
					 return FLOATING_PT_LITERAL;
					}
	     ")"			{
					BEGIN(INITIAL);
					return PARENTRIGHT;
					}
	     .				/* error ("Bad Version_Nr.") */
	    }

import					{
					idl_string_buf_ptr = idl_string_buf;
					BEGIN(IMPORT);
					return IMPORTFILE;
					}
<IMPORT>{
	 idl { 
             yyidllval.str_literal = new string(idl_string_buf);
             cat(*yyidllval.str_literal, "idl", *yyidllval.str_literal);
             *idl_string_buf_ptr++ = 'i';
             *idl_string_buf_ptr++ = 'd';
             *idl_string_buf_ptr++ = 'l';
             if (idl_import_stack_ptr >= MAX_IMPORT_DEPTH)
             {
                 cerr << "Imports nested too deeply";
                 exit(1); 
             }
             idl_import_file_name[idl_import_stack_ptr++] = idl_string_buf;
             return STRING_LITERAL;	
         }
	 \"				return DOUBLEQUOTE;
	 [a-zA-Z0-9_]+ {
             const char *yptr = YYText();
             while (*yptr)
                 *idl_string_buf_ptr++ = *yptr++;
         }
	 "."				*idl_string_buf_ptr++ = '.';
	 ","				return COMMA;
	 ";" {
             BEGIN(INITIAL);
             return SEMICOLON;
         }
}

<<EOF>> {
    if (--idl_import_stack_ptr < 0)
    {
	idl_import_stack_ptr = 0;
        yyterminate();
    }
    else
    {
        ifstream *file;
        file = new ifstream(idl_import_file_name[idl_import_stack_ptr]); 
        if (!(*file))
        {
            cerr << "Couldn't open import file `"
                 << idl_import_file_name[idl_import_stack_ptr] << "'" << endl;
            exit(1);
        }
        yy_delete_buffer(YY_CURRENT_BUFFER);
        yy_switch_to_buffer(yy_create_buffer(file, YY_BUF_SIZE));
        /*free(idl_lexFile);*/
        idl_lexFile = idl_import_file_name[idl_import_stack_ptr];
        idl_line_no = 1;
        BEGIN(INITIAL);
        return IMPORTFILE;
    }
}

"{"					return CURLYBRACELEFT;
"}"					return CURLYBRACERIGHT;
"["		 			return SQUAREBRACELEFT;
"]"					return SQUAREBRACERIGHT;
"("					return PARENTLEFT;
")"					return PARENTRIGHT;
":"					return COLON;
","					return COMMA;
";"					return SEMICOLON;
"="					return EQUAL;
">>"					return SHIFTRIGHT;
"<<"					return SHIFTLEFT;
"+"					return PLUS;
"-"					return MINUS;
"*"					return ASTERISK;
"/"					return SOLIDUS;
"%"					return PERCENT;
"~"					return TILDE;
"|"					return VERTICAL;
"^"					return CIRCUMFLEX;
"&"					return AMPERSAND;
"<"					return LESSTHAN;
">"					return GREATERTHAN;

const					return CONST;
typedef					return TYPEDEF;
float					return FLOAT;
double					return DOUBLE;
char					return CHAR;
boolean					return BOOLEAN;
string					return STRING;
void					return VOID;
unsigned				return NOSIGN;
long 					return LONG;
short					return SHORT;
FALSE					return FALSE;
TRUE					return TRUE;
struct					return STRUCT;
union					return UNION;
switch					return SWITCH;
case					return CASE;
default					return DEFAULT;
enum					return ENUM;
in					return IN;
out					return OUT;
interface				return INTERFACE;


">="					return GREATERTHANEQUAL;
"<="					return LESSTHANEQUAL;
"!"					return QUESTIONMARK;
"||"					return DOUBLEVERTICAL;
"&&"					return DOUBLEAMPERSAND;
"=="					return DOUBLEEQUAL;
"!="					return QUESTIONMARKEQUAL;
".."					return DOUBLEPERIOD;

ISO_LATIN_1				return ISO_LATIN_1;
ISO_MULTI_LINGUAL			return ISO_MULTI_LINGUAL;
ISO_UCS					return ISO_UCS;
shape					return SHAPE;
idempotent				return IDEMPOTENT;
broadcast				return BROADCAST;
switch_type				return SWITCH_TYPE;
switch_is				return SWITCH_IS;
maybe					return MAYBE;
pipe					return PIPE;
transmit_as				return TRANSMIT_AS;
handle					return HANDLE;
align					return ALIGN;
ignore 					return IGNORE;
context_handle				return CONTEXT_HANDLE;
unique					return UNIQUE;
full					return FULL;
max_is					return MAX_IS;
size_is					return SIZE_IS;
first_is				return FIRST_IS;
last_is					return LAST_IS;
length_is 				return LENGTH_IS;
int					return INT;
byte					return BYTE;
handle_t				return HANDLE_T;
error_status_t				return ERROR_STATUS_T;
small					return SMALL;
hyper					return HYPER;
unsigned[ \t]*int			return NOSIGN_INT;
void"*"					return VOIDPTR;
char"*"					return CHARPTR;
NULL					return NULLPTR;
v1_array				return V1_ARRAY;
v1_string				return V1_STRING;
v1_struct				return V1_STRUCT;
v1_enum					return V1_ENUM;
ref					return REF;
ptr					return PTR;
uuid					return UUID;
pointer_default				return PT_DEFAULT;
local					return LOCAL;


"::"					return SCOPE; 

module					return MODULE;
octet					return OCTET;
any					return ANY;
sequence				return SEQUENCE;
readonly				return READONLY;
attribute				return ATTRIBUTE;
exception				return EXCEPTION;
oneway					return ONEWAY;
inout					return INOUT;
raises					return RAISES;
context					return CONTEXT;
Object					return OBJECT;
TypeCode				return TYPECODE;

"["/[ \t]*default			return SQUAREBRACELEFT_DEFAULT;

{AsteriskIdentifier}			{
					 yyidllval.asteriskidentifier = new string(YYText()); 
					 return ASTERISKIDENTIFIER;
					}
{CORBA_Identifier}			|
{DCE_Identifier}			{
					 yyidllval.identifier = new string(YYText());
					 return IDENTIFIER;
					}
{Float_Literal1}			|
{Float_Literal2}			|
{Float_Literal3}			|
{Float_Literal4}			{
					 yyidllval.float_literal = new string(YYText());
					 return FLOATING_PT_LITERAL;
					}
{Int10_Literal}				| 
{Int8_Literal}				|
{Int16_Literal}				{
					 yyidllval.int_literal = new string(YYText());
					 return INTEGER_LITERAL;
					}
{Char_Literal}				{ 
					 yyidllval.char_literal = new string(YYText());
					 return CHARACTER_LITERAL;
					}
{String_Literal}			{
					 yyidllval.str_literal = new string(YYText());
					 del (*yyidllval.str_literal, "\"");
					 del (*yyidllval.str_literal, "\"", yyleng-2);
					 return STRING_LITERAL;
					}
{Uuid_Nr}				{
					 yyidllval.str_literal = new string(YYText());
					 return STRING_LITERAL;
					}


%%
