// -*- c++ -*-
%{
#include <iostream.h>
#include "idlparser.h"
#include "idlparsenode.h"
#include "cgtransform.h"
#include "idltransform.h"
#include "idlstring.h"
#include "util.h"

extern int idl_line_no;
extern char* idl_lexFile;
extern int yyidllex();

void yyerror(char *s)
    {
        theIDLParser->SetError(idl_lexFile, s, idl_line_no);
    }

DCE2Template* dce_converter;
CORBA2Template* corba_converter;
char*	help_cp;
string* help_sp_1;
string* help_sp_2;
%}

%start specification 

%union{
    string	 *identifier;
    string	 *asteriskidentifier;
    string	 *str_literal;
    string	 *int_literal;
    string 	 *char_literal;
    string	 *float_literal;
    IDLParseNode *idlpn;
}


/********  Definition der gemeinsam benutzten Terminalsymbole ********/

%token INTERFACE
%token CURLYBRACELEFT
%token CURLYBRACERIGHT
%token SQUAREBRACELEFT
%token SQUAREBRACERIGHT
%token PARENTLEFT
%token PARENTRIGHT
%token COMMA
%token COLON
%token SEMICOLON
%token DOUBLEQUOTE
%token EQUAL 
%token SHIFTLEFT
%token SHIFTRIGHT
%token PLUS
%token MINUS
%token ASTERISK
%token SOLIDUS
%token PERCENT
%token TILDE
%token VERTICAL
%token CIRCUMFLEX
%token AMPERSAND
%token LESSTHAN
%token GREATERTHAN
%token CONST
%token TYPEDEF
%token FLOAT
%token DOUBLE
%token CHAR
%token BOOLEAN
%token STRING
%token VOID
%token ANY
%token OBJECT
%token TYPECODE
%token NOSIGN
%token LONG
%token SHORT
%token TRUE
%token FALSE
%token STRUCT
%token UNION
%token SWITCH
%token CASE
%token DEFAULT
%token ENUM
%token IN
%token OUT
%token<str_literal>	STRING_LITERAL 
%token<int_literal>	INTEGER_LITERAL
%token<char_literal>	CHARACTER_LITERAL 
%token<float_literal>	FLOATING_PT_LITERAL 
%token<identifier> IDENTIFIER


/********  Definition der von DCE zusaetzlich benoetigten Terminalsymbole  ********/ 

%token IMPORTFILE
%token ISO_LATIN_1
%token ISO_MULTI_LINGUAL
%token ISO_UCS
%token SHAPE
%token BROADCAST
%token IDEMPOTENT
%token SWITCH_TYPE
%token SWITCH_IS
%token MAYBE
%token PIPE
%token TRANSMIT_AS
%token HANDLE
%token ALIGN
%token IGNORE
%token CONTEXT_HANDLE
%token<asteriskidentifier> ASTERISKIDENTIFIER
%token UNIQUE
%token FULL
%token FIRST_IS
%token LAST_IS
%token LENGTH_IS
%token MAX_IS
%token SIZE_IS
%token LESSTHANEQUAL
%token GREATERTHANEQUAL
%token QUESTIONMARK
%token DOUBLEVERTICAL
%token DOUBLEAMPERSAND
%token DOUBLEEQUAL
%token QUESTIONMARKEQUAL
%token DOUBLEPERIOD
%token INT
%token BYTE
%token HANDLE_T
%token ERROR_STATUS_T
%token SMALL
%token HYPER
%token NOSIGN_INT
%token VOIDPTR 
%token CHARPTR 
%token NULLPTR
%token V1_STRUCT
%token V1_ENUM
%token V1_STRING
%token V1_ARRAY
%token NCACN_IP_TCP
%token NCADG_IP_UDP
%token PTR
%token REF
%token VERSION
%token PT_DEFAULT
%token ENDPOINT
%token LOCAL
%token UUID

%token SQUAREBRACELEFT_DEFAULT

/********  Definition der von CORBA zusaetzlich benoetigten Terminalsymbole  ********/

%token MODULE 
%token SCOPE 
%token OCTET 
%token SEQUENCE 
%token READONLY	 
%token ATTRIBUTE 
%token EXCEPTION 
%token ONEWAY 
%token INOUT 
%token RAISES 
%token CONTEXT 

/********  Definition des Rueckgabetyps fuer nichtterminale Symbole der CORBA-Grammatik ********/

%type<idlpn> specification
%type<idlpn> c_definition 
%type<idlpn> c_module
%type<idlpn> c_definitions
%type<idlpn> c_interface_dcl
%type<idlpn> c_forward_dcl
%type<idlpn> c_interface
%type<idlpn> c_interface_header
%type<idlpn> c_interface_body
%type<idlpn> c_exports
%type<idlpn> c_export
%type<idlpn> c_inheritance_spec
%type<idlpn> c_scoped_names
%type<idlpn> c_scoped_name
%type<idlpn> c_const_dcl
%type<idlpn> c_const_expr
%type<idlpn> c_const_type
%type<idlpn> c_or_exp
%type<idlpn> c_xor_exp
%type<idlpn> c_and_exp
%type<idlpn> c_shift_exp
%type<idlpn> c_add_exp 
%type<idlpn> c_mult_exp
%type<idlpn> c_unary_exp 
%type<idlpn> c_unary_operator 
%type<idlpn> c_primary_exp 
%type<idlpn> c_literal  
%type<idlpn> c_boolean_literal 
%type<idlpn> c_positive_int_const
%type<idlpn> c_type_dcl
%type<idlpn> c_type_declarator 
%type<idlpn> c_type_spec
%type<idlpn> c_simple_type_spec
%type<idlpn> c_base_type_spec
%type<idlpn> c_template_type_spec 
%type<idlpn> c_constr_type_spec 
%type<idlpn> c_declarators 
%type<idlpn> c_declarator 
%type<idlpn> c_simple_declarator  
%type<idlpn> c_complex_declarator 
%type<idlpn> c_floating_pt_type 
%type<idlpn> c_integer_type 
%type<idlpn> c_signed_int 
%type<idlpn> c_unsigned_int 
%type<idlpn> c_signed_long_int 
%type<idlpn> c_signed_short_int 
%type<idlpn> c_unsigned_long_int 
%type<idlpn> c_unsigned_short_int 
%type<idlpn> c_char_type  
%type<idlpn> c_boolean_type 
%type<idlpn> c_octet_type
%type<idlpn> c_any_type
%type<idlpn> c_object_type
%type<idlpn> c_typecode_type
%type<idlpn> c_struct_type
%type<idlpn> c_member_list 
%type<idlpn> c_member 
%type<idlpn> c_union_type 
%type<idlpn> c_switch_type_spec
%type<idlpn> c_switch_body
%type<idlpn> c_case
%type<idlpn> c_case_label
%type<idlpn> c_element_spec
%type<idlpn> c_enum_type
%type<idlpn> c_enumerators
%type<idlpn> c_enumerator
%type<idlpn> c_sequence_type 
%type<idlpn> c_string_type
%type<idlpn> c_array_declarator
%type<idlpn> c_fixed_array_sizes
%type<idlpn> c_fixed_array_size 
%type<idlpn> c_attr_dcl
%type<idlpn> c_except_dcl
%type<idlpn> c_members
%type<idlpn> c_op_dcl
%type<idlpn> c_op_attribute
%type<idlpn> c_op_type_spec
%type<idlpn> c_parameter_dcls
%type<idlpn> c_param_dcls
%type<idlpn> c_param_dcl
%type<idlpn> c_param_attribute
%type<idlpn> c_raises_expr 
%type<idlpn> c_context_expr
%type<idlpn> c_string_literals
%type<idlpn> c_string_literal
%type<idlpn> c_integer_literal
%type<idlpn> c_floating_pt_literal
%type<idlpn> c_character_literal

/********  Definition des Rueckgabetyps fuer nichtterminale Symbole der DCE-Grammatik  *********/

%type<idlpn> d_definition
%type<idlpn> d_importfiles
%type<idlpn> d_importfile
%type<idlpn> d_interface
%type<idlpn> d_interface_header
%type<idlpn> d_interface_attribute
%type<idlpn> d_interface_attributes
%type<idlpn> d_uuid_rep
%type<idlpn> d_version_number
%type<idlpn> d_endpoint_specs
%type<idlpn> d_endpoint_spec
%type<idlpn> d_family_string
%type<idlpn> d_endpoint_string
%type<idlpn> d_pointer_attribute
%type<idlpn> d_interface_body
%type<idlpn> d_imports
%type<idlpn> d_import
%type<idlpn> d_filenames
%type<idlpn> d_filename
%type<idlpn> d_exports
%type<idlpn> d_export
%type<idlpn> d_const_declaration
%type<idlpn> d_const_type_spec
%type<idlpn> d_integer_type
%type<idlpn> d_signed_int
%type<idlpn> d_unsigned_int
%type<idlpn> d_int_size
%type<idlpn> d_const_exp
%type<idlpn> d_integer_const_exp
%type<idlpn> d_conditional_exp
%type<idlpn> d_logical_or_exp
%type<idlpn> d_logical_and_exp
%type<idlpn> d_inclusive_or_exp
%type<idlpn> d_exclusive_or_exp
%type<idlpn> d_and_exp
%type<idlpn> d_equality_exp 
%type<idlpn> d_relational_exp 
%type<idlpn> d_shift_exp
%type<idlpn> d_additive_exp
%type<idlpn> d_multiplicative_exp
%type<idlpn> d_unary_exp
%type<idlpn> d_primary_exp
%type<idlpn> d_character_const
%type<idlpn> d_string_const
%type<idlpn> d_type_declaration
%type<idlpn> d_type_attr
%type<idlpn> d_type_attributes
%type<idlpn> d_type_spec
%type<idlpn> d_simple_type_spec
%type<idlpn> d_declarators
%type<idlpn> d_declarator
%type<idlpn> d_simple_declarator
%type<idlpn> d_complex_declarator
%type<idlpn> d_tagged_declarator
%type<idlpn> d_base_type_spec
%type<idlpn> d_floating_type
%type<idlpn> d_char_type
%type<idlpn> d_boolean_type
%type<idlpn> d_byte_type
%type<idlpn> d_any_type
%type<idlpn> d_void_type
%type<idlpn> d_handle_type
%type<idlpn> d_constructed_type_spec
%type<idlpn> d_tagged_struct_declarator
%type<idlpn> d_struct_type
%type<idlpn> d_tagged_struct
%type<idlpn> d_tag
%type<idlpn> d_member_list
%type<idlpn> d_member
%type<idlpn> d_field_declarator
%type<idlpn> d_field_attribute_list
%type<idlpn> d_field_attributes
%type<idlpn> d_tagged_union_declarator
%type<idlpn> d_union_type
%type<idlpn> d_union_switch
%type<idlpn> d_switch_type_spec
%type<idlpn> d_tagged_union
%type<idlpn> d_union_name
%type<idlpn> d_union_body
%type<idlpn> d_union_body_n_e
%type<idlpn> d_union_cases
%type<idlpn> d_union_case
%type<idlpn> d_union_case_n_e
%type<idlpn> d_union_cases_n_e
%type<idlpn> d_union_case_labels
%type<idlpn> d_union_case_label
%type<idlpn> d_union_case_label_n_e
%type<idlpn> d_const_exps
%type<idlpn> d_default_case
%type<idlpn> d_default_case_n_e 
%type<idlpn> d_union_arm 
%type<idlpn> d_union_type_switch_attr 
%type<idlpn> d_union_instance_switch_attr 
%type<idlpn> d_enumeration_type 
%type<idlpn> d_identifiers 
%type<idlpn> d_identifier 
%type<idlpn> d_pipe_type
%type<idlpn> d_array_declarator
%type<idlpn> d_array_bounds_list
%type<idlpn> d_array_bounds_declarator 
%type<idlpn> d_array_bounds_pair
%type<idlpn> d_array_bound 
%type<idlpn> d_type_attribute
%type<idlpn> d_xmit_type
%type<idlpn> d_usage_attribute 
%type<idlpn> d_field_attribute
%type<idlpn> d_attr_var_list
%type<idlpn> d_attr_var
%type<idlpn> d_ptr_declarator
%type<idlpn> d_ptr_attr 
%type<idlpn> d_op_declaration
%type<idlpn> d_operation_attributes
%type<idlpn> d_op_attributes
%type<idlpn> d_operation_attribute
%type<idlpn> d_param_declarators
%type<idlpn> d_param_declarator
%type<idlpn> d_param_dcls
%type<idlpn> d_param_attributes
%type<idlpn> d_directional_attribute
%type<idlpn> d_param_attrs
%type<idlpn> d_param_attribute
%type<idlpn> d_predefined_type_spec
%type<idlpn> d_function_ptr_declarator 
%type<idlpn> d_international_character_type
%type<idlpn> d_pipe_declarator
%type<idlpn> d_pipe_declarators 
 
%%
/********  Allgemeine Grammatik-Regeln  ********/

specification
	:	d_definition
		{
		 $$ = new IDLParseNode(specification, $1);
		 dce_converter = new DCE2Template(theIDLParser->GetOutputStream(), $$);
		 delete dce_converter;
		}
	|	c_definitions
		{
		 $$ = new IDLParseNode(specification, $1);
		 corba_converter = new CORBA2Template(theIDLParser->GetOutputStream(), $$);
		 delete corba_converter;
		}
	;

	
/********  CORBA Grammatik-Regeln  ********/

c_definitions	
	:	 c_definition
		{		
		 $$ = $1;
		}
	|	 c_definition c_definitions 
		{
		 $$ = new IDLParseNode(definition_list, $1, $2);		
		}
	;
c_definition	
	:	 c_type_dcl SEMICOLON
		{		
		 $$ = $1;
		}
	|	 c_const_dcl SEMICOLON
		{			
		 $$ = $1;
		}
	|	 c_except_dcl SEMICOLON
		{		
		 $$ = $1;
		}
	|	 c_interface_dcl SEMICOLON
		{		
		 $$ = $1;
		}
	|	 c_module SEMICOLON
		{
		 $$ = $1;
		}
	;

c_module		
	:	 MODULE IDENTIFIER CURLYBRACELEFT c_definitions CURLYBRACERIGHT 
		{
		 $$ = new IDLParseNode(module, $4);
		 $$->SetString($2);
		}
	; 

c_interface_dcl	
	:	 c_interface 
		{
		 $$ = $1;		
		}
	|	 c_forward_dcl 
		{
		 $$ = $1;		
		}
	;

c_forward_dcl	
	:	 INTERFACE IDENTIFIER 
		{
		 $$ = new IDLParseNode(forward_dcl);
		 $$->SetString($2);	
		}
	;

c_interface	
	:	 c_interface_header CURLYBRACELEFT c_interface_body CURLYBRACERIGHT 
		{		
		 $$ = new IDLParseNode(interface_dcl, $1, $3);
		}
	;

c_interface_header 
	:	 INTERFACE IDENTIFIER
		{		
		 $$ = new IDLParseNode(interface_header);
		 $$->SetString($2);
		}		 
	|	 INTERFACE IDENTIFIER c_inheritance_spec 
		{
		 $$ = new IDLParseNode(interface_header, $3);
		 $$->SetString($2);
		}		
	; 
c_interface_body	
	:	 /*empty*/
		{		
		 $$ = NULL;
		}
	|	 c_exports 
		{
		 $$ = $1;
		}
	;

c_exports		
	:	 c_export
		{		
		 $$ = $1;
		}
	|	 c_export c_exports 
		{
		 $$ = new IDLParseNode(interface_body, $1, $2);
		}
	;

c_export		
	:	 c_type_dcl SEMICOLON
		{		
		 $$ = $1;
		}
	|	 c_const_dcl SEMICOLON 
		{		
		 $$ = $1;
		}
	|	 c_except_dcl SEMICOLON
		{		
		 $$ = $1;
		}
	|	 c_attr_dcl SEMICOLON
		{		
		 $$ = $1;
		}
	|	 c_op_dcl SEMICOLON 
		{
		 $$ = $1;
		}
	;

c_inheritance_spec 
	:	 COLON c_scoped_names 
		{
		 $$ = $2;
		}		
	;

c_scoped_names	
	:	 c_scoped_name
		{
		 $$ = $1;		
		}
	|	 c_scoped_name COMMA c_scoped_names 
		{
		 $$ = new IDLParseNode(scoped_name_list, $1, $3);
		}
	;

c_scoped_name	
	:	 IDENTIFIER
		{	
		 $$ = new IDLParseNode(scoped_name);
		 $$->SetString($1);	
		}	
	|	 SCOPE IDENTIFIER
		{	
		 $$ = new IDLParseNode(scoped_name);
		 help_cp = "::";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *$2, *help_sp_1);
		 $$->SetString(help_sp_1);	
		}
	|	 c_scoped_name SCOPE IDENTIFIER 
		{
		 $$ = $1;
		 help_cp = "::";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *$3, *help_sp_1);
		 help_sp_2 = $1->GetString();
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_1);
		 $$->SetString(help_sp_1);	
		}
	;

c_const_dcl	
	:	 CONST c_const_type IDENTIFIER EQUAL c_const_expr 
		{
		 $$ = new IDLParseNode(const_dcl, $2, $5);
		 $$->SetString($3);
		}
	;

c_const_type	
	:	 c_integer_type
		{			
		 $$ = $1;
		}
	|	 c_char_type
		{		
		 $$ = $1;
		}
	|	 c_boolean_type
		{		
		 $$ = $1;
		}
	|	 c_floating_pt_type
		{		
		 $$ = $1;
		}
	|	 c_string_type
		{		
		 $$ = $1;
		}
	|	 c_scoped_name 
		{
		 $$ = $1;
		}
	;

c_const_expr	
	:	 c_or_exp 
		{
		 $$ = new IDLParseNode(const_expr, $1);
		}
	;

c_or_exp		
	:	 c_xor_exp
		{		
		 $$ = $1;
		}
	|	 c_or_exp VERTICAL c_xor_exp 
		{
		 $$ = new IDLParseNode(or_exp, $1, $3);
		 help_cp = " | ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_xor_exp		
	:	 c_and_exp
		{		
		 $$ = $1;
		}
	|	 c_xor_exp CIRCUMFLEX c_and_exp 
		{
		 $$ = new IDLParseNode(xor_exp, $1, $3);
		 help_cp = " ^ ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_and_exp		
	:	 c_shift_exp
		{		
		 $$ = $1;
		}
	|	 c_and_exp AMPERSAND c_shift_exp 
		{
		 $$ = new IDLParseNode(and_exp, $1, $3);
		 help_cp = " & ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_shift_exp	
	:	 c_add_exp
		{ 		
		 $$ = $1;
		}
	|	 c_shift_exp SHIFTRIGHT c_add_exp
		{		
		 $$ = new IDLParseNode(shift_exp, $1, $3);
		 help_cp = " >> ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 c_shift_exp SHIFTLEFT c_add_exp 
		{
		 $$ = new IDLParseNode(shift_exp, $1, $3);
		 help_cp = " << ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_add_exp		
	:	 c_mult_exp
		{
		 $$ = $1;
		}
	|	 c_add_exp PLUS c_mult_exp
		{
		 $$ = new IDLParseNode(add_exp, $1, $3);
		 help_cp = " + ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 c_add_exp MINUS c_mult_exp 
		{
		 $$ = new IDLParseNode(add_exp, $1, $3);
		 help_cp = " - ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_mult_exp	
	:	 c_unary_exp
		{
		 $$ = $1;
		}
	|	 c_mult_exp ASTERISK c_unary_exp
		{
		 $$ = new IDLParseNode(mult_exp, $1, $3);
		 help_cp = " * ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 c_mult_exp SOLIDUS c_unary_exp
		{
		 $$ = new IDLParseNode(mult_exp, $1, $3);
		 help_cp = " / ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 c_mult_exp PERCENT c_unary_exp 
		{
		 $$ = new IDLParseNode(mult_exp, $1, $3);
		 help_cp = " % ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_unary_exp	
	:	 c_unary_operator c_primary_exp
		{
		 $$ = new IDLParseNode(unary_exp, $1, $2);
		}
	|	 c_primary_exp 
		{
		 $$ = $1;
		}
	;

c_unary_operator	
	:	 MINUS
		{		
		 $$ = new IDLParseNode(unary_operator);
		 help_cp = "-";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 PLUS
		{		
		 $$ = new IDLParseNode(unary_operator);
		 help_cp = "+";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 TILDE 
		{
		 $$ = new IDLParseNode(unary_operator);
		 help_cp = "~";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_primary_exp	
	:	 c_scoped_name
		{		
		 $$ = new IDLParseNode(primary_exp, $1);
		}
	|	 c_literal
		{		
		 $$ = new IDLParseNode(primary_exp, $1);
		}
	|	 PARENTLEFT c_const_expr PARENTRIGHT 
		{
		 $$ = new IDLParseNode(primary_exp, $2);
		}
	;

c_literal		
	:	 c_integer_literal
		{
		 $$ = $1;		
		}
	|	 c_string_literal 
		{		
		 $$ = $1;		
		}
	|	 c_character_literal
		{		
		 $$ = $1;		
		}
	|	 c_floating_pt_literal
		{		
		 $$ = $1;		
		}
	|	 c_boolean_literal 
		{		
		 $$ = $1;		
		}
	;

c_boolean_literal	
	:	 TRUE
		{
		 $$ = new IDLParseNode(literal);
		 help_cp = "TRUE";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 FALSE 
		{		
		 $$ = new IDLParseNode(literal);
		 help_cp = "FALSE";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_positive_int_const 
	:	 c_const_expr 
		{
		 $$ = $1;		
		}
	;

c_type_dcl	
	:	 TYPEDEF c_type_declarator
		{	
		 $$ = new IDLParseNode(type_dcl, $2);	
		}
	|	 c_struct_type
		{		
		 $$ = new IDLParseNode(type_dcl, $1);	
		}
	|	 c_union_type
		{		
		 $$ = new IDLParseNode(type_dcl, $1);	
		}
	|	 c_enum_type 
		{		
		 $$ = new IDLParseNode(type_dcl, $1);	
		}
	;

c_type_declarator	
	:	 c_type_spec c_declarators 
		{	
		 $$ = new IDLParseNode(type_declarator, $1, $2);	
		}
	;

c_type_spec	
	:	 c_simple_type_spec
		{
		 $$ = $1;		
		}
	|	 c_constr_type_spec 
		{
		 $$ = $1;		
		}
	;

c_simple_type_spec 
	:	 c_base_type_spec
		{		
		 $$ = $1;
		}
	|	 c_template_type_spec
		{		
		 $$ = $1;
		}
	|	 c_scoped_name 
		{		
		 $$ = $1;
		}
	;

c_base_type_spec
	:	 c_floating_pt_type
		{
		 $$ = $1;		
		}
	|	 c_integer_type
		{
		 $$ = $1;		
		}
	|	 c_char_type
		{		
		 $$ = $1;		
		}
	|	 c_boolean_type
		{		
		 $$ = $1;		
		}
	|	 c_octet_type
		{		
		 $$ = $1;		
		}
	|	 c_any_type 
		{		
		 $$ = $1;		
		}
	|	 c_object_type
		{		
		 $$ = $1;		
		}
	|	 c_typecode_type
		{		
		 $$ = $1;		
		}
	;

c_template_type_spec 
	:	 c_sequence_type
		{
		 $$ = $1;		
		}
	|	 c_string_type 
		{
		 $$ = $1;		
		}
	;

c_constr_type_spec 
	:	 c_struct_type
		{
		 $$ = $1;		
		}
	|	 c_union_type
		{
		 $$ = $1;		
		}
	|	 c_enum_type 
		{
		 $$ = $1;		
		}
	;

c_declarators	
	:	 c_declarator
		{
		 $$ = $1;		
		}
	|	 c_declarator COMMA c_declarators 
		{
		 $$ = new IDLParseNode(declarator_list, $1, $3);		
		}
	;

c_declarator	
	:	 c_simple_declarator 
		{		
		 $$ = $1;
		}
	|	 c_complex_declarator 
		{
		 $$ = $1;		
		}
	;

c_simple_declarator 
	:	 IDENTIFIER 
		{		
		 $$ = new IDLParseNode(declarator);
		 $$->SetString($1);
		}
	;

c_complex_declarator 
	:	 c_array_declarator 
		{
		 $$ = $1;		
		}
	;

c_floating_pt_type 
	:	 FLOAT
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "float";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 DOUBLE 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "double";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_integer_type	
	:	 c_signed_int
		{
		 $$ = $1;		
		}
	|	 c_unsigned_int 
		{
		 $$ = $1;		
		}
	;

c_signed_int	
	:	 c_signed_long_int
		{
		 $$ = $1;			
		}
	|	 c_signed_short_int 
		{
		 $$ = $1;		
		}
	;

c_signed_long_int	
	:	 LONG 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "long";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_signed_short_int 
	:	 SHORT 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "short";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_unsigned_int	
	:	 c_unsigned_long_int
		{		
		 $$ = $1;
		}
	|	 c_unsigned_short_int 
		{		
		 $$ = $1;
		}
	;

c_unsigned_long_int 
	:	 NOSIGN LONG 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "unsigned long";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_unsigned_short_int 
	:	 NOSIGN SHORT 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "unsigned short";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_char_type	
	:	 CHAR 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "char";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_boolean_type	
	:	 BOOLEAN 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "boolean";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_octet_type	
	:	 OCTET 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "octet";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_any_type	
	:	 ANY 
		{		
		 $$ = new IDLParseNode(base_type);
		 help_cp = "any";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_object_type
	:	OBJECT
	       {
		 $$ = new IDLParseNode(base_type);
		 help_cp = "Object";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
               }

c_typecode_type
	:	TYPECODE
	       {
		 $$ = new IDLParseNode(base_type);
		 help_cp = "TypeCode";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
               }

c_struct_type	
	:	 STRUCT IDENTIFIER CURLYBRACELEFT c_member_list CURLYBRACERIGHT 
		{
		 $$ = new IDLParseNode(struct_type, $4);
		 $$->SetString($2);		
		}
	; 

c_member_list	
	:	 c_member
		{
		 $$ = $1;		
		}
	|	 c_member c_member_list 
		{
		 $$ = new IDLParseNode(member_list, $1, $2);		
		}
	;

c_member		
	:	 c_type_spec c_declarators SEMICOLON 
		{
		 $$ = new IDLParseNode(member_declarator, $1, $2);		
		}
	;

c_union_type	
	:	 UNION IDENTIFIER SWITCH PARENTLEFT c_switch_type_spec PARENTRIGHT CURLYBRACELEFT c_switch_body CURLYBRACERIGHT 
		{
		 $$ = new IDLParseNode(union_type, $5, $8);
		 $$->SetString($2);		
		}
	; 

c_switch_type_spec 
	:	 c_integer_type
		{		
		 $$ = $1;
		}
	|	 c_char_type
		{		
		 $$ = $1;
		}
	|	 c_boolean_type
		{		
		 $$ = $1;
		}
	|	 c_enum_type
		{		
		 $$ = $1;
		}
	|	 c_scoped_name 
		{		
		 $$ = $1;
		}
	;

c_switch_body	
	:	 c_case
		{
		 $$ = $1;		
		}
	|	 c_case c_switch_body 
		{
		 $$ = new IDLParseNode(union_body, $1, $2);		
		}
	;

c_case		
	:	 c_case_label c_case 
		{
		 $$ = new IDLParseNode(case_type, $1, $2);		
		}
	|	 c_case_label c_element_spec SEMICOLON 
		{
		 $$ = new IDLParseNode(case_type, $1, $2);		
		}
	;

c_case_label	
	:	 CASE c_const_expr COLON 
		{
		 $$ = new IDLParseNode(case_label, $2);
		}
	|	 DEFAULT COLON 
		{		
		 $$ = new IDLParseNode(case_label);
		 help_cp = "default";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);		
		}
	;

c_element_spec	
	:	 c_type_spec c_declarator 
		{		
		 $$ = new IDLParseNode(element_declarator, $1, $2);
		}
	;

c_enum_type 	
	:	 ENUM IDENTIFIER CURLYBRACELEFT c_enumerators CURLYBRACERIGHT 
		{		
		 $$ = new IDLParseNode(enum_type, $4);
		 $$->SetString($2);
		}
	; 

c_enumerators	
	:	 c_enumerator
		{
		 $$ = $1;		
		}
	|	 c_enumerator COMMA c_enumerators 
		{
		 $$ = new IDLParseNode(declarator_list, $1, $3);		
		}
	;

c_enumerator	
	:	 IDENTIFIER 
		{
		 $$ = new IDLParseNode(declarator);
		 $$->SetString($1);		
		}
	;

c_sequence_type	
	:	 SEQUENCE LESSTHAN c_simple_type_spec COMMA c_positive_int_const GREATERTHAN 
		{		
		 $$ = new IDLParseNode(template_type, $3, $5);
		 help_cp = "sequence";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);	
		}
	|	 SEQUENCE LESSTHAN c_simple_type_spec GREATERTHAN 
		{		
		 $$ = new IDLParseNode(template_type, $3);
		 help_cp = "sequence";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);	
		}
	;

c_string_type	
	:	 STRING LESSTHAN c_positive_int_const GREATERTHAN
		{	
		 $$ = new IDLParseNode(template_type, $3);
		 help_cp = "string";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);	
		}
	|	 STRING 
		{		
		 $$ = new IDLParseNode(template_type);
		 help_cp = "string";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);	
		}
	;

c_array_declarator 
	:	 IDENTIFIER c_fixed_array_sizes 
		{
		 $$ = new IDLParseNode(array_declarator, $2); 
		 $$->SetString($1);
		}
	;

c_fixed_array_sizes 
	:	 c_fixed_array_size
		{
		 $$ = $1;		
		}
	|	 c_fixed_array_size c_fixed_array_sizes 
		{
		 $$ = new IDLParseNode(array_bound_list, $1, $2);
		}
	;

c_fixed_array_size 
	:	 SQUAREBRACELEFT c_positive_int_const SQUAREBRACERIGHT 
		{		
		 $$ = new IDLParseNode(array_bound, $2);
		}
	;

c_attr_dcl	
	:	 READONLY ATTRIBUTE c_simple_type_spec c_declarators
		{		
		 $$ = new IDLParseNode(attr_dcl, $3, $4);
		 help_cp = "readonly";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	 ATTRIBUTE c_simple_type_spec c_declarators 
		{		
		 $$ = new IDLParseNode(attr_dcl, $2, $3);
		}
	; 

c_except_dcl	
	:	 EXCEPTION IDENTIFIER CURLYBRACELEFT c_members CURLYBRACERIGHT 
		{		
		 $$ = new IDLParseNode(except_dcl, $4);
		 $$->SetString($2);
		}
	;

c_members		
	:	 /*empty*/
		{		
		 $$ = NULL;
		}
	|	 c_member c_members 
		{
		 $$ = new IDLParseNode(member_list, $1, $2);
		}
	;

c_op_dcl
	:	c_op_attribute c_op_type_spec IDENTIFIER c_parameter_dcls c_raises_expr c_context_expr
		{
		 $$ = new IDLParseNode(op_dcl, $1, $2, $4, $5, $6);
		 $$->SetString($3);
		}
	;

c_op_attribute	
	:	/*empty*/
		{		
		 $$ = NULL;
		}
	|	ONEWAY 
		{		
		 $$ = new IDLParseNode(attribute);
		 help_cp = "oneway";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

c_op_type_spec	
	:	 c_simple_type_spec
		{
		 $$ = new IDLParseNode(op_type, $1);
		}
	|	 VOID 
		{ 
		 $$ = new IDLParseNode(op_type);
		 help_cp = "void";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);		
		}
	;

c_parameter_dcls	
	:	 PARENTLEFT c_param_dcls PARENTRIGHT  
		{	
		 $$ = $2;	
		}
	|	 PARENTLEFT PARENTRIGHT
		{		
		 $$ = NULL;		
		}
	;

c_param_dcls	
	:	 c_param_dcl
		{	
		 $$ = $1;	
		}
	|	 c_param_dcl COMMA c_param_dcls 
		{
		 $$ = new IDLParseNode(param_declarator_list, $1, $3);		
		}
	;

c_param_dcl	
	:	 c_param_attribute c_simple_type_spec c_declarator 
		{
		 $$ = new IDLParseNode(param_declarator, $2, $3);
		 help_sp_1 = new string(*$1->GetString());
		 $$->SetString(help_sp_1);
		 delete($1);		
		}
	;

c_param_attribute	
	:	 IN
		{	
		 $$ = new IDLParseNode(param_attribute);
		 help_cp = "in";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);		
		}
	|	 OUT
		{		
		 $$ = new IDLParseNode(param_attribute);
		 help_cp = "out";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);		
		}
	|	 INOUT 
		{		
		 $$ = new IDLParseNode(param_attribute);
		 help_cp = "inout";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);		
		}
	;

c_raises_expr	
	:	 /*empty*/
		{
		 $$ = NULL;		
		}
	|	 RAISES PARENTLEFT c_scoped_names PARENTRIGHT 
		{
		 $$ = $3;
		}
	;

c_context_expr	
	:	 /*empty*/
		{		
		 $$ = NULL;		
		}
	|	 CONTEXT PARENTLEFT c_string_literals PARENTRIGHT 
		{		
		 $$ = $3;
		}
	;

c_string_literals	
	:	 c_string_literal
		{
		 $$ = $1;		
		}
	|	 c_string_literal COMMA c_string_literals 
		{		
		 $$ = new IDLParseNode(literal_list, $1, $3);		
		}
	;

c_string_literal 
	:	 STRING_LITERAL 
		{
		 $$ = new IDLParseNode(literal);
		 $$->SetString($1);
		}
	;	

c_integer_literal	
	:	 INTEGER_LITERAL 
		{		
		 $$ = new IDLParseNode(literal);
		 $$->SetString($1);	
		}
	;

c_floating_pt_literal 
	:	 FLOATING_PT_LITERAL 
		{		
		 $$ = new IDLParseNode(literal);
		 $$->SetString($1);	
		}
	;

c_character_literal 
	:	 CHARACTER_LITERAL 
		{		
		 $$ = new IDLParseNode(literal);
		 $$->SetString($1);	
		}
	;


/********  DCE Grammatik_Regeln  ********/

d_definition
	:	d_interface
		{
		 $$ = $1;
		}
	|	d_interface d_importfiles
		{
		 $$ = new IDLParseNode(interface_list, $1, $2);
		}
	;

d_importfiles
	:	d_importfile
		{
		 $$ = $1;
		}
	|	d_importfile d_importfiles
		{
		 $$ = new IDLParseNode(importfile_list, $1, $2);
		}
	;
	
d_importfile
	:	IMPORTFILE d_interface
		{
		 $$ = new IDLParseNode(importfile, $2);
		}
	;

d_interface		
	: 	d_interface_header CURLYBRACELEFT d_interface_body CURLYBRACERIGHT 
		{
		 $$ = new IDLParseNode(interface_dcl, $1, $3);
		}
	;

d_interface_header	
	: 	SQUAREBRACELEFT d_interface_attributes SQUAREBRACERIGHT INTERFACE IDENTIFIER 
		{
		 $$ = new IDLParseNode(interface_header, $2);
		 $$->SetString($5); 
		}
	;

d_interface_attributes 
	: 	d_interface_attribute
		{
		 $$ = $1;
		}
	| 	d_interface_attribute COMMA d_interface_attributes 
		{
		 $$ = new IDLParseNode(attribute_list, $1, $3);
		}
	;

d_interface_attribute  
	: 	UUID PARENTLEFT d_uuid_rep PARENTRIGHT 
		{
		 $$ = new IDLParseNode(interface_attribute, $3);
		 help_cp = "uuid";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	VERSION PARENTLEFT d_version_number PARENTRIGHT 
		{
		 $$ = new IDLParseNode(interface_attribute, $3);
		 help_cp = "version";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	| 	PT_DEFAULT PARENTLEFT d_pointer_attribute PARENTRIGHT 
		{
		 $$ = new IDLParseNode(interface_attribute, $3);
		 help_cp = "ptr_default";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	ENDPOINT PARENTLEFT d_endpoint_specs PARENTRIGHT	
		{
		 $$ = new IDLParseNode(interface_attribute, $3);
		 help_cp = "endpoint";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	| 	LOCAL 
		{
		 $$ = new IDLParseNode(interface_attribute);
		 help_cp = "local";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_uuid_rep	
	:	STRING_LITERAL
		{ 
		 $$ = new IDLParseNode(attr_part);
		 $$->SetString($1);
		}
	;

d_version_number	
	: 	FLOATING_PT_LITERAL
		{
		 $$ = new IDLParseNode(attr_part);
		 $$->SetString($1);
		}
	;

d_endpoint_specs		
	: 	d_endpoint_spec
		{
		 $$ = $1;
		}
	| 	d_endpoint_spec COMMA d_endpoint_specs 
		{
		 $$ = new IDLParseNode(endpoint_list, $1, $3);
		}
	;

d_endpoint_spec	
	: 	DOUBLEQUOTE d_family_string COLON SQUAREBRACELEFT d_endpoint_string SQUAREBRACERIGHT DOUBLEQUOTE 
		{
		 $$ = new IDLParseNode(endpoint);
		 help_cp = "]\"";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$5->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = ":[";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 help_sp_2 = new string(*$2->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "\"";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $2;
		 delete $5;
		}
	; 

d_family_string		
	: 	NCACN_IP_TCP
		{
		 $$ = new IDLParseNode(attr_part);
		 help_cp = "ncacn_ip_tcp";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	| 	NCADG_IP_UDP 
		{
		 $$ = new IDLParseNode(attr_part);
		 help_cp = "ncadg_ip_udp";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_endpoint_string		
	:	INTEGER_LITERAL 
		{
		 $$ = new IDLParseNode(attr_part);
		 $$->SetString($1);
		}
	;

d_pointer_attribute	
	: 	PTR
		{
		 $$ = new IDLParseNode(attr_part);
		 help_cp = "ptr";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	| 	REF 
		{
		 $$ = new IDLParseNode(attr_part);
		 help_cp = "ref";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_interface_body	
	: 	d_imports d_exports 
		{
		 $$ = $2;
		}
	;

d_imports		
	: 	/*empty*/	
		{
		}
	| 	d_import d_imports 
		{
		}
	;

d_import	
	: 	IMPORTFILE d_filenames SEMICOLON 
		{
		}
	;

d_filenames		
	:	d_filename 
		{
		}
	| 	d_filename COMMA d_filenames 
		{
		}
	;

d_filename
	:	DOUBLEQUOTE STRING_LITERAL DOUBLEQUOTE
		{
		}
	;

d_exports
	:	/*empty*/
		{
		 $$ = NULL;
		}
	|	d_export d_exports
		{
		 $$ = new IDLParseNode(interface_body, $1, $2);
		}
	;

d_export
	:	d_const_declaration SEMICOLON
		{
		 $$ = $1;
		}
	|	d_type_declaration SEMICOLON
		{
		 $$ = $1;
		}
	|	d_op_declaration SEMICOLON
		{
		 $$ = $1;
		}
	;

d_const_declaration		
	: 	CONST d_const_type_spec IDENTIFIER EQUAL d_const_exp
		{
		 $$ = new IDLParseNode(const_dcl, $2, $5);
		 $$->SetString($3);
		}
	;

d_const_type_spec
	:	d_integer_type 
		{
		 $$ = $1;
		}
	|	CHAR
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "char";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	CHARPTR 
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "char*";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	BOOLEAN
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "boolean";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	VOIDPTR 
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "void*";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_integer_type
	:	d_signed_int
		{
		 $$ = $1;
		}
	|	d_unsigned_int
		{
		 $$ = $1;
		}
	;

d_signed_int
	:	d_int_size 
		{
		 $$ = $1;
		}
	|	d_int_size INT
		{
		 $$ = $1;
		}
	;

d_unsigned_int
	: 	NOSIGN d_int_size  
		{
		 $$ = $2;
		 help_cp = "unsigned ";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = $2->GetString();
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		}
	|	NOSIGN d_int_size INT
		{
		 $$ = $2;
		 help_cp = "unsigned ";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = $2->GetString();
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		}
	|	d_int_size NOSIGN_INT
		{
		 $$ = $1;
		 help_cp = "unsigned ";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = $1->GetString();
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		}
	|	d_int_size NOSIGN 
		{
		 $$ = $1;
		 help_cp = "unsigned ";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = $1->GetString();
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		}
	;

d_int_size		
	:      	LONG
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "long";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	| 	SHORT
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "short";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	HYPER
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "hyper";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	| 	SMALL 
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "small";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_const_exp	
	: 	d_integer_const_exp
		{
		 $$ = new IDLParseNode(const_expr, $1);
		}
	|	d_character_const
		{
		 $$ = new IDLParseNode(const_expr, $1);
		}
	|	d_string_const
		{
		 $$ = new IDLParseNode(const_expr, $1);
		}
	|	TRUE
		{
		 $$ = new IDLParseNode(const_expr);
		 help_cp = "TRUE";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	FALSE
		{
		 $$ = new IDLParseNode(const_expr);
		 help_cp = "FALSE";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	NULLPTR 
		{
		 $$ = new IDLParseNode(const_expr);
		 help_cp = "NULL"; 
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_integer_const_exp
	:	d_conditional_exp
		{
		 $$ = $1;
		}
	;

d_conditional_exp
	:	d_logical_or_exp
		{
		 $$ = $1;
		}
	|	d_logical_or_exp QUESTIONMARK d_integer_const_exp COLON d_conditional_exp
		{
		 $$ = new IDLParseNode(conditional_exp, $1, $3, $5);
		 help_cp = " ! ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	; 

d_logical_or_exp
	:	d_logical_and_exp
		{
		 $$ = $1;
		}
	|	d_logical_or_exp DOUBLEVERTICAL d_logical_and_exp
		{ 
		 $$ = new IDLParseNode(logical_or_exp, $1, $3);
		 help_cp = " || ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_logical_and_exp
	:	d_inclusive_or_exp
		{
		 $$ = $1;
		}
	|	d_logical_and_exp DOUBLEAMPERSAND d_inclusive_or_exp
		{
		 $$ = new IDLParseNode(logical_and_exp, $1, $3);
		 help_cp = " && ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_inclusive_or_exp
	:	d_exclusive_or_exp
		{
		 $$ = $1;
		}
	|	d_inclusive_or_exp VERTICAL d_exclusive_or_exp
		{
		 $$ = new IDLParseNode(inclusive_or_exp, $1, $3);
		 help_cp = " | ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_exclusive_or_exp
	:	d_and_exp
		{
		 $$ = $1;
		}
	|	d_and_exp CIRCUMFLEX d_and_exp
		{
		 $$ = new IDLParseNode(exclusive_or_exp, $1, $3);
		 help_cp = " ^ ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_and_exp
	:	d_equality_exp
		{
		 $$ = $1;
		}
	|	d_and_exp AMPERSAND d_equality_exp
		{		
		 $$ = new IDLParseNode(and_exp, $1, $3);
		 help_cp = " & ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_equality_exp
	:	d_relational_exp
		{
		 $$ = $1;
		}
	|	d_equality_exp DOUBLEEQUAL d_relational_exp
		{
		 $$ = new IDLParseNode(equality_exp, $1, $3);
		 help_cp = " == ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_equality_exp QUESTIONMARKEQUAL d_relational_exp
		{
		 $$ = new IDLParseNode(equality_exp, $1, $3);
		 help_cp = " != ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}	
	;

d_relational_exp
	:	d_shift_exp
		{
		 $$ = $1;
		}
	|	d_relational_exp LESSTHAN d_shift_exp
		{
		 $$ = new IDLParseNode(relational_exp, $1, $3);
		 help_cp = " < ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_relational_exp GREATERTHAN d_shift_exp
		{
		 $$ = new IDLParseNode(relational_exp, $1, $3);
		 help_cp = " > ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_relational_exp LESSTHANEQUAL d_shift_exp
		{
		 $$ = new IDLParseNode(relational_exp, $1, $3);
		 help_cp = " <= ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_relational_exp GREATERTHANEQUAL d_shift_exp
		{
		 $$ = new IDLParseNode(relational_exp, $1, $3);
		 help_cp = " >= ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_shift_exp
	:	d_additive_exp
		{
		 $$ = $1;
		}
	|	d_shift_exp SHIFTLEFT d_additive_exp
		{
		 $$ = new IDLParseNode(shift_exp, $1, $3);
		 help_cp = " << ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_shift_exp SHIFTRIGHT d_additive_exp
		{
		 $$ = new IDLParseNode(shift_exp, $1, $3);
		 help_cp = " >> ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_additive_exp
	:	d_multiplicative_exp
		{
		 $$ = $1;
		}
	|	d_additive_exp PLUS d_multiplicative_exp
		{
		 $$ = new IDLParseNode(add_exp, $1, $3);
		 help_cp = " + ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_additive_exp MINUS d_multiplicative_exp
		{
		 $$ = new IDLParseNode(add_exp, $1, $3);
		 help_cp = " - ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_multiplicative_exp
	:	d_unary_exp
		{
		 $$ = $1;
		}
	|	d_multiplicative_exp ASTERISK d_unary_exp
		{
		 $$ = new IDLParseNode(mult_exp, $1, $3);
		 help_cp = " * ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_multiplicative_exp SOLIDUS d_unary_exp
		{
		 $$ = new IDLParseNode(mult_exp, $1, $3);
		 help_cp = " / ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_multiplicative_exp PERCENT d_unary_exp
		{
		 $$ = new IDLParseNode(mult_exp, $1, $3);
		 help_cp = " % ";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_unary_exp
	:	d_primary_exp
		{
		 $$ = $1;
		}
	|	PLUS d_primary_exp
		{
		 $$ = new IDLParseNode(unary_exp, $2);
		 help_cp = "+";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	MINUS d_primary_exp
		{
		 $$ = new IDLParseNode(unary_exp, $2);
		 help_cp = "-";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	TILDE d_primary_exp
		{
		 $$ = new IDLParseNode(unary_exp, $2);
		 help_cp = "~";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	QUESTIONMARK d_primary_exp
		{
		 $$ = new IDLParseNode(unary_exp, $2);
		 help_cp = "!";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_primary_exp
	:	INTEGER_LITERAL 
		{
		 $$ = new IDLParseNode(primary_exp);
		 $$->SetString($1);
		}
	|	IDENTIFIER
		{
		 $$ = new IDLParseNode(primary_exp);
		 $$->SetString($1);
		}
	;

d_character_const
	:	CHARACTER_LITERAL
		{
		 $$ = new IDLParseNode(literal);
		 $$->SetString($1);
		}
	;

d_string_const
	:	STRING_LITERAL
		{
		 $$ = new IDLParseNode(literal);
		 $$->SetString($1);
		}
	;

d_type_declaration
	:	TYPEDEF d_type_attr d_type_spec d_declarators
		{
		 $$ = new IDLParseNode(type_dcl, $3, $4, $2);
		}
	;

d_type_attr
	:	/*empty*/
		{
		 $$ = NULL;
		}
	|	SQUAREBRACELEFT d_type_attributes SQUAREBRACERIGHT
		{
		 $$ = $2;
		}
	;

d_type_attributes
	:	d_type_attribute
		{
		 $$ = $1;
		}
	|	d_type_attribute COMMA d_type_attributes
		{
		 $$ = new IDLParseNode(attribute_list, $1, $3);
		}
	;

d_type_spec
	:	d_simple_type_spec
		{
		 $$ = $1;
		}
	|	d_constructed_type_spec
		{
		 $$ = $1;
		}
	;

d_simple_type_spec
	:	d_base_type_spec
		{
		 $$ = $1;
		}
	|	d_predefined_type_spec
		{
		 $$ = $1;
		}
	|	IDENTIFIER
		{
		 $$ = new IDLParseNode(scoped_name);
		 $$->SetString($1);
		}
	;

d_declarators
	:	d_declarator
		{
		 $$ = $1;
		}
	|	d_declarator COMMA d_declarators
		{
		 $$ = new IDLParseNode(declarator_list, $1, $3);
		}
	;

d_declarator
	:	d_simple_declarator
		{
		 $$ = $1;
		}
	|	d_complex_declarator
		{
		 $$ = $1;
		}
	;

d_simple_declarator
	:	IDENTIFIER
		{
		 $$ = new IDLParseNode(declarator);
		 $$->SetString($1);
		}
	;

d_complex_declarator
	:	d_array_declarator
		{
		 $$ = $1;
		}
	|	d_function_ptr_declarator
		{
		 $$ = $1;
		}
	|	d_ptr_declarator
		{
		 $$ = $1;
		}
	;

d_tagged_declarator
	:	d_tagged_struct_declarator
		{
		 $$ = $1;
		}
	|	d_tagged_union_declarator
		{
		 $$ = $1;
		}
	;

d_base_type_spec
	:	d_integer_type 
		{
		 $$ = $1;
		}
	|	d_floating_type
		{
		 $$ = $1;
		}
	|	d_char_type
		{
		 $$ = $1;
		}
	|	d_boolean_type
		{
		 $$ = $1;
		}
	|	d_byte_type
		{
		 $$ = $1;
		}
	|	d_void_type
		{
		 $$ = $1;
		}
	|	d_handle_type
		{
		 $$ = $1;
		}
	|	d_any_type
		{
		 $$ = $1;
		}
	;

d_floating_type
	:	FLOAT
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "float";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	DOUBLE
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "double";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_char_type
	:	CHAR
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "char";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	NOSIGN CHAR
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "unsigned char";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_boolean_type
	:	BOOLEAN
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "boolean";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_any_type
	:
		ANY
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "byte";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_byte_type
	:	BYTE
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "byte";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_void_type
	:	VOID
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "void";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_handle_type
	:	HANDLE_T
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "handle_t";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_constructed_type_spec
	:	d_struct_type
		{
		 $$ = $1;
		}
	|	d_union_type
		{
		 $$ = $1;
		}
	|	d_tagged_declarator
		{
		 $$ = $1;
		}
	|	d_enumeration_type
		{
		 $$ = $1;
		}
	|	d_pipe_type
		{
		 $$ = $1;
		}
	;

d_tagged_struct_declarator
	:	STRUCT d_tag
		{
		 $$ = new IDLParseNode(struct_type);
		 help_sp_1 = new string(*$2->GetString());
		 $$->SetString(help_sp_1);
		 delete $2;
		}
	|	d_tagged_struct
		{
		 $$ = $1;
		}
	;

d_struct_type
	:	STRUCT CURLYBRACELEFT d_member_list CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(struct_type, $3);
		}
	;

d_tagged_struct
	:	STRUCT d_tag CURLYBRACELEFT d_member_list CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(struct_type, $4, $2);
		}
	;

d_tag	
	:	IDENTIFIER
		{
		 $$ = new IDLParseNode(declarator);
		 $$->SetString($1);
		}
	;

d_member_list
	:	d_member 
		{
		 $$ = $1;
		}
	|	d_member d_member_list
		{
		 $$ = new IDLParseNode(member_list, $1, $2);
		}
	;

d_member
	:	d_field_declarator SEMICOLON
		{
		 $$ = $1;
		}
	;

d_field_declarator
	:	d_field_attribute_list d_type_spec d_declarators
		{
		 $$ = new IDLParseNode(field_declarator, $2, $3, $1);
		}
	;

d_field_attribute_list
	:	/*empty*/
		{
		 $$ = NULL;
		}
	|	SQUAREBRACELEFT d_field_attributes SQUAREBRACERIGHT
		{
		 $$ = $2;
		}
	;

d_field_attributes
	:	d_field_attribute
		{
		 $$ = $1;
		}
	|	d_field_attribute COMMA d_field_attributes
		{
		 $$ = new IDLParseNode(attribute_list, $1, $3);
		}
	;

d_tagged_union_declarator
	:	UNION d_tag
		{
		 $$ = new IDLParseNode(union_type);
		 help_sp_1 = new string(*$2->GetString());
		 $$->SetString(help_sp_1);
		 delete $2;
		}
	|	d_tagged_union
		{
		 $$ = $1;
		}
	;

d_union_type
	:	UNION d_union_switch CURLYBRACELEFT d_union_body CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(union_type, $4, $2);
		}
	|	UNION CURLYBRACELEFT d_union_body_n_e CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(union_type_n_e, $3);
		}
	;

d_union_switch
	:	SWITCH PARENTLEFT d_switch_type_spec IDENTIFIER PARENTRIGHT d_union_name
		{
		 $$ = new IDLParseNode(union_switch, $3, $6);
		 $$->SetString($4);
		}
	;

d_switch_type_spec
	:	d_integer_type
		{
		 $$ = $1;
		}
	|	d_char_type
		{
		 $$ = $1;
		}
	|	d_boolean_type
		{
		 $$ = $1;
		}
	|	d_enumeration_type
		{
		 $$ = $1;
		}
	;

d_tagged_union
	:	UNION d_tag d_union_switch CURLYBRACELEFT d_union_body CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(union_type, $5, $3, $2);
		}
	|	UNION d_tag CURLYBRACELEFT d_union_body_n_e CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(union_type_n_e, $4, $2);
		}
	;

d_union_name
	:	/*empty*/
		{
		 $$ = NULL;
		}
	|	IDENTIFIER
		{
		 $$ = new IDLParseNode(declarator);
		 $$->SetString($1);
		}
	;

d_union_body
	:	d_union_cases d_default_case
		{
		 $$ = new IDLParseNode(union_body, $1, $2);
		}
	|	d_union_cases
		{
		 $$ = new IDLParseNode(union_body, $1);
		}
	;

d_union_body_n_e
	:	d_union_cases_n_e d_default_case_n_e
		{
		 $$ = new IDLParseNode(union_body, $1, $2);
		}
	|	d_union_cases_n_e
		{
		 $$ = new IDLParseNode(union_body, $1);
		}
	;

d_union_cases
	:	d_union_case
		{
		 $$ = $1;
		}
	|	d_union_case d_union_cases
		{
		 $$ = new IDLParseNode(case_list, $1, $2);
		}
	;

d_union_case
	:	d_union_case_labels d_union_arm
		{
		 $$ = new IDLParseNode(case_type, $2, $1);
		}
	;

d_union_cases_n_e
	:	d_union_case_n_e
		{
		 $$ = $1;
		}
	|	d_union_case_n_e d_union_cases_n_e
		{
		 $$ = new IDLParseNode(case_list, $1, $2);
		}
	;

d_union_case_n_e
	:	d_union_case_label_n_e d_union_arm
		{
		 $$ = new IDLParseNode(case_type_n_e, $2, $1);
		}
	;

d_union_case_labels
	:	d_union_case_label
		{
		 $$ = $1;
		}
	|	d_union_case_label d_union_case_labels
		{
		 $$ = new IDLParseNode(case_label_list, $1, $2);
		}
	;

d_union_case_label
	:	CASE d_const_exp COLON
		{
		 $$ = new IDLParseNode(case_label, $2);
		}
	;

d_union_case_label_n_e
	:	SQUAREBRACELEFT CASE PARENTLEFT d_const_exps PARENTRIGHT SQUAREBRACERIGHT
		{
		 $$ = new IDLParseNode(case_label, $4);
		}
	;

d_const_exps
	:	d_const_exp
		{
		 $$ = $1;
		}
	|	d_const_exp COMMA d_const_exps
		{
		 $$ = new IDLParseNode(const_expr_list, $1, $3);
		}
	;

d_default_case
	:	DEFAULT COLON d_union_arm
		{
		 $$ = new IDLParseNode(case_type, $3);
		}
	;

d_default_case_n_e
	:	SQUAREBRACELEFT_DEFAULT DEFAULT SQUAREBRACERIGHT d_union_arm		
		{
		 $$ = new IDLParseNode(case_type_n_e, $4);
		}
	;

d_union_arm
	:	d_field_declarator SEMICOLON
		{
		 $$ = $1;
		}
	|	SEMICOLON
		{
		 $$ = NULL;
		}
	;

d_union_type_switch_attr 
	:	SWITCH_TYPE PARENTLEFT d_switch_type_spec PARENTRIGHT
		{
		 $$ = new IDLParseNode(switch_type_attr, $3);
		}
	;

d_union_instance_switch_attr
	:	SWITCH_IS PARENTLEFT d_attr_var PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "switch_is(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	;

d_enumeration_type
	:	ENUM CURLYBRACELEFT d_identifiers CURLYBRACERIGHT
		{
		 $$ = new IDLParseNode(enum_type, $3);
		}
	;

d_identifiers
	:	d_identifier
		{
		 $$ = $1;
		}
	|	d_identifier COMMA d_identifiers
		{
		 $$ = new IDLParseNode(declarator_list, $1, $3);
		}
	;

d_identifier
	:	IDENTIFIER
		{
		 $$ = new IDLParseNode(declarator);
		 $$->SetString($1);
		}
	;

d_pipe_type
	:	PIPE d_type_spec d_pipe_declarators
		{
		 $$ = new IDLParseNode(pipe_type, $2, $3);
		}
	;

d_array_declarator
	:	IDENTIFIER d_array_bounds_list
		{
		 $$ = new IDLParseNode(array_declarator, $2);
		 $$->SetString($1);
		}
	;

d_array_bounds_list
	:	d_array_bounds_declarator
		{
		 $$ = $1;
		}
	|	d_array_bounds_declarator d_array_bounds_list 
		{
		 $$ = new IDLParseNode(array_bound_list, $1, $2);
		}
	;

d_array_bounds_declarator
	:	SQUAREBRACELEFT d_array_bound SQUAREBRACERIGHT
		{
		 $$ = $2;
		}
	|	SQUAREBRACELEFT d_array_bounds_pair SQUAREBRACERIGHT
		{
		 $$ = $2;
		}
	;

d_array_bounds_pair
	:	d_array_bound DOUBLEPERIOD d_array_bound
		{
		 $$ = new IDLParseNode(array_bound);
		 help_sp_1 = new string(*$3->GetString());
		 help_cp = "..";
		 help_sp_2 = new string(help_cp);
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_sp_1 = new string(*$1->GetString());
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $1;
		 delete $3;
		}
	;

d_array_bound
	:	ASTERISK
		{
		 $$ = new IDLParseNode(array_bound);
		 help_cp = "*";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	INTEGER_LITERAL
		{
		 $$ = new IDLParseNode(array_bound);
		 $$->SetString($1);
		}
	|	IDENTIFIER
		{
		 $$ = new IDLParseNode(array_bound);
		 $$->SetString($1);
		}
	;

d_type_attribute
	:	TRANSMIT_AS PARENTLEFT d_xmit_type PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "transmit_as(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	HANDLE
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "handle";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	ALIGN PARENTLEFT d_int_size PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "align(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	V1_STRUCT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "v1_struct";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	V1_ENUM
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "v1_enum";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		} 
	|	d_usage_attribute
		{
		 $$ = $1;
		}
	|	d_union_type_switch_attr
		{
		 $$ = $1;
		}
	|	d_ptr_attr
		{
		 $$ = $1;
		}
	;

d_usage_attribute
	:	STRING
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "string";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	V1_STRING
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "v1_string";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	CONTEXT_HANDLE
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "context_handle";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_xmit_type
	:	d_simple_type_spec
		{
		 $$ = $1;
		}
	;

d_field_attribute
	:	FIRST_IS PARENTLEFT d_attr_var_list PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "first_is(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	LAST_IS PARENTLEFT d_attr_var_list PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "last_is(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	LENGTH_IS PARENTLEFT d_attr_var_list PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "length_is(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	MAX_IS PARENTLEFT d_attr_var_list PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "max_is(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	SIZE_IS PARENTLEFT d_attr_var_list PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 help_sp_2 = new string(*$3->GetString());
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_cp = "size_is(";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $3;
		}
	|	V1_ARRAY
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "v1_array";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_usage_attribute
		{
		 $$ = $1;
		}
	|	d_union_instance_switch_attr
		{
		 $$ = $1;
		}
	|	IGNORE
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "ignore";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_ptr_attr
		{
		 $$ = $1;
		}
	;

d_attr_var_list
	:	d_attr_var
		{
		 $$ = $1;
		}
	|	d_attr_var COMMA d_attr_var_list
		{
		 $$ = new IDLParseNode(attr_var);
		 help_sp_1 = new string(*$3->GetString());
		 help_cp = ",";
		 help_sp_2 = new string(help_cp);
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 help_sp_1 = new string(*$1->GetString());
		 (void) cat(*help_sp_1, *help_sp_2, *help_sp_1);
		 $$->SetString(help_sp_1);
		 delete $1;
		 delete $3;
		}
	;

d_attr_var
	:	/*empty*/
		{
		 $$ = new IDLParseNode(attr_var);
		 help_cp = "";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	IDENTIFIER
		{
		 $$ = new IDLParseNode(attr_var);
		 $$->SetString($1);
		}
	|	ASTERISKIDENTIFIER
		{
		 $$ = new IDLParseNode(attr_var);
		 $$->SetString($1);
		}
	;

d_ptr_declarator	
	:	ASTERISKIDENTIFIER
		{
		 $$ = new IDLParseNode(ptr_declarator);
		 $$->SetString($1);
		}
	;

d_ptr_attr
	:	REF
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "ref";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	UNIQUE
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "unique";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	FULL
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "full";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	PTR
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "ptr";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_op_declaration
	:	d_simple_type_spec IDENTIFIER d_param_declarators
		{
		 $$ = new IDLParseNode(op_dcl, $1, $3);
		 $$->SetString($2);
		}
	| 	d_operation_attributes d_simple_type_spec IDENTIFIER d_param_declarators
		{
		 $$ =new IDLParseNode(op_dcl, $2, $4, $1);
		 $$->SetString($3);
		}
	;

d_operation_attributes
	:	SQUAREBRACELEFT d_op_attributes SQUAREBRACERIGHT 
		{
		 $$ = $2;
		}
	;

d_op_attributes
	:	d_operation_attribute
		{
		 $$ = $1;
		}
	|	d_operation_attribute COMMA d_op_attributes
		{
		 $$ = new IDLParseNode(attribute_list, $1, $3);
		}
	;

d_operation_attribute
	:	IDEMPOTENT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "idempotent";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	BROADCAST
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "broadcast";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	MAYBE
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "maybe";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_usage_attribute
		{
		 $$ = $1;
		}
	|	d_ptr_attr
		{
		 $$ = $1;
		}
	;

d_param_declarators
	:	PARENTLEFT VOID PARENTRIGHT
		{
		 $$ = NULL;
		}
	|	PARENTLEFT d_param_dcls PARENTRIGHT
		{
		 $$ = $2;
		}
	;

d_param_dcls
	:	d_param_declarator
		{
		 $$ = $1;
		}
	|	d_param_declarator COMMA d_param_dcls
		{
		 $$ = new IDLParseNode(param_declarator_list, $1, $3);
		}
	;

d_param_declarator
	:	d_param_attributes d_type_spec d_declarator
		{
		 $$ = new IDLParseNode(param_declarator, $2, $3, $1);
		}
	;

d_param_attributes
	:	SQUAREBRACELEFT d_param_attrs SQUAREBRACERIGHT
		{
		 $$ = $2;
		}
	;

d_param_attrs
	:	d_param_attribute
		{
		 $$ = $1;
		}
	|	d_param_attribute COMMA d_param_attrs
		{
		 $$ = new IDLParseNode(attribute_list, $1, $3);
		}
	;

d_param_attribute
	:	d_directional_attribute
		{
		 $$ = $1;
		}
	|	d_field_attribute
		{
		 $$ = $1; 
		}
	;

d_directional_attribute
	:	IN
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "in";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	IN PARENTLEFT SHAPE PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "in(shape)";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	OUT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "out";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	OUT PARENTLEFT SHAPE PARENTRIGHT
		{
		 $$ = new IDLParseNode(attribute);
		 help_cp = "out(shape)";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;
/* In dieser DCE-Regel wurde rechts 'd_simple_type_spec' geloescht */
d_function_ptr_declarator
	:	PARENTLEFT ASTERISKIDENTIFIER PARENTRIGHT d_param_declarators
		{
		 $$ = new IDLParseNode(function_ptr_dcl, $4);
		 help_cp = ")";
		 help_sp_1 = new string(help_cp);
		 (void) cat(*$2, *help_sp_1, *help_sp_1);
		 help_cp = "(";
		 help_sp_2 = new string(help_cp);
		 (void) cat(*help_sp_2, *help_sp_1, *help_sp_2);
		 $$->SetString(help_sp_2);
		}
	;

d_predefined_type_spec
	:	ERROR_STATUS_T
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "error_status_t";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	d_international_character_type
		{
		 $$ = $1;
		}
	;

d_international_character_type
	:	ISO_LATIN_1
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "iso_latin_1";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	ISO_MULTI_LINGUAL
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "iso_multi_lingual";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	|	ISO_UCS
		{
		 $$ = new IDLParseNode(base_type);
		 help_cp = "iso_ucs";
		 help_sp_1 = new string(help_cp);
		 $$->SetString(help_sp_1);
		}
	;

d_pipe_declarators
	:	d_pipe_declarator 
		{
		 $$ = $1;
		}
	|	d_pipe_declarator COMMA d_pipe_declarators
		{
		 $$ = new IDLParseNode(declarator_list, $1, $3);
		}
	;

d_pipe_declarator
	:	d_simple_declarator
		{
		 $$ = $1;
		}
	|	d_ptr_declarator
		{
		 $$ = $1;
		}
	;

%%
