/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.applet.Applet;

import GlobalVars;
import Viewport;
import CGEditor;
import EditLinearForm;
import Proxy;
import ChooseMatchingRules;
import MatchGraph;
import CGTemplates;
import TypeBrowser;
import TypeHier;
import EditMatchingRules;
import ProfileBox;


public class AIT extends Applet implements Runnable {
private volatile boolean pleaseCreate;
private Thread windowThread;
private AITFrame frame;
private Label label;
private Button button;

public void init ()
    {
        String version = System.getProperty ("java.version");

        GlobalVars.applet = this;
        pleaseCreate = false;
        windowThread = null;
        frame = null;

        setLayout(new GridLayout(2,0));

        add (button = new Button ("Start AI-Trader GUI"));
        add (label = new Label ("", Label.CENTER));

        validate ();

        GlobalVars.serverhost = getCodeBase().getHost();
        GlobalVars.serverhost = "apple.ap-c.org";
    }

public void start ()
    {
        if (windowThread == null) {
            windowThread = new Thread (this, "window thread");
            windowThread.start ();
        }
    }

public void stop ()
    {
        /*
         * this is called when you switch the virtual window in some
         * window managers, which is really bad...
        if (windowThread != null) {
            windowThread.stop ();
            windowThread = null;
            GlobalVars.CloseAll ();
        }
         */
    }

public synchronized void run ()
    {
        while (windowThread != null) {
            while (pleaseCreate == false) {
                try {
                    wait ();
                } catch (InterruptedException e) {}
            }
            try {
                button.disable ();

                frame = new AITFrame ();
                frame.pack();
                frame.init();
                frame.show();
                label.setText ("");

                while (pleaseCreate == true) {
                    try {
                        wait ();
                    } catch (InterruptedException e) {}
                }
            } catch (AITException e) {
                label.setText (e.getErrMsg());
                pleaseCreate = false;
            }
            button.enable ();
        }
    }

public synchronized boolean action (Event ev, Object obj)
    {
        if (ev.target == button) {
            pleaseCreate = true;
            label.setText ("Please wait while applet is being loaded ...");
            notify ();
        }
        return true;
    }

public synchronized void FrameClosed ()
    {
        pleaseCreate = false;
        notify ();
    }
}


class AITFrame extends Frame {
private CGEditor cgedit;
private MenuItem mnArrGraph, mnLinForm, mnExit, mnNewGraph,
    mnMRules, mnMatch, mnTypeBrowse, mnExport, mnEditMRules, mnProfiles;

public AITFrame ()
    {
        GlobalVars.frame = this;

        setTitle ("AI-Trader");

        MenuBar mbar = new MenuBar ();
        Menu editmenu = new Menu ("Edit");        
        editmenu.add (mnNewGraph = new MenuItem ("New graph"));        
        editmenu.add (mnArrGraph = new MenuItem ("Arrange graph"));
        editmenu.add (mnLinForm = new MenuItem ("Linear form of graph ..."));
        editmenu.add (mnEditMRules = new MenuItem ("Edit matching rules ..."));
        editmenu.addSeparator ();
        editmenu.add (mnExit = new MenuItem ("Exit"));

        Menu servmenu = new Menu ("Server");
        servmenu.add (mnMRules = new MenuItem ("Choose matching rules ..."));
        servmenu.add (mnProfiles = new MenuItem ("User Profiles ..."));

        Menu impmenu = new Menu ("Import");
        impmenu.add (mnMatch = new MenuItem ("Match graph ..."));
        impmenu.add (mnTypeBrowse = new MenuItem ("Type Browser ..."));

        Menu expmenu = new Menu ("Export");
        expmenu.add (mnExport = new MenuItem ("Export graph ..."));

        mbar.add (editmenu);
        mbar.add (servmenu);
        mbar.add (impmenu);
        mbar.add (expmenu);
        setMenuBar (mbar);

        setLayout (new GridBagLayout());

        cgedit = new CGEditor (new Dimension (500, 500), true);
        Viewport vp = new Viewport (cgedit, new Dimension (300, 300));
        GlobalVars.status = new Label ("");

        GlobalVars.constrain (this, vp, 0, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, GlobalVars.status, 0, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 0.0);

        GlobalVars.SetStatus ("");
    }
    
public void init ()
        throws AITException
    {
	try {
	    CGTemplates.SetTemplates (cgedit);
	} catch (AITException e) {
	    System.out.println ("AITException");
	}

        GlobalVars.transport = new TransportLayer (GlobalVars.serverhost);
        if (!GlobalVars.transport.connected()) {
            throw new AITException (1, "cannot connect to AIT proxy");
        }
        GlobalVars.proxy = new Proxy (GlobalVars.transport);

        GlobalVars.matchrules.addElement ("Specialization");
        GlobalVars.matchrules.addElement ("Negation");
        GlobalVars.matchrules.addElement ("Quantity");

        GlobalVars.profilekey = GlobalVars.proxy.createProfile();
    }

public boolean handleEvent (Event ev)
    {
        if (ev.target == mnLinForm) {
            if (GlobalVars.editlinform == null) {
                GlobalVars.editlinform = new EditLinearForm (cgedit);
                GlobalVars.editlinform.pack();
                GlobalVars.editlinform.show();
            }
        } else if (ev.target == mnArrGraph) {
            cgedit.Layout (null);
        } else if (ev.target == mnExit) {
            // dispose() kills netscape.
            GlobalVars.CloseAll ();
        } else if (ev.target == mnNewGraph) {
            cgedit.SetCG (null);
        } else if (ev.target == mnMRules) {
            ChooseMatchRules();
        } else if (ev.target == mnProfiles) {
            ProfileBox pb = new ProfileBox ();
            pb.pack();
            pb.show();
        } else if (ev.target == mnMatch) {
            Match ();
        } else if (ev.target == mnTypeBrowse) {
            ConceptualGraph cg = cgedit.GetCG ();
            String type = (cg != null) ? cg.getNode().getName() : "something";
            TypeBrowser b = new TypeBrowser (type, cg);
            b.pack();
            b.show();
        } else if (ev.target == mnExport) {
            Export ();
        } else if (ev.target == mnEditMRules) {
            EditMatchingRules emr = new EditMatchingRules ();
            emr.pack ();
            emr.show ();
        } else
            return false;
        return true;
    }

private void ChooseMatchRules ()
    {
        ChooseMatchingRules mr = new ChooseMatchingRules ();
        mr.pack ();
        mr.show ();
    }

private void Match ()
    {
        ConceptualGraph cg = cgedit.GetCG ();
        if (cg == null) {
            AlertBox a = new AlertBox ("AIT Error", "Enter a graph first");
            a.pack();
            a.show();
            return;
        }
        if (GlobalVars.matchrules.size() == 0) {
            ChooseMatchRules ();
            if (GlobalVars.matchrules.size() == 0) {
                AlertBox a = new AlertBox ("AIT Error",
                                           "You must select at least one " +
                                           "matching rule");
                a.pack();
                a.show();
                return;
            }
        }
        MatchGraph mg = new MatchGraph (cg);
        mg.pack();
        mg.show();
    }

private void Export ()
    {
        ConceptualGraph cg = cgedit.GetCG ();
        if (cg == null) {
            AlertBox a = new AlertBox ("AIT Error", "Enter a graph first");
            a.pack();
            a.show();
            return;
        }
        if (GlobalVars.matchrules.size() == 0) {
            ChooseMatchRules ();
            if (GlobalVars.matchrules.size() == 0) {
                AlertBox a = new AlertBox ("AIT Error",
                                           "You must select at least one " +
                                           "matching rule");
                a.pack();
                a.show();
                return;
            }
        }
        try {
            String roottype = cg.getNode().getName();
            if (!GlobalVars.proxy.containsWord (roottype)) {
                TypeHier h = new TypeHier (roottype);
                h.pack ();
                h.show ();
            }
            if (!GlobalVars.proxy.containsWord (roottype)) {
                AlertBox a = new AlertBox ("AIT Error",
                                           "You must insert '" + roottype +
                                           "' into dict first");
                a.pack();
                a.show();
                return;
            }
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
            return;
        }
        ExportGraph eg = new ExportGraph (cg);
        eg.pack();
        eg.show();
    }
}
