/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;

import GlobalVars;
import AlertBox;
import CGEditor;
import BorderPanel;
import ListBox;
import MatchResult;

public class CGViewer extends BorderPanel {
private CGEditor cgedit;
private ListBox list, olist;
private Vector keylist;

public CGViewer (Dimension sz)
    {
        super (BorderPanel.RIDGE);

        GridBagLayout gb = new GridBagLayout ();
        setLayout (gb);

        GlobalVars.constrain (this, new Label ("Variants"), 0, 0, 1, 1,
                              GridBagConstraints.NONE, 0.0, 0.0);
        list = new ListBox();
        GlobalVars.constrain (this, list, 0, 1, 1, 1,
                              GridBagConstraints.BOTH, 0.0, 1.0);

        GlobalVars.constrain (this, new Label ("OID's"), 1, 0, 1, 1,
                              GridBagConstraints.NONE, 0.0, 0.0);
        olist = new ListBox();
        GlobalVars.constrain (this, olist, 1, 1, 1, 1,
                              GridBagConstraints.BOTH, 0.0, 1.0);

        cgedit = new CGEditor (sz, false);
        Viewport vp = new Viewport (cgedit, sz);
        GlobalVars.constrain (this, vp, 2, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
    }

public void SetKeys (Vector keys)
    {
        boolean isstring;
        
        if (keys.size() > 0 && keys.elementAt(0) instanceof MatchResult) {
            keylist = MatchResult.ExtractKeys (keys);
            isstring = false;
        } else {
            keylist = keys;
            isstring = true;
        }
        list.clear();
        for (int i = 0; i < keylist.size(); ++i) {
            String entry;
            if (!isstring) {
                MatchResult mr = (MatchResult)keys.elementAt(i);
                int mval = mr.GetMatchVal();

                StringBuffer s = new StringBuffer ("(%) ");
                s.append (GetRootNode (mr.GetKey()));
                s.insert (1, mval);
                for (int j = 3 - Integer.toString(mval).length(); --j >= 0; )
                    s.insert (1, " ");
                entry = s.toString();
            } else {
                entry = GetRootNode ((String)keys.elementAt(i));
            }
            list.addItem (entry);
        }
        if (keylist.size() > 0) {
            String key0 = (String)keylist.elementAt(0);
            cgedit.SetCG (GetGraph (key0));
            list.select (0);
            SetOIDs (key0);
        } else {
            cgedit.SetCG (null);
            SetOIDs ();
        }
    }

public void Update ()
    {
        String key = GetSelectionKey();
        if (key != null) {
            cgedit.SetCG (GetGraph (key));
            SetOIDs (key);
        }
    }

private void SetOIDs (String key)
    {
        olist.clear ();
        try {
            Vector oids = GlobalVars.proxy.getProviders (key);
            if (oids.size() > 0) {
                for (int i = 0; i < oids.size(); ++i) {
                    ServiceEntry sve = (ServiceEntry)oids.elementAt(i);
                    olist.addItem (sve.getOID());
                }
                olist.select (0);
            }
        } catch (AITException e) {
        }
    }

private void SetOIDs ()
    {
        olist.clear ();
    }
    
private String GetRootNode (String key)
    {
        ConceptualGraph cg = GetGraph (key);
        if (cg == null)
            return "(cannot get graph)";
        StringBuffer ostr = new StringBuffer();
        cg.getNode().print (ostr);
        return ostr.toString();
    }

private ConceptualGraph GetGraph (String key)
    {
        try {
            ConceptualGraph cg;
            cg = GlobalVars.proxy.getGraph (key);
            return cg;
        } catch (AITException e) {
            return null;
        }
    }

public String GetSelectionOID ()
    {
        int idx;
        if ((idx = olist.getSelectedIndex()) < 0)
            return null;
        return (String)olist.getItem(idx);
    }

public String GetSelectionKey ()
    {
        int idx;
        if (keylist.size() == 0 || (idx = list.getSelectedIndex()) < 0)
            return null;
        return (String)keylist.elementAt(idx);
    }

public ConceptualGraph GetSelection ()
    {
        return GetGraph (GetSelectionKey());
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.LIST_SELECT:
            if (ev.target == list) {
                int idx = ((Integer)ev.arg).intValue();
                String key = (String)keylist.elementAt (idx);
                cgedit.SetCG (GetGraph (key));
                SetOIDs (key);
            } else if (ev.target == olist) {
            }
            return true;
        }
        return false;
    }
}
