/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;

import GlobalVars;
import AlertBox;


public class ChooseMatchingRules extends Dialog {
private Button ok, cancel;
private Vector boxes;

public ChooseMatchingRules ()
    {
        super (GlobalVars.frame, true);
        try {
            Vector rules = GlobalVars.proxy.getMatchingRuleNames ();
            Panel p1 = new Panel ();

            GridBagLayout gb = new GridBagLayout ();
            GridBagConstraints gc = new GridBagConstraints ();
            gc.fill = GridBagConstraints.NONE;
            gc.anchor = GridBagConstraints.WEST;
            gc.gridwidth = GridBagConstraints.REMAINDER;
            setLayout (gb);

            boxes = new Vector();
            for (int i = 0; i < rules.size(); ++i) {
                String s = (String)rules.elementAt(i);
                Checkbox b = new Checkbox (s);
                b.setState (IsSelected (s));
                gb.setConstraints (b, gc);
                add (b);
                boxes.addElement (b);
            }
            gc.fill = GridBagConstraints.BOTH;
            gb.setConstraints (p1, gc);
            add (p1);

            ok = new Button ("Ok");
            cancel = new Button ("Cancel");
            p1.add ("West", ok);
            p1.add ("East", cancel);
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
        }
    }

private boolean IsSelected (String rule)
    {
        int n = GlobalVars.matchrules.size();
        for (int i = 0; i < n; ++i) {
            if (rule.equals ((String)GlobalVars.matchrules.elementAt(i)))
                return true;
        }
        return false;
    }

private void SetRules ()
    {
        GlobalVars.matchrules.removeAllElements ();
        for (int i = 0; i < boxes.size(); ++i) {
            Checkbox b = (Checkbox)boxes.elementAt (i);
            if (b.getState())
                GlobalVars.matchrules.addElement (b.getLabel());
        }
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.ACTION_EVENT:
            if (ev.target == ok) {
                SetRules();
                hide ();
                return true;
            } else if (ev.target == cancel) {
                hide ();
                return true;
            }
            break;
        }
        return false;
    }
}
