/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;

import GlobalVars;
import AlertBox;
import BorderPanel;
import ListBox;

class Set extends Vector {
public Set ()
    {
    }

public boolean Contains (Object o)
    {
        int sz = size();
        for (int i = 0; i < sz; ++i) {
            Object o2 = elementAt (i);
            if (o2 != null && o != null && o.equals (o2))
                return true;
        }
        return false;
    }

public void Add (Vector v, boolean checkunique)
    {
        int vsz = v.size();
        for (int i = 0; i < vsz; ++i) {
            Object o = v.elementAt (i);
            if (!checkunique || !Contains (o))
                addElement (o);
        }
    }

public void Sub (Object o)
    {
        for (int i = 0; i < size(); ) {
            Object o2 = elementAt (i);
            if (o2 != null && o != null && o.equals (o2))
                removeElementAt (i);
            else
                ++i;
        }
    }
    
public void Sub (Vector v)
    {
        int vsz = v.size();
        for (int j = 0; j < vsz; ++j) {
            Object o = v.elementAt (j);
            for (int i = 0; i < size(); ) {
                Object o2 = elementAt (i);
                if (o2 != null && o != null && o.equals (o2))
                    removeElementAt (i);
                else
                    ++i;
            }
        }
    }
}

public class DictBrowser extends BorderPanel {
public static final int SUCC = 0;
public static final int PRED = 1;
public static final int STARTWORD = 0;
public static final int ENDWORD = 1;

private int dir;
private String startword;
private Vector rels;
private Vector words;
private Vector sels;
private int offs;

private Scrollbar sb;
private Choice dirchoice;
private TextField wordentry;
private ListBox l1, l2, l3;
private int dirty;
private boolean showsyns, cyclicrels;
private static final int CLEAN = 4;

public DictBrowser (String word, int whichword, Vector rels, int dir)
    {
        super (BorderPanel.RIDGE);

        setLayout (new GridBagLayout());

        this.dir = dir;
        this.rels = new Vector ();
        this.words = new Vector();
        this.sels = new Vector();
        this.offs = 0;
        this.dirty = CLEAN;
        this.showsyns = false;
        this.cyclicrels = false;

        for (int i = 0; i < rels.size(); ++i) {
            if (((String)rels.elementAt (i)).equals ("SYNONYM")) {
                showsyns = true;
            } else {
                this.rels.addElement (rels.elementAt (i));
            }
        }

        sb = new Scrollbar (Scrollbar.HORIZONTAL);
        l1 = new ListBox ();
        l2 = new ListBox ();
        l3 = new ListBox ();

        dirchoice = new Choice ();
        dirchoice.addItem ("predecessors");
        dirchoice.addItem ("successors");
        dirchoice.select (dir == PRED ? 0 : 1);
        wordentry = new TextField (word);

        Panel p = new Panel ();
        p.setLayout (new GridBagLayout ());
        GlobalVars.constrain (p, new Label ("browse"), 0, 0, 1, 1,
                              GridBagConstraints.NONE, 0, 1.0);
        GlobalVars.constrain (p, dirchoice, 1, 0, 1, 1,
                              GridBagConstraints.BOTH,
                              GridBagConstraints.CENTER,
                              0, 1.0,
                              0, 0, 0, 5);
        GlobalVars.constrain (p, new Label ("of"), 2, 0, 1, 1,
                              GridBagConstraints.NONE, 0, 1.0);
        GlobalVars.constrain (p, wordentry, 3, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        GlobalVars.constrain (this, p, 0, 0, 3, 1,
                              GridBagConstraints.BOTH, 1.0, 0.0);
        GlobalVars.constrain (this, l1, 0, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, l2, 1, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, l3, 2, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, sb, 0, 2, 3, 1,
                              GridBagConstraints.BOTH, 1.0, 0.0);

        SetWord (word, whichword);
    }

private void ConfigScroll ()
    {
        sb.setValues (offs, 3, 0, words.size()-3);
        sb.setPageIncrement (3);
        sb.setLineIncrement (1);
    }

    
private void RedrawBox (ListBox l, int idx)
    {
        Vector v = (Vector)words.elementAt (idx);
        l.setItems (v);
        int sel = ((Integer)sels.elementAt (idx)).intValue();
        if (sel >= 0) {
            l.select (sel);
            l.makeVisible (sel);
        }
    }

private void RedrawAll ()
    {
        if (dirty <= 0)
            RedrawBox (l1, offs);
        if (dirty <= 1)
            RedrawBox (l2, offs+1);
        if (dirty <= 2)
            RedrawBox (l3, offs+2);
        dirty = CLEAN;
    }

private void Fill ()
    {
        for (int n = words.size() - offs; n < 3; ++n) {
            words.addElement (new Vector());
            sels.addElement (new Integer (-1));
        }
    }

public boolean SetWord (String w, int which)
    {
        if (w == null)
            w = "something";

        offs = 0;
        dirty = 0;
        words.removeAllElements ();
        sels.removeAllElements ();

        Vector v = new Vector ();
        AddElement (v, w);

        if (which == STARTWORD) {
            words.addElement (v);
            sels.addElement (new Integer (0));
            words.addElement (GetPred (w, words));
            sels.addElement (new Integer (-1));
        } else {
            Vector rwords = new Vector ();
            Vector rsels = new Vector ();
            rwords.addElement (v);
            while (true) {
                Vector lastvec = (Vector)rwords.lastElement();
                Vector pred = null;
                int i, max;

                max = (rwords.size() == 1) ? 1 : lastvec.size();
                for (i = 0; i < max; ++i) {
                    String e = (String)lastvec.elementAt (i);
                    if (e != null) {
                        pred = GetSucc (e, rwords);
                        if (pred.size() != 0)
                            break;
                    }
                }
                if (i >= max) {
                    rsels.addElement (new Integer (0));
                    break;
                }
                rwords.addElement (pred);
                rsels.addElement (new Integer (i));
            }
            for (int i = rwords.size(); --i >= 0; ) {
                words.addElement (rwords.elementAt (i));
                sels.addElement (rsels.elementAt (i));
            }
            words.addElement (GetPred (w, words));
            sels.addElement (new Integer (-1));
            offs = words.size() - 2;
            if (offs < 0)
                offs = 0;
        }
        Fill ();
        RedrawAll ();
        ConfigScroll ();

        startword = w;
        return true;
    }

public void SetDir (int dir)
    {
        if (this.dir != dir) {
            this.dir = dir;
            SetWord (startword, STARTWORD);
        }
    }

public String GetSelection ()
    {
        return startword != null ? startword : "";
    }
    
private Vector GetSucc (String w, Vector seen)
    {
        Set res = new Set();

        try {
            for (int i = 0; i < rels.size(); ++i) {
                Vector v = (dir == PRED)
                    ? GlobalVars.proxy.getRelationSucc (
                        w, (String)rels.elementAt (i))
                    : GlobalVars.proxy.getRelationPred (
                        w, (String)rels.elementAt (i));
                res.Add (v, i>0);
            }
            if (cyclicrels) {
                for (int i = 0; i < seen.size(); ++i) {
                    res.Sub ((Vector)seen.elementAt (i));
                }
            }
            if (!showsyns)
                return res;
            Vector v = new Vector ();
            for (int i = 0; i < res.size(); ++i) {
                AddElement (v, (String)res.elementAt (i));
            }
            return v;
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
            return new Vector();
        }
    }

private Vector GetPred (String w, Vector seen)
    {
        Set res = new Set();

        try {
            for (int i = 0; i < rels.size(); ++i) {
                Vector v = (dir == SUCC)
                    ? GlobalVars.proxy.getRelationSucc (
                        w, (String)rels.elementAt (i))
                    : GlobalVars.proxy.getRelationPred (
                        w, (String)rels.elementAt (i));
                res.Add (v, i>0);
            }
            if (cyclicrels) {
                for (int i = 0; i < seen.size(); ++i) {
                    res.Sub ((Vector)seen.elementAt (i));
                }
            }
            if (!showsyns)
                return res;
            Vector v = new Vector ();
            for (int i = 0; i < res.size(); ++i) {
                AddElement (v, (String)res.elementAt (i));
            }
            return v;
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
            return new Vector();
        }
    }

private void AddElement (Vector v, String s)
    {
        if (showsyns && v.size () > 0) {
            // separator
            v.addElement (null);
        }
        v.addElement (s);
        if (showsyns) {
            try {
                Vector syns = GlobalVars.proxy.getRelationAllPred (
                    s, "SYNONYM");
                for (int i = 0; i < syns.size(); ++i) {
                    v.addElement (syns.elementAt (i));
                }
            } catch (AITException e) {
                GlobalVars.DisplayError (e.getErrMsg());
            }
        }
    }

private void NotifyParent (String s)
    {
        getParent().postEvent (new Event (this, Event.ACTION_EVENT, s));
    }
    
private int List2Index (ListBox l)
    {
        if (l == l1)
            return 0;
        if (l == l2)
            return 1;
        if (l == l3)
            return 2;
        System.out.println ("DictBrowser::List2Index: bad listbox");
        return offs;
    }

private ListBox Index2List (int idx)
    {
        switch (idx) {
        case 0:
            return l1;
        case 1:
            return l2;
        case 2:
            return l3;
        default:
            System.out.println ("DictBrowser::Index2List: bad index");
            break;
        }
        return null;
    }

private void ListSelected (ListBox l, int selection)
    {
        int idx = List2Index (l);
        dirty = idx+1;
        for (int i = words.size() - (offs+idx+1); i > 0; --i) {
            words.removeElementAt (offs+idx+1);
            sels.removeElementAt (offs+idx+1);
        }
        sels.setElementAt (new Integer (selection), idx+offs);
        String s = l.getItem (selection);
        startword = s;
        words.addElement (GetPred (s, words));
        sels.addElement (new Integer (-1));
        if (idx == 2) {
            dirty = 0;
            ++offs;
        }
        Fill ();
        RedrawAll ();
        ConfigScroll ();
        NotifyParent (s);
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.LIST_SELECT:
            ListSelected ((ListBox)ev.target, ((Integer)ev.arg).intValue());
            return true;

        case Event.ACTION_EVENT:
            if (ev.target == dirchoice) {
                if (((String)ev.arg).equals ("predecessors"))
                    SetDir (PRED);
                else
                    SetDir (SUCC);
                return true;
            } else if (ev.target == wordentry) {
                SetWord ((String)ev.arg, STARTWORD);
                NotifyParent ((String)ev.arg);
                return true;
            }
            break;

        case Event.SCROLL_LINE_UP:
        case Event.SCROLL_LINE_DOWN:
        case Event.SCROLL_PAGE_UP:
        case Event.SCROLL_PAGE_DOWN:
        case Event.SCROLL_ABSOLUTE:
            offs = ((Integer)ev.arg).intValue();
            dirty = 0;
            RedrawAll ();
            return true;
        }
        return false;
    }
}
