/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.net.*;

import GlobalVars;
import ListBox;
import EntryBox;

class EditMatchingRules extends Dialog implements EntryBoxHandler {
private Button done, commit, newrule, delrule;
private ListBox list;
private TextArea text;
private String currule, currulename;

public EditMatchingRules ()
    {
        super (GlobalVars.frame, true);
        setLayout (new GridBagLayout());

        done = new Button ("Done");
        commit = new Button ("Commit");
        newrule = new Button ("New rule");
        delrule = new Button ("Delete rule");
        Panel p = new Panel ();

        p.add ("West", done);
        p.add ("West", commit);
        p.add ("West", newrule);
        p.add ("West", delrule);

        list = new ListBox ();
        text = new TextArea (20, 60);
        text.setFont (new Font ("fixed", Font.PLAIN, 14));

        GlobalVars.constrain (this, list, 0, 0, 1, 1,
                              GridBagConstraints.BOTH, 0.0, 1.0);
        GlobalVars.constrain (this, text, 1, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, p, 0, 1, 2, 1,
                              GridBagConstraints.BOTH, 1.0, 0.0);
    }

private void Update (boolean dolist)
    {
        currulename = list.getSelectedItem ();
        try {
            if (dolist) {
                Vector rules = GlobalVars.proxy.getMatchingRuleNames ();
                list.setItems (rules);
                if (currulename != null) {
                    int i;
                    for (i = 0; i < rules.size(); ++i) {
                        if (currulename.equals ((String)rules.elementAt(i)))
                            break;
                    }
                    if (i < rules.size())
                        list.select (i);
                    else if (rules.size() > 0) {
                        list.select (0);
                        currulename = (String)rules.elementAt (0);
                    } else {
                        currulename = null;
                    }
                } else if (rules.size() > 0) {
                    list.select (0);
                    currulename = (String)rules.elementAt (0);
                } else {
                    currulename = null;
                }
            }
            if (currulename != null) {
                currule = GlobalVars.proxy.getMatchingRule (currulename);
                text.setText (currule);
            }
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
        }
    }

public void show ()
    {
        Update (true);
        super.show ();
    }

public void entryBoxHandler (EntryBox eb, Object arg)
    {
        if (eb.GetButton () == 0) {
            try {
                GlobalVars.proxy.defineMatchingRule (
                    eb.GetText(), "% new matching rule\n");
                Update (true);
            } catch (AITException e) {
                GlobalVars.DisplayError (e.getErrMsg());
            }
        }
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.ACTION_EVENT:
            if (ev.target == done) {
                hide ();
                return true;
            } else if (ev.target == commit) {
                String s = text.getText ();
                if (currulename != null && !s.equals (currule)) {
                    try {
                        GlobalVars.proxy.defineMatchingRule (currulename, s);
                    } catch (AITException e) {
                        GlobalVars.DisplayError (e.getErrMsg());
                    }
                }
                return true;
            } else if (ev.target == newrule) {
                EntryBox eb = new EntryBox ("New matching rule",
                                            "Rule name:", "newrule");
                eb.pack ();
                eb.show ();
                return true;
            } else if (ev.target == delrule) {
                if (currulename != null) {
                    try {
                        GlobalVars.proxy.deleteMatchingRule (currulename);
                        Update (true);
                    } catch (AITException e) {
                        GlobalVars.DisplayError (e.getErrMsg());
                    }
                }
                return true;
            }
            break;

        case Event.LIST_SELECT:
            if (ev.target == list) {
                Update (false);
                return true;
            }
        }
        return false;
    }
}
