/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.net.*;

import GlobalVars;
import AlertBox;
import CGViewer;

public class ExportGraph extends Dialog implements EntryBoxHandler {
private Button ok, show, addnew, join;
private Vector keys;
private CGViewer cgview;
private ConceptualGraph cg;

public ExportGraph (ConceptualGraph cg)
    {
        super (GlobalVars.frame, true);

        this.cg = cg;

        try {
            keys = GlobalVars.proxy.match (cg, GlobalVars.matchrules,
                                           GlobalVars.profilekey);
            MatchResult.SortByMatchVal (keys);
            if (keys.size() == 0) {
                String roottype = cg.getNode().getName();
                int n = GlobalVars.proxy.getNumberOfVariants (roottype);
                for (int i = 0; i < n; ++i) {
                    keys.addElement (
                        GlobalVars.proxy.getNthVariant (roottype, i));
                }
            }
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
            return;
        }

        setLayout (new GridBagLayout ());

        cgview = new CGViewer (new Dimension (300, 300));
        GlobalVars.constrain (this, cgview, 0, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        Panel panel = new Panel ();
        GlobalVars.constrain (this, panel, 0, 1, 1, 1,
                              GridBagConstraints.NONE,
                              GridBagConstraints.CENTER,
                              0.0, 0.0, 0, 0, 2, 2);

        ok = new Button ("Dismiss");
        show = new Button ("Show OID");
        addnew = new Button ("Add new");
        join = new Button ("Join to selection");
        panel.add ("Center", ok);
        panel.add ("Center", show);
        panel.add ("Center", addnew);
        if (keys.size() > 0) {
            panel.add (join);
        }
    }

public void entryBoxHandler (EntryBox eb, Object arg)
    {
        if (eb.GetButton() != 0) {
            hide();
        } else {
            addnew.disable();
            join.disable();
            try {
                String key = GlobalVars.proxy.add (cg);
                GlobalVars.proxy.addProvider (key, "(no maintainer)",
                                              eb.GetText());
                keys = GlobalVars.proxy.match (cg, GlobalVars.matchrules,
                                               GlobalVars.profilekey);
                cgview.SetKeys (MatchResult.SortByMatchVal (keys));
            } catch (AITException e) {
                GlobalVars.DisplayError (e.getErrMsg());
                hide ();
            }
        }
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.ACTION_EVENT:
            if (ev.target == ok) {
                hide ();
                return true;
            } else if (ev.target == show) {
                String oid = cgview.GetSelectionOID ();
                if (oid != null)
                    GlobalVars.ShowDocument (oid);
                return true;
            } else if (ev.target == addnew) {
                EntryBox eb = new EntryBox ("Enter OID", "OID:", "noObjID");
                eb.pack();
                eb.show();
                return true;
            } else if (ev.target == join) {
                String key = cgview.GetSelectionKey();
                addnew.disable();
                join.disable();
                try {
                    GlobalVars.proxy.join (key, cg);
                    cgview.SetKeys (
                        GlobalVars.proxy.match (cg, GlobalVars.matchrules,
                                                GlobalVars.profilekey));
                } catch (AITException e) {
                    GlobalVars.DisplayError (e.getErrMsg());
                    hide ();
                }
                return true;
            }
            break;
        }
        return false;
    }

public void show ()
    {
        cgview.SetKeys (keys);
        super.show ();
    }
}
