/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.awt.*;
import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

import AIT;
import EditLinearForm;
import Proxy;
import ShowImage;

public class GlobalVars {
public static EditLinearForm editlinform = null;
public static AIT applet = null;
public static Frame frame = null;
public static Label status = null;
public static Proxy proxy = null;
public static TransportLayer transport = null;
public static Vector matchrules = new Vector();
public static String serverhost = null;
public static String profilekey;

public static void CloseAll ()
    {
        if (editlinform != null) {
            editlinform.hide();
            editlinform = null;
        }
        if (frame != null) {
            frame.hide();
            frame = null;
            if (applet != null)
                applet.FrameClosed();
        }
        try {
            if (profilekey != "")
                proxy.removeProfile (profilekey);
        } catch (AITException ex) {
        }
        if (transport != null)
            transport.disconnect();
        proxy = null;
    }

public static void SetStatus (String txt)
    {
        if (status != null)
            status.setText (txt);
    }

public static void DisplayError (String txt)
    {
        AlertBox a = new AlertBox ("AIT Error", txt);
        a.pack();
        a.show();
    }

private static String GetPath (String prog)
    {
        int retval;

        try {
            Process p = Runtime.getRuntime().exec ("/usr/bin/which "+prog);
            while (true) {
                try {
                    retval = p.waitFor ();
                    break;
                } catch (InterruptedException e) {
                } catch (Exception e) {
                    return prog;
                }
            }
            if (retval != 0) {
                return prog;
            }
            DataInputStream inp = new DataInputStream (p.getInputStream());
            return inp.readLine();
        } catch (Exception e) {
            return prog;
        }
    }
    
public static boolean ShowDocument (String oid)
    {
        int retval;
        URL u;

        try {
            u = new URL (oid);
        } catch (MalformedURLException e) {
            try {
                ShowImage si = new ShowImage (oid);
                si.pack();
                si.show();
                return true;
            } catch (NoSuchElementException e2) {
                DisplayError ("No such file: '" + oid + "'");
                return false;
            }
        }

        try {
            Process p = Runtime.getRuntime().exec (
                GetPath("netscape") + " -remote openURL("+oid+",new-window)");
            while (true) {
                try {
                    retval = p.waitFor ();
                    break;
                } catch (InterruptedException e) {
                } catch (Exception e) {
                    DisplayError ("error waiting for netscape");
                    return false;
                }
            }
            if (retval != 0) {
                DataInputStream inp = new DataInputStream (p.getErrorStream());
                DisplayError (inp.readLine());
                return false;
            }
        } catch (SecurityException e) {
            applet.getAppletContext().showDocument (u);
        } catch (Exception e) {
            DisplayError ("error showing document");
            return false;
        }
        return true;
    }

public static void constrain (
    Container container, Component component,
    int grid_x, int grid_y,
    int grid_width, int grid_height,
    int fill, int anchor, double weight_x, double weight_y,
    int top, int left, int bottom, int right)
    {
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = grid_x; c.gridy = grid_y;
        c.gridwidth = grid_width; c.gridheight = grid_height;
        c.fill = fill; c.anchor = anchor;
        c.weightx = weight_x; c.weighty = weight_y;
        if( top + bottom + left + right > 0 )
            c.insets = new Insets( top, left, bottom, right );
        ((GridBagLayout) container.getLayout()).setConstraints( component, c );
        container.add( component );
    }

public static void constrain (
    Container container, Component component,
    int gridx, int gridy, int gridw, int gridh,
    int fill,
    double wx, double wy)
    {
        constrain (container, component, gridx, gridy, gridw, gridh,
                   fill, GridBagConstraints.NORTHWEST, wx, wy,
                   0, 0, 0, 0);
    }
}
