/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.net.*;

import GlobalVars;
import AlertBox;
import CGViewer;
import MatchResult;

public class MatchGraph extends Dialog {
private Button ok, show;
private Vector keys;
private CGViewer cgview;

public MatchGraph (ConceptualGraph cg)
    {
        super (GlobalVars.frame, true);

        try {
            keys = GlobalVars.proxy.match (cg, GlobalVars.matchrules,
                                           GlobalVars.profilekey);
            MatchResult.SortByMatchVal (keys);
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
            return;
        }

        setLayout (new GridBagLayout ());

        cgview = new CGViewer (new Dimension (300, 300));
        GlobalVars.constrain (this, cgview, 0, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        Panel panel = new Panel ();
        GlobalVars.constrain (this, panel, 0, 1, 1, 1,
                              GridBagConstraints.NONE,
                              GridBagConstraints.CENTER,
                              0.0, 0.0, 0, 0, 2, 2);

        ok = new Button ("Done");
        show = new Button ("Show OID");
        panel.add ("Center", ok);
        panel.add ("Center", show);
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.ACTION_EVENT:
            if (ev.target == ok) {
                hide ();
                return true;
            } else if (ev.target == show) {
                String oid = cgview.GetSelectionOID ();
                if (oid != null)
                    GlobalVars.ShowDocument (oid);
            }
            break;
        }
        return false;
    }

public void show ()
    {
        cgview.SetKeys (keys);
        super.show ();
    }
}
