/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;

import GlobalVars;
import Scale;

class ProfileBox extends Dialog {
private Scale scale;
private Button ok, cancel;
private Choice onoff;
private TextField timeout;

public ProfileBox ()
    {
        super (GlobalVars.frame, "AI-Trader - User Profiles", true);
        setLayout (new GridBagLayout());

        int minmatchval;
        int tmo;
        try {
            minmatchval =
                GlobalVars.proxy.getMinMatchval (GlobalVars.profilekey);
            tmo =
                GlobalVars.proxy.getTimeout (GlobalVars.profilekey);
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
            return;
        }

        Label l1 = new Label ("minimum match value:");
        scale = new Scale (1, 100, 1, minmatchval);

        GlobalVars.constrain (this, l1, 0, 0, 1, 1,
                              GridBagConstraints.NONE,
                              GridBagConstraints.WEST, 0, 0,
                              2, 2, 2, 2);
        GlobalVars.constrain (this, scale, 1, 0, 2, 1,
                              GridBagConstraints.BOTH,
                              GridBagConstraints.CENTER, 1.0, 0,
                              2, 2, 2, 2);

        Label l2 = new Label ("timeout:");
        onoff = new Choice ();
        onoff.addItem ("off");
        onoff.addItem ("on");
        timeout = new TextField ("");
        if (tmo <= 0) {
            onoff.select (0);
            timeout.setText ("0");
            timeout.disable();
        } else {
            onoff.select (1);
            timeout.setText (Integer.toString (tmo));
        }
        GlobalVars.constrain (this, l2, 0, 1, 1, 1,
                              GridBagConstraints.NONE,
                              GridBagConstraints.WEST, 0, 0,
                              2, 2, 2, 2);
        GlobalVars.constrain (this, onoff, 1, 1, 1, 1,
                              GridBagConstraints.BOTH,
                              GridBagConstraints.WEST, 0, 0,
                              2, 2, 2, 2);
        GlobalVars.constrain (this, timeout, 2, 1, 1, 1,
                              GridBagConstraints.BOTH,
                              GridBagConstraints.WEST, 1.0, 0,
                              2, 2, 2, 2);
        
        Panel p2 = new Panel();
        ok = new Button ("Ok");
        cancel = new Button ("Cancel");
        p2.add ("West", ok);
        p2.add ("West", cancel);

        GlobalVars.constrain (this, p2, 0, 2, 3, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        Dimension d = getLayout().minimumLayoutSize(this);
        resize (d.width, d.height);
    }
    
public boolean handleEvent (Event e)
    {
        switch (e.id) {
        case Event.ACTION_EVENT:
            if (e.target == ok) {
                dispose ();
                try {
                    int minmatchval = scale.getValue();
                    GlobalVars.proxy.setMinMatchval (GlobalVars.profilekey,
                                                     minmatchval);
                    int tmo = 0;
                    if (onoff.getSelectedIndex() == 1)
                        tmo = Integer.parseInt (timeout.getText());
                    GlobalVars.proxy.setTimeout (GlobalVars.profilekey,
                                                 tmo);
                } catch (AITException ex) {
                    GlobalVars.DisplayError (ex.getErrMsg());
                }
                return true;
            } else if (e.target == onoff) {
                if (((String)e.arg).equals ("on")) {
                    timeout.enable();
                } else {
                    timeout.disable();
                }
            } else if (e.target == cancel) {
                dispose ();
                return true;
            }
            break;
        }
        return false;
    }
}
