/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

/*
 * automatic WWW indexer, (w) 1997 Kay Roemer.
 *
 * $Id: Robot.java,v 1.6 1997/09/13 15:04:16 roemer Exp $
 */

import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.*;

import Proxy;
import CGParser;
import outFrame;


class NamedValue {
public String name, value;
public NamedValue (String n, String v)
    {
        name = n;
        value = v;
    }

public NamedValue (String n)
    {
        name = n;
        value = null;
    }

public boolean check (String n, String v)
    {
        return (name.equalsIgnoreCase (n) && value != null &&
                value.equalsIgnoreCase (v));
    }

public boolean check (String n, boolean needvalue)
    {
        return (name.equalsIgnoreCase (n) &&
                (needvalue == false || value != null));
    }
}


class HTMLParser {
private DataInputStream is;
private Vector refs, cgs;
private int tok, ungetchar;
private URL url;
private static Hashtable spcChars = null;

private final static int
    TOK_SPACE = ' ',
    TOK_TAB   = '\t',
    TOK_NL    = '\n',
    TOK_LT    = '<',
    TOK_GT    = '>',
    TOK_AMP   = '&',
    TOK_SEMICO= ';',
    TOK_QUOT  = '"',
    TOK_SQUOT = '\'',
    TOK_EQU   = '=',
    TOK_EOF   = -1,
    TOK_NONE  = -2;

public HTMLParser (URL u)
    {
        refs = new Vector();
        cgs = new Vector();
        tok = ungetchar = TOK_NONE;
        url = u;

        if (spcChars != null)
            return;

        spcChars = new Hashtable ();
        spcChars.put ("AElig",  new Integer (198));
        spcChars.put ("Aacute", new Integer (193));
        spcChars.put ("Acirc",  new Integer (194));
        spcChars.put ("Agrave", new Integer (192));
        spcChars.put ("Aring",  new Integer (197));
        spcChars.put ("Atilde", new Integer (195));
        spcChars.put ("Auml",   new Integer (196));
        spcChars.put ("Ccedil", new Integer (199));
        spcChars.put ("ETH",    new Integer (208));
        spcChars.put ("Eacute", new Integer (201));
        spcChars.put ("Ecirc",  new Integer (202));
        spcChars.put ("Egrave", new Integer (200));
        spcChars.put ("Euml",   new Integer (203));
        spcChars.put ("Iacute", new Integer (205));
        spcChars.put ("Icirc",  new Integer (206));
        spcChars.put ("Igrave", new Integer (204));
        spcChars.put ("Iuml",   new Integer (207));
        spcChars.put ("Ntilde", new Integer (209));
        spcChars.put ("Oacute", new Integer (211));
        spcChars.put ("Ocirc",  new Integer (212));
        spcChars.put ("Ograve", new Integer (210));
        spcChars.put ("Oslash", new Integer (216));
        spcChars.put ("Otilde", new Integer (213));
        spcChars.put ("Ouml",   new Integer (214));
        spcChars.put ("THORN",  new Integer (222));
        spcChars.put ("Uacute", new Integer (218));
        spcChars.put ("Ucirc",  new Integer (219));
        spcChars.put ("Ugrave", new Integer (217));
        spcChars.put ("Uuml",   new Integer (220));
        spcChars.put ("Yacute", new Integer (221));
        spcChars.put ("aacute", new Integer (225));
        spcChars.put ("acirc",  new Integer (226));
        spcChars.put ("acute",  new Integer (180));
        spcChars.put ("aelig",  new Integer (230));
        spcChars.put ("agrave", new Integer (224));
        spcChars.put ("amp",    new Integer ('&'));
        spcChars.put ("aring",  new Integer (229));
        spcChars.put ("atilde", new Integer (227));
        spcChars.put ("auml",   new Integer (228));
        spcChars.put ("brvbar", new Integer (166));
        spcChars.put ("ccedil", new Integer (231));
        spcChars.put ("cedil",  new Integer (184));
        spcChars.put ("cent",   new Integer (162));
        spcChars.put ("copy",   new Integer (169));
        spcChars.put ("curren", new Integer (164));
        spcChars.put ("deg",    new Integer (176));
        spcChars.put ("divide", new Integer (247));
        spcChars.put ("eacute", new Integer (233));
        spcChars.put ("ecirc",  new Integer (234));
        spcChars.put ("egrave", new Integer (232));
        spcChars.put ("eth",    new Integer (240));
        spcChars.put ("euml",   new Integer (235));
        spcChars.put ("frac12", new Integer (189));
        spcChars.put ("frac14", new Integer (188));
        spcChars.put ("frac34", new Integer (190));
        spcChars.put ("gt",	new Integer ('>'));
        spcChars.put ("iacute", new Integer (237));
        spcChars.put ("icirc",  new Integer (238));
        spcChars.put ("iexcl",  new Integer (161));
        spcChars.put ("igrave", new Integer (236));
        spcChars.put ("iquest", new Integer (191));
        spcChars.put ("iuml",   new Integer (239));
        spcChars.put ("laquo",  new Integer (171));
        spcChars.put ("lt",	new Integer ('<'));
        spcChars.put ("macr",   new Integer (175));
        spcChars.put ("micro",  new Integer (181));
        spcChars.put ("middot", new Integer (183));
        spcChars.put ("nbsp",   new Integer (160));
        spcChars.put ("not",    new Integer (172));
        spcChars.put ("ntilde", new Integer (241));
        spcChars.put ("oacute", new Integer (243));
        spcChars.put ("ocirc",  new Integer (244));
        spcChars.put ("ograve", new Integer (242));
        spcChars.put ("ordf",   new Integer (170));
        spcChars.put ("ordm",   new Integer (186));
        spcChars.put ("oslash", new Integer (248));
        spcChars.put ("otilde", new Integer (245));
        spcChars.put ("ouml",   new Integer (246));
        spcChars.put ("para",   new Integer (182));
        spcChars.put ("plusmn", new Integer (177));
        spcChars.put ("pound",  new Integer (163));
        spcChars.put ("quot",   new Integer ('"'));
        spcChars.put ("raquo",  new Integer (187));
        spcChars.put ("reg",    new Integer (174));
        spcChars.put ("sect",   new Integer (167));
        spcChars.put ("shy",    new Integer (173));
        spcChars.put ("sup1",   new Integer (185));
        spcChars.put ("sup2",   new Integer (178));
        spcChars.put ("sup3",   new Integer (179));
        spcChars.put ("szlig",  new Integer (223));
        spcChars.put ("thorn",  new Integer (254));
        spcChars.put ("times",  new Integer (215));
        spcChars.put ("uacute", new Integer (250));
        spcChars.put ("ucirc",  new Integer (251));
        spcChars.put ("ugrave", new Integer (249));
        spcChars.put ("uml",    new Integer (168));
        spcChars.put ("uuml",   new Integer (252));
        spcChars.put ("yacute", new Integer (253));
        spcChars.put ("yen",    new Integer (165));
        spcChars.put ("yuml",   new Integer (255));
    }

public boolean parse (outFrame outframe)
    {
        try {
            URLConnection conn = url.openConnection();
            String type = conn.getContentType();
            if (!type.equalsIgnoreCase ("text/html")) {
               outframe.appendText ("no html: " + url.toString() + "\n");
                return false;
            }
            is = new DataInputStream (url.openStream ());
            for (int c; (c = getToken ()) != TOK_EOF; ) {
                eatToken ();
                if (c == TOK_LT)
                    parseTag ();
                else
                    parseChar (c);
            }
            is.close ();
        } catch (IOException e) {
            outframe.appendText ("cannot connect to: " + url.toString() + "\n");
            return false;
        }
        return true;
    }

public Vector getRefs ()
    {
        return refs;
    }

public Vector getCGs ()
    {
        return cgs;
    }

private boolean isSpace (int c)
    {
        return c == TOK_SPACE || c == TOK_TAB || c == TOK_NL;
    }

private boolean isDigit (int c)
    {
        return c >= '0' && c <= '9';
    }
    
private int getChar ()
    {
        if (ungetchar != TOK_NONE) {
            int c = ungetchar;
            ungetchar = TOK_NONE;
            return c;
        }            
        try {
            return ((int)is.readByte()) & 0xff;
        } catch (EOFException e) {
            return TOK_EOF;
        } catch (IOException e) {
            return TOK_EOF;
        }
    }

private void ungetChar (int c)
    {
        ungetchar = c;
    }

private int getToken ()
    {
        int c;
        if (tok != TOK_NONE)
            return tok;
        if ((tok = getChar()) == TOK_EOF)
            return tok;
        if (!isSpace (tok))
            return tok;
        while ((c = getChar()) != TOK_EOF && isSpace(c))
            ;
        ungetChar (c);
        return tok;
    }

private void eatToken ()
    {
        tok = TOK_NONE;
    }

private int skip (int token)
    {
        int c;
        while ((c = getToken()) == token)
            eatToken();
        return c;
    }
    
private int parseSpecialChar ()
    {
        int c;
        StringBuffer name = new StringBuffer();

        while ((c = getToken()) != TOK_EOF && c != TOK_SPACE &&
               c != TOK_SEMICO) {
            eatToken();
            name.append ((char)c);
        }
        eatToken ();
        String sname = name.toString();

        if (c != TOK_SEMICO || sname.length() == 0) {
            // hmm...
            return TOK_SPACE;
        }

        // see if its a numeric value ...
        if ((int)sname.charAt(0) == '#') {
            c = Integer.parseInt (sname.substring (1));
            return c != 0 ? c : TOK_SPACE;
        } else if (isDigit ((int)sname.charAt(0))){
            c = Integer.parseInt (sname);
            return c != 0 ? c : TOK_SPACE;
        }

        // lookup special char ...
        Integer i = (Integer)spcChars.get(sname);
        return ((i != null) ? i.intValue() : TOK_SPACE);
    }

private int parseChar (int c)
    {
        return c != TOK_AMP ? c : parseSpecialChar();
    }
    
private String parseName ()
    {
        int c, end;
        StringBuffer sbuf = new StringBuffer ();

        c = skip (TOK_SPACE);
        switch (c) {
        case TOK_EOF:
            // missing >
            eatToken();
            return null;

        case TOK_GT:
            // no more name
            return null;

        case TOK_QUOT:
            // value enclosed in double quotes
            eatToken();
            end = TOK_QUOT;
            break;

        case TOK_SQUOT:
            // value enclosed in single quotes
            eatToken();
            end = TOK_SQUOT;
            break;

        default:
            // single word value
            eatToken();
            sbuf.append ((char)parseChar (c));
            end = TOK_SPACE;
        }
        while ((c = getToken ()) != TOK_EOF && c != end &&
               (end != TOK_SPACE || (c != TOK_GT && c != TOK_EQU))) {
            eatToken ();
            sbuf.append ((char)parseChar (c));
        }
        if (c != TOK_GT && c != TOK_EQU)
            eatToken ();
        return sbuf.toString();
    }

private int parseComment ()
    {
        int c, lastc = 0;

        while (true) {
            while ((c = getToken()) != TOK_EOF) {
                eatToken ();
                if (lastc == '-' && c == '-')
                    break;
                lastc = c;
            }
            if (c == TOK_EOF)
                return -1;

            c = skip (TOK_SPACE);
            eatToken();
            if (c == TOK_EOF)
                return -1;

            if (c == TOK_GT)
                return 0;
        }
    }

private int parseTag ()
    {
        Vector args = new Vector();
        String name, value;
        int c, i = 0;
        boolean bad = false;

        while (true) {
            name = parseName ();
            if (name == null) {
                bad = true;
                break;
            }
            if (++i == 1 && name.startsWith ("!--"))
                return parseComment();

            c = skip (TOK_SPACE);
            if (c == TOK_EOF)
                return -1;

            if (c == TOK_EQU) {
                eatToken ();
                value = parseName ();
                c = skip (TOK_SPACE);
                if (c == TOK_EOF)
                    return -1;
            } else {
                value = null;
            }
            args.addElement (new NamedValue (name, value));
            if (c == TOK_GT) {
                eatToken ();
                break;
            }
        }
        if (bad) {
            // something went wrong ...
            while ((c = getToken ()) != TOK_EOF && c != TOK_GT)
                eatToken ();
            eatToken ();
            return -1;
        } else {
            return evalTag (args);
        }
    }

private void dumpTag (Vector args)
    {
        NamedValue nv;

        System.out.print ("<");
        for (int i = 0; i < args.size(); ++i) {
            if (i > 0)
                System.out.print (" ");
            nv = (NamedValue)args.elementAt(i);
            System.out.print (nv.name);
            if (nv.value != null)
                System.out.print ("=\"" + nv.value + "\"");
        }
        System.out.println (">");
    } // dumpTag


private int evalTag (Vector args)
    {
        if (args.size() < 1)
            return 0;

        NamedValue nv;
        
        nv = (NamedValue)args.elementAt(0);
        if (nv.check ("A", false)) {
            // ein A tag
            for (int i = 1; i < args.size(); ++i) {
                nv = (NamedValue)args.elementAt(i);
                if (nv.check ("HREF", true)) {
                    int idx = nv.value.indexOf ('#');
                    String href = (idx < 0)
                        ? nv.value
                        : nv.value.substring (0, idx);
                    refs.addElement (href);
                    break;
                }
            }
        } else if (nv.check ("META", false)) {
            // ein META tag
            String cg = null;
            boolean iscg = false;
            for (int i = 1; i < args.size(); ++i) {
                nv = (NamedValue)args.elementAt(i);
                if (nv.check ("NAME", "ConceptualGraph"))
                    iscg = true;
                else if (nv.check ("CONTENT", true))
                    cg = nv.value;
                if (cg != null && iscg) {
                    cgs.addElement (cg);
                    break;
                }
            }
        }
        return 0;

    } // evalTag
}


public class Robot {
private static String aitserver = "localhost";
private static boolean dokill = false;
private static Proxy aitproxy;
private static TransportLayer aittransport;
private static InetAddress startaddr;
private static Hashtable visited;
private static Vector mrules;
private static outFrame outframe;
private static String profilekey;

public static void main (String args[])
    {
        outframe = new outFrame ("Robot");

        outframe.pack();
        outframe.show();

        int firstarg = evalArgs (args);

        aittransport = new TransportLayer (aitserver);
        if (!aittransport.connected()) {
            outframe.appendText ("cannot connect to AIT server '" + aitserver + "'\n");
            return;
        }
        aitproxy = new Proxy (aittransport);
        try {
            profilekey = aitproxy.createProfile ();
        } catch (AITException e) {
            outframe.appendText ("killAll: " + e.getErrMsg() + "\n");
        }
        visited = new Hashtable ();

        mrules = new Vector ();
        mrules.addElement ("Specialization");
        mrules.addElement ("Negation");
        mrules.addElement ("Quantity");

        if (dokill)
            killAll ();

        for (int i = firstarg; i < args.length; ++i) {
            URL url = null;
            try {
                url = new URL (args[i]);
                startaddr = InetAddress.getByName (url.getHost());
                download (url, outframe);
            } catch (MalformedURLException e) {
                outframe.appendText("cannot create URL: " + args[i] + "\n");
            } catch (UnknownHostException e) {
                outframe.appendText("no ip for host: " + url.getHost() + "\n");
            }
        }
        aittransport.disconnect();
        aitproxy = null;

        outframe.appendText ("\n\nProgram is terminated, please hit the Exit button\n\n");

    } // main


private static int evalArgs (String args[])
    {
        for (int i = 0; i < args.length; ++i) {
            if (args[i].startsWith ("-s") && i+1 < args.length)
                aitserver = args[++i];
            else if (args[i].startsWith ("-k"))
                dokill = true;
            else if (args[i].equals ("--"))
                return i+1;
            else if (!args[i].startsWith ("-"))
                return i;
        }
        return args.length;

    } // evalArgs
    

private static void killAll ()
    {
        try {
            CGParser cgp = new CGParser();
            ConceptualGraph cg = cgp.Parse ("[something]");
            Vector matches = aitproxy.match (cg, mrules, profilekey);
            for (int i = 0; i < matches.size(); ++i) {
                String key = ((MatchResult)matches.elementAt(i)).GetKey();
                aitproxy.replace (key, null);

                outframe.appendText ("killing key: " + key + "\n");
            }
        } catch (AITException e) {
            outframe.appendText ("killAll: " + e.getErrMsg() + "\n");
        } catch (Exception e) {
        }

    } // killAll

private static void export (URL url, ConceptualGraph cg)
    {
        Vector matches;
        String key = null;
        boolean url_already_exported = false;
        boolean cg_already_exported = false;
        
        try {
            matches = aitproxy.match (cg, mrules, profilekey);

            for (int i = 0; i < matches.size(); ++i) {
                MatchResult mres = (MatchResult)matches.elementAt(i);
                if (mres.GetMatchVal() == 100) {
                    key = mres.GetKey();
                    ConceptualGraph cg2 = aitproxy.getGraph (key);
                    if (cg.equals (cg2)) {
                        // identischer Graph schonmal exportiert
                        cg_already_exported = true;
                        Vector serv = aitproxy.getProviders (key);
                        for (int j = 0; j < serv.size(); ++j) {
                            try {
                                ServiceEntry ent =
                                    (ServiceEntry)serv.elementAt(j);
                                URL u = new URL (ent.getOID());
                                if (u.equals (url)) {
                                    // URL auch schon als OID eigetragen
                                    url_already_exported = true;
                                    break;
                                }
                            } catch (MalformedURLException e) {
                            }
                        }
                        break;
                    }
                }
            }
        } catch (AITException e) {
            outframe.appendText("export error: " + e.getErrMsg() + "\n");
            return;
        }
        StringBuffer sb = new StringBuffer();
        cg.print (sb, 2);
        try {
            if (!cg_already_exported) {
                // Graph noch nicht exportiert
                key = aitproxy.add (cg);
                outframe.appendText("exporting cg:" + "\n");
            } else {
                // Graph schon exportiert
                outframe.appendText("already exported cg:" + "\n");
            }
            outframe.appendText(sb.toString() + "\n");

            if (url_already_exported) {
                // URL schon eingetragen
                outframe.appendText("already exported URL: " + url.toString() + "\n");
            } else {
                // URL noch nicht eingetragen
                aitproxy.addProvider (key, "(Robot)", url.toString());
                outframe.appendText("exporting URL: " + url.toString() + "\n");
            }
        } catch (AITException e) {
            outframe.appendText("export error (" + url.toString() + "): " + 
                              e.getErrMsg() + "\n" + sb.toString() + "\n");
        }

    } // export


private static void download (URL url, outFrame outframe)
    {
        if (visited.containsKey (url)) {
            // outframe.appendText("already visited: " + url.toString() + "\n");
            return;
        }
        visited.put (url, mrules);

        try {
            if (!startaddr.equals (InetAddress.getByName (url.getHost()))) {
                outframe.appendText("foreign host: " + url.getHost() + "\n");
                return;
            }
        } catch (UnknownHostException e) {
            outframe.appendText("no ip for host: " + url.getHost() + "\n");
            return;
        }

        if (!url.getProtocol().equalsIgnoreCase ("HTTP")) {
            outframe.appendText("no http: " + url.toString() + "\n");
            return;
        }

        outframe.appendText("\n\t/* ---------------------------- */\n\n" + 
                            "downloading: " + url.toString() + "\n");

        HTMLParser hp = new HTMLParser (url);
        if (!hp.parse (outframe)) {
            return;
        }

        Vector cgs = hp.getCGs ();
        CGParser cgp = new CGParser ();
        for (int i = 0; i < cgs.size(); ++i) {
            String scg = (String)cgs.elementAt(i);
            try {
                ConceptualGraph cg = cgp.Parse (scg);
                export (url, cg);
            } catch (Exception e) {
                outframe.appendText("cannot parse cg: " + scg + "\n");
            }
        }

        Vector refs = hp.getRefs();
        for (int i = 0; i < refs.size(); ++i) {
            String surl = (String)refs.elementAt(i);
            try {
                download (new URL (url, surl), outframe);
            } catch (MalformedURLException e) {
                outframe.appendText("cannot create URL: " + surl + "\n");
            }
        }

    } //download

}
