/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.net.*;

import GlobalVars;

class ImgCanvas extends Canvas {
private Image img;
private Dimension prefsize;

public ImgCanvas (Image img)
        throws NoSuchElementException
    {
        this.img = img;

        MediaTracker m = new MediaTracker (this);
        m.addImage (img, 0);
        try {
            m.waitForID (0);
            if (m.isErrorAny ())
                throw new NoSuchElementException();
            prefsize = new Dimension (img.getWidth (this),
                                      img.getHeight (this));
        } catch (InterruptedException e) {
            prefsize = new Dimension (200, 200);
        }
    }

public Dimension minimumSize ()
    {
        return prefsize;
    }

public Dimension preferredSize ()
    {
        return prefsize;
    }

public void paint (Graphics g)
    {
        g.drawImage (img, 0, 0, getBackground (), this);
    }
}


public class ShowImage extends Dialog {
private ImgCanvas cv;
private Button done;

public ShowImage (String name)
    {
        super (GlobalVars.frame, true);

        Image img = GlobalVars.applet.getImage (
            GlobalVars.applet.getCodeBase(),
            name);

        cv = new ImgCanvas (img);
        done = new Button ("Done");

        add ("North", cv);
        add ("South", done);
    }

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.ACTION_EVENT:
            if (ev.target == done) {
                hide ();
                return true;
            }
            break;
        }
        return false;
    }
}
