/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;

import GlobalVars;
import DictBrowser;

public class TypeBrowser extends Dialog {
private Button ok, show, join;
private DictBrowser dict;
private CGViewer cgview;
private String type;
private ConceptualGraph cg;

public TypeBrowser (String type, ConceptualGraph cg)
    {
        super (GlobalVars.frame, true);
        if (type == null)
            type = "something";

        setLayout (new GridBagLayout());
	setTitle ("Type Browser");

        ok = new Button ("Done");
        show = new Button ("Show OID");
        if (cg != null)
            join = new Button ("Join to Variant");
        else
            join = null;

        Panel p = new Panel();
        p.add ("Center", ok);
        p.add ("Center", show);
        if (cg != null)
            p.add ("Center", join);

        cgview = new CGViewer (new Dimension (300, 300));

        Vector rels = new Vector ();
        rels.addElement ("IS_A");
        rels.addElement ("SYNONYM");
        dict = new DictBrowser (type, DictBrowser.ENDWORD, rels,
                                DictBrowser.PRED);

        GlobalVars.constrain (this, cgview, 0, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, dict, 0, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, p, 0, 2, 1, 1,
                              GridBagConstraints.BOTH, 0.0, 0.0);

        this.type = type;
        this.cg = cg;
    }

private void ShowVariants (String s)
    {
        try {
            Vector v = new Vector();
            int n = GlobalVars.proxy.getNumberOfVariants (s);
            for (int i = 0; i < n; ++i) {
                v.addElement (GlobalVars.proxy.getNthVariant (s, i));
            }
            cgview.SetKeys (v);
        } catch (AITException e) {
            GlobalVars.DisplayError (e.getErrMsg());
        }
    }
    
public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.ACTION_EVENT:
            if (ev.target == ok) {
                hide ();
                return true;
            } else if (ev.target == join && cg != null) {
                String key = cgview.GetSelectionKey ();
                if (key != null) {
                    try {
                        GlobalVars.proxy.join (key, cg);
                        cgview.Update ();
                    } catch (AITException e) {
                        GlobalVars.DisplayError (e.getErrMsg());
                    }
                }
                return true;
            } else if (ev.target == dict) {
                ShowVariants ((String)ev.arg);
                return true;
            } else if (ev.target == show) {
                String oid = cgview.GetSelectionOID ();
                if (oid != null)
                    GlobalVars.ShowDocument (oid);
            }
            break;
        }
        return false;
    }

public void show ()
    {
	super.show ();
    }

public void pack ()
    {
	super.pack ();
	ShowVariants (type);
    }
}

