/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.net.*;

import GlobalVars;
import AlertBox;
import ListBox;
import DictBrowser;

public class TypeHier extends Dialog {
private ListBox superlist, sublist;
private Button addsuper, delsuper, addsub, delsub;
private Button insert, cancel;
private DictBrowser browse;
private String theword;

public TypeHier (String word)
    {
        super (GlobalVars.frame, true);
        setLayout (new GridBagLayout());

        Panel p = new Panel();
        insert = new Button ("Insert as new type");
        insert.disable ();
        cancel = new Button ("Cancel");
        p.add ("Center", insert);
        p.add ("Center", cancel);

        BorderPanel bp = new BorderPanel (BorderPanel.RIDGE);
        bp.setLayout (new GridBagLayout());
        superlist = new ListBox ();
        sublist = new ListBox ();
        addsuper = new Button ("Add supertype");
        delsuper = new Button ("Remove supertype");
        addsub = new Button ("Add subtype");
        delsub = new Button ("Remove subtype");
        delsuper.disable ();
        delsub.disable ();

        GlobalVars.constrain (bp, new Label ("Supertypes of '" + word + "'"),
                              0, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 0);
        GlobalVars.constrain (bp, superlist,
                              0, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (bp, addsuper,
                              0, 2, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 0);
        GlobalVars.constrain (bp, delsuper,
                              0, 3, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        GlobalVars.constrain (bp, new Label ("Subtypes of '" + word + "'"),
                              1, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 0);
        GlobalVars.constrain (bp, sublist,
                              1, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (bp, addsub,
                              1, 2, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 0);
        GlobalVars.constrain (bp, delsub,
                              1, 3, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        Vector rels = new Vector();
        rels.addElement ("IS_A");
        rels.addElement ("SYNONYM");
        browse = new DictBrowser ("something", DictBrowser.STARTWORD,
                                  rels, DictBrowser.PRED);

        GlobalVars.constrain (this, bp,
                              0, 0, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, browse,
                              0, 1, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);
        GlobalVars.constrain (this, p,
                              0, 2, 1, 1,
                              GridBagConstraints.BOTH, 1.0, 1.0);

        theword = word;
    }

private void updateButtons ()
    {
        int i = superlist.countItems ();
        boolean s = insert.isEnabled ();
        if (i == 0 && s)
            insert.disable ();
        else if (i != 0 && !s)
            insert.enable ();

        i = superlist.getSelectedIndex ();
        s = delsuper.isEnabled ();
        if (i < 0 && s)
            delsuper.disable ();
        else if (i >= 0 && !s)
            delsuper.enable ();

        i = sublist.getSelectedIndex ();
        s = delsub.isEnabled ();
        if (i < 0 && s)
            delsub.disable ();
        else if (i >= 0 && !s)
            delsub.enable ();
}

public boolean handleEvent (Event ev)
    {
        switch (ev.id) {
        case Event.LIST_SELECT:
        case Event.LIST_DESELECT:
            updateButtons ();
            return true;

        case Event.ACTION_EVENT:
            if (ev.target == addsuper) {
                superlist.addItem (browse.GetSelection ());
                updateButtons ();
            } else if (ev.target == addsub) {
                sublist.addItem (browse.GetSelection ());
                updateButtons ();
            } else if (ev.target == delsuper) {
                int i = superlist.getSelectedIndex ();
                if (i >= 0)
                    superlist.delItem (i);
                updateButtons ();
            } else if (ev.target == delsub) {
                int i = sublist.getSelectedIndex ();
                if (i >= 0)
                    sublist.delItem (i);
                updateButtons ();
            } else if (ev.target == insert) {
                try {
                    if (!GlobalVars.proxy.containsWord (theword))
                        GlobalVars.proxy.addWord (theword);
                    for (int i = superlist.countItems(); --i >= 0; ) {
                        GlobalVars.proxy.addRelation (
                            theword, "IS_A", superlist.getItem (i));
                    }
                    for (int i = sublist.countItems(); --i >= 0; ) {
                        GlobalVars.proxy.addRelation (
                            sublist.getItem (i), "IS_A", theword);
                    }
                } catch (AITException e) {
                    GlobalVars.DisplayError (e.getErrMsg());
                }
                hide ();
            } else if (ev.target == cancel) {
                hide ();
            } else
                break;
            return true;
        }
        return false;
    }
}
