/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

import java_cup.runtime.*;
import ServiceEntry;


/** This subclass of token represents symbols that need
 *  to maintain one ServiceEntry value as an attribute.  
 *  It maintains that value in the public field serv_val.
 *
 * @see java_cup.runtime.serv_token
 * @version last updated: 6/4/96
 * @mail aitrader@informatik.uni-frankfurt.de
 * @author  Reza Farsi
 */

public class serv_token extends token {

  /** Full constructor. */
  public serv_token(int term_num, ServiceEntry se)
    {
      /* super class does most of the work */
      super(term_num);

      serv_val = se;
    }

  /** Constructor with default value of 0 */
  public serv_token(int term_num)
    {
      this(term_num, null);
    }

  /** The stored conceptual node value. */
  public ServiceEntry serv_val;
};
