
/*
*  AI-Trader --- Service Trading using Conceptual Graphs
*  Copyright (C) 1995 Arno Puder et al.
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*  Send comments and/or bug reports to:
*                 aitrader@informatik.uni-frankfurt.de
*/


/* CHB
    Name:	  client.cc
    Created:  15.10.1995, Wolfgang Schmitz
    Modified: 07.11.1995, Arno Puder
		      18.11.1995, Wolfgang Schmitz
			    #include <rpc/rpc.h> moved out of
				    #ifdef _AIX
			    #include "rpc_if.h" moved out of
				    #ifdef _AIX
		      20.11.1995, Wolfgang Schmitz
    ------------------------------------------------------------
    Implementation of the client class ...
CHE */



#include <stdlib.h>
#include <sys/types.h>
#include <rpc/rpc.h>


#ifdef _AIX
#include <rpc/clnt.h>
#endif

#if _AIX || sun

#undef connect_1
extern "C" int *connect_1(char **, CLIENT *);

#undef disconnect_1
extern "C" int *disconnect_1(char **, CLIENT *);

#undef exec_request_1
extern "C" char **exec_request_1(char **, CLIENT *);

#undef init_shutdown_1
extern "C" int *init_shutdown_1(char **, CLIENT *);

#undef force_shutdown_1
extern "C" int *force_shutdown_1(char **, CLIENT *);


#endif

#include "rpc_if.h"
#include "rpc_client.h"



/* CHB
    Name:		RPCClinet (Contructor)
    Parameter:	HostName: the RPC server's hostname
			    ProgNr: the program number of the RPC server
			    Timeout: timeout for server calls in seconds
    Returncode:	none
    Created:	24.10.1995, Wolfgang Schmitz
    Modified:   07.11.1995, Arno Puder
    ------------------------------------------------------------
    Creates a client handle for the RPCClient instance.
CHE */

RPCClient::RPCClient( char *HostName, u_long ProgNr, int Timeout )
{
    struct timeval	tv;
    
    ClientHandle = clnt_create( HostName, ProgNr, TRADER_VERS, "tcp" );
    rc = ( ClientHandle ) ? 0 : -1;
    if( rc == 0 ) {
	    tv.tv_sec  = Timeout;
	    tv.tv_usec = 0;
	    clnt_control( ClientHandle, CLSET_TIMEOUT, (char *)&tv );
    }
}



/* CHB
    Name:		~RPCIClient (Destructor)
    Parameter:	none
    Returncode:	none
    Created:	24.10.1995, Wolfgang Schmitz
    Modified:   07.11.1995, Arno Puder
    ------------------------------------------------------------
    Performs all actions to shut down the client. (Destroys the
    client handle );
CHE */

RPCClient::~RPCClient( void )
{
    if( ClientHandle ) {
	    clnt_destroy( ClientHandle );
    }
}



/* CHB
    Name:		Connect
    Parameter:	rc: an error code provided by the server
    Returncode:	an error code indicating success (0) or failure
			    (-1) of the RPC
    Created:	18.11.1995, Wolfgang Schmitz
    Modified:   20.11.1995, Wolfgang Schmitz
				    New parameter User
    ------------------------------------------------------------
    Executes a server request by executing the remote procedure
    Connect
CHE */

int RPCClient::Connect( char *User, int *retcode )
{
    int		*i;

    i = connect_1( &User, ClientHandle );
    if( !i ) {
	    return( -1 );
    }
    
    *retcode = *i;
    return( 0 );
}



/* CHB
    Name:		Disconnect
	Parameter:	rc: an error code provided by the server
	Returncode:	an error code indicating success (0) or failure
				(-1) of the RPC
	Created:	18.11.1995, Wolfgang Schmitz
	Modified:   20.11.1995, Wolfgang Schmitz
					New parameter User
	------------------------------------------------------------
	Executes a server request by executing the remote procedure
	Disconnect
CHE */

int RPCClient::Disconnect( char *User, int *retcode )
{
	int		*i;

	i = disconnect_1( &User, ClientHandle );
	if( !i ) {
		return( -1 );
	}
	
	*retcode = *i;
	return( 0 );
}



/* CHB
	Name:		ExecRequest
	Parameter:	InString: the input string to be processed
				OutString: the result string provided by the
					RPC server
	Returncode:	an error code indicating success (0) or failure
				(-1)
	Created:	24.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
	------------------------------------------------------------
	Executes a server request by executing the remote procedure
	ExecRequest.
CHE */

int RPCClient::ExecRequest( char *InString, char **OutString )
{
	char	**Str;

	Str = exec_request_1( &InString, ClientHandle );
	if( !Str ) {
		return( -1 );
	}
	
	*OutString = *Str;
	return( 0 );
}



/* CHB
	Name:		InitShutdown
	Parameter:	PassWord: the password needed to shutdown the
					server
				rc: an error code provided by the server
	Returncode:	an error code indicating success (0) or failure
				(-1) of the RPC
	Created:	18.11.1995, Wolfgang Schmitz
	Modified:   20.11.1995, Wolfgang Schmitz
					Renamed from Shutdown to InitShutdown
	------------------------------------------------------------
	Executes a server request by executing the remote procedure
	InitShutdown
CHE */

int RPCClient::InitShutdown( char *Password, int *retcode )
{
	int		*i;

	i = init_shutdown_1( &Password, ClientHandle );
	if( !i ) {
		return( -1 );
	}
	
	*retcode = *i;
	return( 0 );
}



/* CHB
	Name:		ForceShutdown
	Parameter:	PassWord: the password needed to shutdown the
					server
				rc: an error code provided by the server
	Returncode:	an error code indicating success (0) or failure
				(-1) of the RPC
	Created:	20.11.1995, Wolfgang Schmitz
	Modified:
	------------------------------------------------------------
	Executes a server request by executing the remote procedure
	ForceShutdown
CHE */

int RPCClient::ForceShutdown( char *Password, int *retcode )
{
	int		*i;

	i = force_shutdown_1( &Password, ClientHandle );
	if( !i ) {
		return( -1 );
	}
	
	*retcode = *i;
	return( 0 );
}



/* CHB
	Name:		GetErrorCode
	Parameter:	none
	Returncode:	the error code stored inside of the client class
	Created:	24.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
	------------------------------------------------------------
	Returns the error code, stored inside of the client class.
CHE */

int RPCClient::GetErrorCode( void )
{
	return( rc );
}

