
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


/* CHB
	Name:	  server.cc
	Created:  04.10.1995, Wolfgang Schmitz
	Modified: 07.11.1995, Arno Puder
			  18.11.1995, Wolfgang Schmitz
			  20.11.1995, Wolfgang Schmitz
	------------------------------------------------------------
	Implementation of the RPC server functions and RPC server
	class
CHE */



#include <assert.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <rpc/pmap_prot.h>

#include "rpc_if.h"
#include "rpc_server.h"


/* Global data */
u_long   trader_prog_nr;				/* The AI-Trader's program number */


/* Static data */
static RPCServer	*server = NULL;		/* A pointer to the server instance */



/* CHB
	Name:	    connect_1
	Parameter:  user: the user executinging the connect
	Returncode: an error code indicating success or failure of
					the connect
	Created:    18.11.1995, Wolfgang Schmitz
	Modified:   20.11.1995, Wolfgang Schmitz
					user parameter added
	------------------------------------------------------------
	This function is called by the generated RPC server. It calls
	the function Connect of the RPCServer class.
CHE */

#ifdef __linux__
extern "C" int *connect_1_svc( char **user, struct svc_req* )
#else
extern "C" int *connect_1( char **user, CLIENT* )
#endif
{
	static int	rc;				/* Return code */
	
	/* Call virtual function of the server class */
	assert( server != NULL );
	rc = server->Connect( *user );
	
	return( &rc );
}



/* CHB
	Name:	    disconnect_1
	Parameter:  user: the user executing the disconnect
	Returncode: an error code indicating success or failure of
					the disconnect
	Created:    18.11.1995, Wolfgang Schmitz
	Modified:   
	------------------------------------------------------------
	This function is called by the generated RPC server. It calls
	the function Disconnect of the RPCServer class.
CHE */

#ifdef __linux__
extern "C" int *disconnect_1_svc( char **user, struct svc_req* )
#else
extern "C" int *disconnect_1( char **user, CLIENT* )
#endif
{
	static int	rc;				/* Return code */
	
	/* Call virtual function of the server class */
	assert( server != NULL );
	rc = server->Disconnect( *user );
	
	return( &rc );
}



/* CHB
	Name:	    exec_request_1
	Parameter:  s: the request string to be executed
	Returncode: a result string, produced by the ai_trader
	Created:    04.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
				18.11.1995, Wolfgang Schmitz
					Prototyp changed for linux
	------------------------------------------------------------
	This function is called by the generated RPC server. It calls
	the function ExecRequest of the AIServer class.
CHE */

#ifdef __linux__
extern "C" char **exec_request_1_svc( char **s, struct svc_req* )
#else
extern "C" char **exec_request_1( char **s, CLIENT* )
#endif
{
	static char	*ret_string = NULL;	/* Vector to store the result string */
	
	/* Call virtual function of the server class */
	assert( server != NULL );
	server->ExecRequest( *s, &ret_string );
	
	return( &ret_string );
}



/* CHB
	Name:	    init_shutdown_1
	Parameter:  none
	Returncode: an error code indicating success or failure of
					the shutdown
	Created:    18.11.1995, Wolfgang Schmitz
	Modified:   20.11.1995, Wolfgang Schmitz
					Renamed form shutdown_1 to init_shutdown_1
                30.11.1995 Arno Puder
                    Removed minor bug in prototype for AIX
	------------------------------------------------------------
	This function is called by the generated RPC server. It calls
	the function InitShutdown of the RPCServer class.
CHE */

#ifdef __linux__
extern "C" int *init_shutdown_1_svc( char **passwd, struct svc_req* )
#else
extern "C" int *init_shutdown_1( char **passwd, CLIENT* )
#endif
{
	static int	rc;				/* Return code */
	
	/* Call virtual function of the server class */
	assert( server != NULL );
	rc = server->InitShutdown( *passwd );
	
	return( &rc );
}



/* CHB
	Name:	    force_shutdown_1
	Parameter:  none
	Returncode: an error code indicating success or failure of
					the shutdown
	Created:    20.11.1995, Wolfgang Schmitz
	Modified:   30.11.1995 Arno Puder
                    Removed minor bug in prototype for AIX
	------------------------------------------------------------
	This function is called by the generated RPC server. It calls
	the function ForceShutdown of the RPCServer class.
CHE */

#ifdef __linux__
extern "C" int *force_shutdown_1_svc( char **passwd, struct svc_req* )
#else
extern "C" int *force_shutdown_1( char **passwd, CLIENT* )
#endif
{
	static int	rc;				/* Return code */
	
	/* Call virtual function of the server class */
	assert( server != NULL );
	rc = server->ForceShutdown( *passwd );
	
	return( &rc );
}



/* CHB
	Name:		RPCServer
	Parameter:	ProgNr: The server program number
	Returncode:	none
	Created:	11.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
				18.11.1995, Wolfgang Schmitz
					Initialization of ResultString removed
	------------------------------------------------------------
	The Constructor of the RPCServer class. It sets the global
	program number, a global pointer to the server. If an instance
	of the server class is already existing an error code is set,
	that can be retrieved using GetErrorCode
CHE */

RPCServer::RPCServer( u_long ProgNr )
{
    struct sockaddr_in sin;

    sin.sin_family = AF_INET;
    sin.sin_port = htons (PMAPPORT);
    sin.sin_addr.s_addr = htonl (INADDR_LOOPBACK);

#if 0
    if (pmap_getport (&sin, ProgNr, TRADER_VERS, IPPROTO_UDP) || server) {
/* Not the first server instance */
        rc = -1;
    }
#else
    pmap_getport (&sin, ProgNr, TRADER_VERS, IPPROTO_UDP);
    if (server) {
/* Not the first server instance */
        rc = -1;
    }
#endif
    else {
/* First instace */
        rc = 0;
        trader_prog_nr = ProgNr;
        server = this;
    }
}



/* CHB
	Name:		~RPCServer
	Parameter:	none
	Returncode:	none
	Created:	15.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
				18.11.1995, Wolfgang Schmitz
					delete ResultString removed
	------------------------------------------------------------
	The server's destructor. It frees memory allocated to store
	result string and sets the variables server and trader_prog_nr
	to 0.
CHE */

RPCServer::~RPCServer( void )
{
	/* There is another server running */
	if( rc == -1 ) {
		return;
	}
	
	server = NULL;
	trader_prog_nr = 0;
}



/* CHB
	Name:		RunServer
	Parameter:	none
	Returncode:	none
	Created:	12.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
	------------------------------------------------------------
	Runs the server loop generated by rpcgen.
CHE */

void RPCServer::RunServer( void )
{
	server_main( 0, (char**) NULL );
}



/* CHB
	Name:		GetErrorCode
	Parameter:	none
	Returncode:	The error code, stored inside of the server
	Created:	11.10.1995, Wolfgang Schmitz
	Modified:   07.11.1995, Arno Puder
	------------------------------------------------------------
	Returns the server's error code.
CHE */

int RPCServer::GetErrorCode( void )
{
	return( rc );
}
