 
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: aitperf.cc,v 1.6 1998/02/06 11:05:39 roemer Exp $


#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>

#include <stream.h>
#include <fstream.h>
#include <strstream.h>
#include <Trader.h>

#include "stl.h"



static string mrule = "Specialization";
static int loops = 1000;
static bool verbose = false;
static Server server;

void
usage (char *prog)
{
    cout<<form ("usage: %s [options] <TypeGraphFile> <QueryGraphFile>\n", prog);
    cout<<form ("options: [-v] [-m <MatchingRule>] [-l <Loops>] "
               "[-i <RCFile>]\n");
    exit (1);
}

bool
loadFile (string name)
{
    ifstream file (name.c_str());

    if (!file) {
        printf ("Couldn't open file '%s'\n", name.c_str());
        return false;
    }
    Parser parser (&server, file, name.c_str());
    parser.Parse();
    if (parser.ErrorOccured ()) {
        cerr << parser.GetErrorFile() << ':'
             << parser.GetErrorLine() << ':'
             << parser.GetErrorMsg() << endl;
        return false;
    }
    return true;
}

ConceptualGraph *
loadGraph (string name)
{
    string s = "@answer-graph=";

    ifstream ifile (name.c_str());
    if (!ifile) {
        printf ("Couldn't open file '%s'\n", name.c_str());
        return NULL;
    }
    char c;
    ifile >> c;
    while (!ifile.eof()) {
        s += c;
        ifile >> c;
    }
    istrstream istr (s.c_str());

    Server tmpserv;
    Parser parser (&tmpserv, istr, name.c_str());
    parser.Parse();
    if (parser.ErrorOccured ()) {
        cerr << parser.GetErrorFile() << ':'
             << parser.GetErrorLine() << ':'
             << parser.GetErrorMsg() << endl;
        return NULL;
    }
    ConceptualGraph *cg;
    tmpserv.GetAnswer (&cg);
    assert (cg);
    return cg;
}

extern "C" {
    extern char *optarg;
    extern int optind;
}

int
main (int argc, char *argv[])
{
    char *initial = NULL;
    char *progname = argv[0];

    int c;
    while ((c = getopt(argc, argv, "vi:l:m:")) != EOF) {
        switch (c) {
        case 'l':
            loops = atoi (optarg);
            break;

        case 'm':
            mrule = optarg;
            break;

        case 'v':
            verbose = true;
            break;

        case 'i':
            initial = optarg;
            break;

        case '?':
            usage (progname);
        }
    }
    argv += optind;
    argc -= optind;

    if (argc != 2)
        usage (progname);

    if (initial && !loadFile (initial))
        exit (1);

    if (verbose)
        server.SetVerbose (true);

    ConceptualGraph *cg1 = loadGraph (argv[0]);
    ConceptualGraph *cg2 = loadGraph (argv[1]);
    if (!cg1 || !cg2)
        exit (1);

    KeyCG newkey;
    server.Add (cg1, newkey);

    bool match = false;

    SetOfString mruleset;
    mruleset.insert (mrule);

    struct timeval tm1, tm2;
    clock_t cl1, cl2;

    gettimeofday (&tm1, NULL);
    cl1 = clock();
    for (int i = 0; i < loops; ++i) {
        SetOfMatchResult res;
        server.Match (cg2, mruleset, res);
        if (i == 0 && !res.empty())
            match = true;
    }
    cl2 = clock();
    gettimeofday (&tm2, NULL);

    double realtime = (double)tm2.tv_sec + (double)tm2.tv_usec/1000000.0 -
        (double)tm1.tv_sec - (double)tm1.tv_usec/1000000.0;
    double cputime = (double)cl2/CLOCKS_PER_SEC - (double)cl1/CLOCKS_PER_SEC;

    cout<<form ("         iterations: %d\n", loops);
    cout<<form ("      matching rule: %s\n", mrule.c_str());
    cout<<form ("    total real time: %f msec\n", realtime*1000);
    cout<<form ("     total cpu time: %f msec\n", cputime*1000);
    cout<<form ("real time per match: %f msec\n", realtime/loops*1000);
    cout<<form (" cpu time per match: %f msec\n", cputime/loops*1000);
    cout<<form ("       match result: %s\n", match ? "match" : "don't match");

    return 0;
}
