
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


/* $Id: proxy.cc,v 1.9 1997/04/09 14:37:05 roemer Exp $ */


#include <fstream.h>
#include <strstream.h>
#include <assert.h>

#include "word.h"
#include "util.h"
#include "proxy.h"



LocalTclClient::LocalTclClient()
	: Server()
{
}


ErrorString LocalTclClient::Connect( string fileName )
{
	ifstream file( fileName.c_str() );
	
	if( !file ) {
            ostrstream ostr;
            ostr << "Couldn't open file '" << fileName << "'" << ends;
            string str (ostr.str ());
            ostr.rdbuf()->freeze (0);
            return ErrorString (str, 9);
	}
	Parser *parser = new Parser( this, file, fileName.c_str() );
	parser->Parse();
	if (parser->ErrorOccured ()) {
            ostrstream ostr;
            ostr << parser->GetErrorFile() << ':'
                 << parser->GetErrorLine() << ':'
                 << parser->GetErrorMsg() << ends;
            string str (ostr.str ());
            ostr.rdbuf()->freeze (0);
            delete parser;
            return ErrorString (str, 9);
	}
	delete parser;
	return ErrorString();
}


ErrorString LocalTclClient::Disconnect()
{
	return ErrorString();
}


void LocalTclClient::ConvertToTcl( ostream &ostr, ConceptualGraph *cg )
{
	if( cg == NULL )
		return;
	
	ostrstream otmp;
	
        otmp << cg->GetNode()->GetName();
	if( cg->GetNode()->IsA() == CONCEPT_NODE ) {
		SetOfWord &inst = ((ConceptNode *) cg->GetNode())->GetInstances();
		if( !inst.empty() ) {
			otmp << " : { ";
                        SetOfWord::const_iterator i;
			for (i = inst.begin(); i != inst.end(); ) {
#if 0
				otmp << '"' << *i << '"';
#else
				otmp << *i;
#endif
				++i;
				if( i != inst.end() )
					otmp << ", ";
			}
			otmp << " }";
		} else {
			long val = ((ConceptNode *) cg->GetNode())->GetValue();
			if( val != NIL_VALUE ) {
				otmp << " : " << val;
			}
		}
	}
	otmp << ends;
	string tmp( otmp.str() );
	otmp.rdbuf()->freeze( 0 );
	gsub (tmp, "\"", "\\\"");
	if( tmp.find (' ') >= tmp.length() )
            ostr << tmp;
	else
            ostr << '"' << tmp << '"';
	
    if( cg->GetWidth() ) {
		ostr << " {";
		ConvertToTcl( ostr, cg->GetWidth() );
		ostr << "}";
    }
    if( cg->GetDepth() ) {
		ostr << "} {";
		ConvertToTcl( ostr, cg->GetDepth() );
    }
}

