
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: cg.h,v 1.10 1998/02/06 11:05:49 roemer Exp $


#ifndef __CG_H__
#define __CG_H__

#include <stdlib.h>
#include "word.h"
#include "stl.h"
#include "globaldef.h"


#if sun || linux
#define NIL_VALUE -543210
#else
#include <stdlib.h>
#define NIL_VALUE LONG_MAX
#endif


#define CONCEPT_NODE 1
#define RELATION_NODE 2

#define NODE_UNKNOWN    0	// no match performed on this node
#define NODE_ACCEPTED   1	// match performed and accepted
#define NODE_REJECTED   2	// match performed and rejected
#define NODE_CANTTELL	3	// match performed and can't tell


class Node 
{
protected:
    Word name;
public:
    Node();
    virtual ~Node();
    Word &GetName();
    virtual int IsA() = 0;
    virtual int operator==( Node &node ) = 0;
    virtual int operator<=( Node &Node ) = 0;
    virtual void Print( ostream &ostr ) = 0;
};


class RelationNode : public Node
{
public:
    RelationNode( const Word &aName );
    RelationNode( RelationNode *node );
    ~RelationNode();
    int IsA();
    int operator==( Node &node );
    int operator<=( Node &node );
    void Print( ostream &ostr );
};


typedef set<Word, less<Word> > SetOfWord;

extern SetOfWord emptyWordSet;


class ConceptNode : public Node
{
private:
    SetOfWord  inst;
	long       value;
public:
    ConceptNode( const Word &aName, SetOfWord &theInst = emptyWordSet );
    ConceptNode( ConceptNode *node );
    ~ConceptNode();
    int IsA();
    int operator==( Node &node );
    int operator<=( Node &node );
    SetOfWord &GetInstances();
    void AddInstances( SetOfWord &new_inst );
    void SetInstances( SetOfWord &new_inst );
    void SetValue( long aValue );
    long GetValue();
    void Print( ostream &ostr );
};



_CLASSDEF( ConceptualGraph )
class ConceptualGraph
{
private:
    Node            *node;
    ConceptualGraph *width;
    ConceptualGraph *depth;
    int              tag;
public:
    ConceptualGraph( Node *aNode, ConceptualGraph *theWidth = NULL,
		     ConceptualGraph *theDepth = NULL );
    ~ConceptualGraph();
    void SetNode( Node *aNode );
    Node *GetNode();
    void SetWidth( ConceptualGraph *theWidth );
    void SetDepth( ConceptualGraph *theDepth );
    ConceptualGraph *GetWidth();
    ConceptualGraph *GetDepth();
    void SetTag( int aTag );
    int GetTag();
    void ResetTags( int aTag );
    void Print( ostream &ostr, bool withTags = false );
    void PrettyPrint( ostream &ostr, int incr );
    bool operator== (ConceptualGraph &);
    void Subtract (ConceptualGraph &);
	
private:
    void DoPrettyPrint( ostream &ostr, int indent, int incr );
    int PrintSpaces( ostream &ostr, int indent, int incr );

    long magic;
#define CGMAGIC 0x31415927L

    void validate ()   { magic = CGMAGIC; }
    void invalidate () { magic = 0L; }
    void sanity ();
};


Node *CopyNode( Node *node );
ConceptualGraph *CopyGraph( ConceptualGraph *node );


#endif
