
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: kb.h,v 1.7 1997/06/27 16:19:14 roemer Exp $


#ifndef __KB_H__
#define __KB_H__

#include "key.h"
#include "dict.h"
#include "match.h"
#include "cg.h"
#include "profiles.h"

class MatchResult 
{
private:
    KeyCG key;
    ConceptualGraph *query;
    int matchval;

public:
    MatchResult( KeyCG theKey, ConceptualGraph *theQuery, int matchval );
    MatchResult()
    {
        query = NULL;
        matchval = 0;
    }
    MatchResult( const MatchResult &m )
    {
        key = m.key;
        query = CopyGraph( m.query );
        matchval = m.matchval;
    }
    ~MatchResult();
    MatchResult &operator=( const MatchResult &m )
    {
        key = m.key;
        if( query )
            delete query;
        query = CopyGraph( m.query );
        matchval = m.matchval;
        return *this;
    }
    bool operator==( const MatchResult &m ) const
    {
        return key == m.key;
    }
    bool operator<( const MatchResult &m ) const
    {
        return key < m.key;
    }
    KeyCG GetKeyCG() const;
    ConceptualGraph *GetQueryGraph() const;
    int GetMatchVal () const;
    void Print( ostream &ostr );
};

typedef set<MatchResult, less<MatchResult> > SetOfMatchResult;



class KB 
{
private:
    unsigned long uuid;
    KeyCG GetUniqueKeyCG();
	
    typedef map<KeyCG, ConceptualGraph *, less<KeyCG> > DB;
    DB db;
public:
    KB();
    ~KB();
    void Add( ConceptualGraph *cg, KeyCG &newKey );
    void Replace( KeyCG aKey, ConceptualGraph *cg );
    void Join( KeyCG aKey, ConceptualGraph *cg );
    SetOfMatchResult Match( Dict &dict, MatchingRule &rules, 
                            SetOfString &selecRules,
                            ConceptualGraph *cg, bool verbose,
                            const ProfileEntry &prof);
    ConceptualGraph *GetGraphByKey( KeyCG key );
    int GetNumberOfVariants( const string &conType );
    KeyCG GetVariant( const string &conType, int num );
    int GetNumberOfGraphs();
    KeyCG GetNthGraph( int num );
    void Print( ostream &ostr );
};



#endif
