
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: parser.cc,v 1.4 1997/07/04 14:20:34 roemer Exp $


#include "parser.h"


extern int line_no;

Parser *theParser;


int yylex()
{
    return theParser->GetLexer()->yylex();
}


Parser::Parser( AITraderInterface *aServer, istream &istr,
                const char *filename )
{
    server = aServer;
    lexfile = filename ? filename : "";
    lexer = new yyFlexLexer( &istr );
}


Parser::~Parser()
{
    delete lexer;
}


void Parser::Parse()
{
    Parser *oldparser = theParser;
    theParser = this;

    line_no = 0;
    errorMsg = NULL;
    LastErrorString.ResetError();
    yyparse();
    server->CheckForError();
#if 0
    // this breaks cgtcl.cc:CG_FreeCommand()...
    theParser = oldparser;
#endif
}


void Parser::SetError( char *file, char *err, int line )
{
    errorFile = file;
    errorMsg  = err;
    errorLine = line;
    if( file == NULL )
        LastErrorString = ErrorString( 8, err, line );
    else
        LastErrorString = ErrorString( 7, file, err, line );
}


bool Parser::ErrorOccured()
{
    return errorMsg != NULL;
}


char *Parser::GetErrorFile()
{
    return errorFile;
}


char *Parser::GetErrorMsg()
{
    return errorMsg;
}


int Parser::GetErrorLine()
{
    return errorLine;
}


char *
Parser::GetLexfile () const
{
    return (char *)lexfile.c_str();
}

void
Parser::SetLexfile (const char *f)
{
    lexfile = f;
}

AITraderInterface *
Parser::GetServer ()
{
    return server;
}


yyFlexLexer *
Parser::GetLexer ()
{
    return lexer;
}
