
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: parser.h,v 1.4 1997/07/04 14:20:35 roemer Exp $


#ifndef __PARSER_H__
#define __PARSER_H__

#include <fstream.h>
#include <FlexLexer.h>
#include "server.h"

class Parser;

extern Parser *theParser;  // Defined in parser.cc
int yyparse();             // Defined through yacc.y

class Parser 
{
private:
    string lexfile;
    AITraderInterface *server;
    yyFlexLexer *lexer;

    char   *errorFile;
    char   *errorMsg;
    int     errorLine;
public:
    Parser( AITraderInterface *aServer, istream &istr,
            const char *filename = NULL );
    ~Parser();
    void Parse();
    void SetError( char *file, char *err, int line );
    bool ErrorOccured();
    char *GetErrorFile();
    char *GetErrorMsg();
    int   GetErrorLine();

    char *GetLexfile () const;
    void SetLexfile (const char *);
    AITraderInterface *GetServer ();
    yyFlexLexer *GetLexer ();
};


#endif
