
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: profiles.cc,v 1.5 1998/03/06 16:15:11 farsi Exp $

#include <assert.h>
#include "profiles.h"

/*************************** ProfileEntry *******************************/

void
ProfileEntry::Print (ostream &ostr, KeyCG key) const
{
    ostr << "@set-timeout $" << key << " : " << _timeout << endl;
    ostr << "@set-minmatchval $" << key << " : " << _minmatchval << endl;
}

/***************************** Profiles *********************************/

KeyCG Profiles::thekey = 0;

KeyCG
Profiles::newkey ()
{
    do {
        thekey++;
    } while (profiles.count (thekey) != 0);
    return thekey;
}

KeyCG
Profiles::RegisterProfile (const ProfileEntry &e)
{
    KeyCG key = newkey();
    profiles[key] = e;
    return key;
}

bool
Profiles::RegisterProfile (KeyCG key, const ProfileEntry &e)
{
    if (profiles.count (key) != 0)
        return false;
    profiles[key] = e;
    return true;
}

void
Profiles::DeleteProfile (KeyCG key)
{
    profiles.erase (key);
}

bool
Profiles::HaveProfile (KeyCG key) const
{
    return (profiles.count (key) > 0);
}

ProfileEntry &
Profiles::GetProfile (KeyCG key)
{
    MapOfProfileEntry::iterator i = profiles.find (key);
    assert (i != profiles.end());
    return (*i).second;
}

const ProfileEntry &
Profiles::GetProfile (KeyCG key) const
{
    MapOfProfileEntry::const_iterator i = profiles.find (key);
    assert (i != profiles.end());
    return (*i).second;
}

void
Profiles::Print (ostream &ostr) const
{
    MapOfProfileEntry::const_iterator i;
    for (i = profiles.begin(); i != profiles.end(); ++i)
        (*i).second.Print (ostr, (*i).first);
}
