
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: profiles.h,v 1.5 1998/02/06 11:05:50 roemer Exp $
 
 
#ifndef __PROFILES_H__
#define __PROFILES_H__

#include "key.h"
#include <fstream.h>
#include "stl.h"

class ProfileEntry {
    long _timeout;
    long _minmatchval;
public:
    ProfileEntry ()
        : _timeout(0), _minmatchval(1)
    {}
    void Print( ostream &ostr, KeyCG ) const;

    void SetTimeout (long t) { _timeout = t; }
    long GetTimeout () const { return _timeout; }

    void SetMinMatchval (long m) { _minmatchval = m; }
    long GetMinMatchval () const { return _minmatchval; }
};


typedef map<KeyCG, ProfileEntry, less<KeyCG> > MapOfProfileEntry;


class Profiles {
    MapOfProfileEntry profiles;
    static KeyCG thekey;
    KeyCG newkey ();
public:
    bool RegisterProfile (KeyCG, const ProfileEntry &);
    KeyCG RegisterProfile (const ProfileEntry &);
    void DeleteProfile (KeyCG);
    bool HaveProfile (KeyCG) const;
    ProfileEntry &GetProfile (KeyCG);
    const ProfileEntry &GetProfile (KeyCG) const;
    void Print (ostream &ostr) const;
};

#endif
