
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


#ifndef __PROXY_H__
#define __PROXY_H__

#include "interface.h"
#include "server.h"


class TransportLayer 
{
public:
	virtual void TransmitString( string &in, string &out ) = 0;
};



class ClientProxy : public AITraderInterface 
{
private:
	TransportLayer *transportLayer;
	Server          localServer;
	
	void ExecuteCommand( string command );
public:
    ClientProxy( TransportLayer *theConnection );
    ErrorString GetVersion( string &version );
    ErrorString SetVerbose( bool flag );
    ErrorString DictQuickMode( bool flag );
    ErrorString SaveTraderDB( string *name );
    ErrorString DefineMatchingRule( string *name, string *rule );
    ErrorString DeleteMatchingRule( string *name );
    ErrorString GetMatchingRuleNames( string *names );
    ErrorString GetMatchingRule( string *name, string *rule );
    ErrorString RegisterRelation( string &rel, int prop );
    ErrorString GetAllRelations( SetOfString &rels );
    ErrorString GetRelationProperties (string &s, int &prop);
    ErrorString DeleteWord( string &word );
    ErrorString AddWord( string &word );
    ErrorString ContainsWord( string &word, bool &yesno );
    ErrorString AddRelation( string &word1, string &rel,
                             string &word2 );
    ErrorString GetRelationPred( string &word, string &rel,
                                 SetOfWord &pred );
    ErrorString GetRelationAllPred( string &word, string &rel,
                                    SetOfWord &pred );
    ErrorString GetRelationSucc( string &word, string &rel,
                                 SetOfWord &succ );
    ErrorString GetRelationAllSucc( string &word, string &rel,
                                    SetOfWord &succ );
    ErrorString AreRelated( string &word1, SetOfString &rel,
                            string &word2, bool &yesno );
    ErrorString Add( ConceptualGraph *cg, KeyCG &newKey );
    ErrorString Replace( KeyCG aKey, ConceptualGraph *cg );
    ErrorString Join( KeyCG aKey, ConceptualGraph *cg );
    ErrorString Match( ConceptualGraph *query,
                       SetOfString &selecRules,
                       SetOfMatchResult &matches,
                       KeyCG profile = -1);
    ErrorString GetGraph( KeyCG aKey, ConceptualGraph **cg );
    ErrorString GetNumberOfVariants( string &conType, int &num );
    ErrorString GetNthVariant( string &conType, int num, KeyCG &theKey );
    ErrorString GetNumberOfGraphs( int &num );
    ErrorString GetNthGraph( int num, KeyCG &theKey );
    ErrorString AddProvider( KeyCG aKey,
                             string &theMaintainer, string &anOID );
    ErrorString RemoveProvider( KeyCG aKey, string &anOID );
    ErrorString GetProviders( KeyCG aKey, SetOfServiceEntry &result );

    ErrorString CreateProfile( KeyCG &aKey );
    ErrorString RemoveProfile( KeyCG aKey );
    ErrorString SetMinmatchval( KeyCG aKey, long minmatchval );
    ErrorString SetTimeout( KeyCG aKey, long timeout );
    ErrorString GetMinmatchval( KeyCG aKey, long &minmatchval );
    ErrorString GetTimeout( KeyCG aKey, long &timeout );

    bool CheckForError();
    void SetError( int err, string *msg );
    void SetAnswer();
    void SetAnswer( ConceptualGraph *cg );
    void SetAnswer( bool flag );
    void SetAnswer( string &str );
    void SetAnswer( SetOfMatchResult &result );
    void SetAnswer( SetOfServiceEntry &entries );
    void SetAnswer( SetOfWord &word_set );
    void SetAnswer( KeyCG aKey );
    void SetAnswer( long number );
    ErrorString GetAnswer();
    ErrorString GetAnswer( ConceptualGraph **cg );
    ErrorString GetAnswer( bool &flag );
    ErrorString GetAnswer( string &str );
    ErrorString GetAnswer( SetOfMatchResult &result );
    ErrorString GetAnswer( SetOfServiceEntry &entries );
    ErrorString GetAnswer( SetOfWord &word_set );
    ErrorString GetAnswer( KeyCG &key );
    ErrorString GetAnswer( long &number );
    string GetAnswerString();
};

#endif
