/* -*- c++ -*- */
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


/* $Id: scanner.l,v 1.23 1998/02/06 11:05:51 roemer Exp $ */


%{
#include <unistd.h>
#include <iostream.h>
#include <fstream.h>
#include "stl.h"

#include "cg.h"
#include "match.h"
#include "kb.h"
#include "services.h"
#include "dict.h"
#include "yacc.h"
#include "util.h"
#include "parser.h"

#define YY_READ_BUF_SIZE 8192
#define MAX_IMPORT_DEPTH 50

YY_BUFFER_STATE  import_stack[ MAX_IMPORT_DEPTH ];
char            *import_file_name[ MAX_IMPORT_DEPTH ];
int              import_line_no[ MAX_IMPORT_DEPTH ];
int              import_stack_ptr = 0;

int line_no = 1;

string
locate_import_file (const char *fname)
{
    if (fname[0] == '/' || !access (fname, R_OK))
        // absolute path or file exits
        return fname;

    char buf[1000];
    int n = readlink (theParser->GetLexfile(), buf, sizeof (buf));
    if (n > 0)
        buf[n] = 0;
    string basename (n > 0 ? buf : theParser->GetLexfile());
    string::size_type pos = basename.rfind ('/');
    if (pos > basename.size())
        return fname;
    return basename.substr (0, pos+1) + fname;
}

%}

%x comment
%x import
%x mrn
%x mr

DIGIT	[0-9]
CHAR	[a-z0-9._]
CHARWS	[a-z0-9._ ]

%%

[ \t]					;
[\n]					line_no++;
"/*"					BEGIN(comment);
<comment>[^*\n]*		;
<comment>"*"+[^*/\n]*	;
<comment>\n				line_no++;
<comment>"*"+"/"		BEGIN(INITIAL);

@import					BEGIN(import);
<import>[ \t]*			;
<import>[^ \t\n]+		{
    if ( import_stack_ptr >= MAX_IMPORT_DEPTH ) {
        cerr << "Imports nested too deeply";
        exit( 1 );
    }
    import_file_name[ import_stack_ptr ] = theParser->GetLexfile();
    import_line_no[ import_stack_ptr ] = line_no;
    line_no = 1;
    import_stack[ import_stack_ptr++ ] = YY_CURRENT_BUFFER;

    char *fname = strdup ((char *)locate_import_file (YYText()).c_str());
    ifstream *file = new ifstream (fname);
    
    if( !(*file) ) {
        cerr << "Couldn't open import file `" << fname << "'" << endl;
        exit( 1 );
    }
    
    theParser->SetLexfile (fname);
    free (fname);
 
    yy_switch_to_buffer( yy_create_buffer( file, YY_BUF_SIZE ) );
    
    BEGIN(INITIAL);
}

<<EOF>>					{
    if( --import_stack_ptr < 0 ) {
        import_stack_ptr = 0;
        yyterminate();
    } else {
        yy_delete_buffer( YY_CURRENT_BUFFER );
        yy_switch_to_buffer(import_stack[ import_stack_ptr ] );
        theParser->SetLexfile (import_file_name[ import_stack_ptr ]);
        line_no = import_line_no[ import_stack_ptr ];
    }
}

<mrn>[ \t]				;
<mrn>[\n]				line_no++;
<mrn>[a-z]{CHAR}*		{
    yylval.str = new string( YYText() );
    BEGIN(mr);
    return STRING;
}
<mrn>\"[^\"]*\"			{
    yylval.str =
        new string( string( YYText() ).substr( 1, YYLeng() - 2 ) );
    BEGIN(mr);
    return STRING;
}
<mrn>'[^']*'			{
    yylval.str = new string( string( YYText() ).substr( 1, YYLeng() - 2 ) );
    BEGIN(mr);
    return STRING;
}
<mr>[ \t]				;
<mr>[\n]				line_no++;
<mr>.					{
    string *rule = new string( YYText() );
    int c;

    while( ( c = yyinput() ) && c != EOF ) {
        if( (char) c == '\n' )
            line_no++;
        *rule += (char) c;
        int len = rule->size();
        if( len > 4 && rule->substr( len - 4, 4 ) == "@end" )
            break;
    }
    BEGIN(INITIAL);
    if( c != EOF )
        *rule = rule->substr( 0, rule->size() - 4 );
    yylval.str = rule;
    return STRING;
}

@version			return VERSION;
@save				return SAVE;
@define-matching-rule	BEGIN(mrn); return DEFINE_MATCHING_RULE;
@delete-matching-rule		return DELETE_MATCHING_RULE;
@show-matching-rules		return SHOW_MATCHING_RULE_NAMES;
@show-matching-rule		return SHOW_MATCHING_RULE;
@add				return ADD;
@remove				return REMOVE;
@join				return JOIN;
@match				return MATCH;
@get-graph			return GET_GRAPH;
@number-of-variants		return NUMBER_OF_VARIANTS;
@number-of-graphs		return NUMBER_OF_GRAPHS;
@verbose-on			return VERBOSE_ON;
@verbose-off			return VERBOSE_OFF;
@dict-quick-mode-on		return DICT_QUICK_MODE_ON;
@dict-quick-mode-off		return DICT_QUICK_MODE_OFF;
@add-provider			return ADD_PROVIDER;
@remove-provider		return REMOVE_PROVIDER;
@get-provider			return GET_PROVIDER;
@echo				return ECHO;

@create-profile			return CREATE_PROFILE;
@remove-profile			return REMOVE_PROFILE;
@set-minmatchval		return SET_MINMATCHVAL;
@set-timeout			return SET_TIMEOUT;
@get-minmatchval		return GET_MINMATCHVAL;
@get-timeout			return GET_TIMEOUT;

@register-relation		return REGISTER_RELATION;
@get-relation-prop		return GET_RELATION_PROP;
@show-all-relations		return SHOW_ALL_RELATIONS;
@symmetric			return REL_SYMMETRIC;
@reflexive			return REL_REFLEXIVE;
@transitive			return REL_TRANSITIVE;
@acyclic			return REL_ACYCLIC;
@delete-word			return DELETE_WORD;
@add-word			return ADD_WORD;
@contains-word			return CONTAINS_WORD;
@add-relation			return ADD_RELATION;
@get-relation-pred		return GET_RELATION_PRED;
@get-relation-all-pred		return GET_RELATION_ALL_PRED;
@get-relation-succ		return GET_RELATION_SUCC;
@get-relation-all-succ		return GET_RELATION_ALL_SUCC;
@are-related			return ARE_RELATED;

@error				return ERROR;
@ok				return ANSWER_OK;
@answer-graph			return ANSWER_GRAPH;
@answer-flag			return ANSWER_FLAG;
@yes				return ANSWER_YES;
@no				return ANSWER_NO;
@answer-string			return ANSWER_STR;
@answer-matches			return ANSWER_MATCHES;
@answer-providers		return ANSWER_PROVIDERS;
@answer-words			return ANSWER_WORDS;
@answer-key             	return ANSWER_KEY;
@answer-properties		return ANSWER_PROPERTIES;
@answer-int			return ANSWER_INT;
"["				return CONCEPT_OPEN;
"]"				return CONCEPT_CLOSE;
"("				return RELATION_OPEN;
")"				return RELATION_CLOSE;
"{"				return ID_LIST_OPEN;
"}"				return ID_LIST_CLOSE;
":"				return COLON;
","				return COMMA;
"->"				return CONNECTS;
"-"				return SUBGRAPH_OPEN;
"."				return SUBGRAPH_CLOSE;
"="				return EQUAL;
$[0-9]+                         {
                                    yylval.key = atoi( string( YYText() ).substr( 1,
                                    YYLeng() - 1 ).c_str() );
                                    return KEY;
                                }
[a-z][a-z0-9_-]*		{
                                     yylval.word = new Word( YYText(), true );
                                     return WORD;
                                }
'([^']|"\\'")*'			{
				     string tmp = string( YYText() ).substr( 1,
                                                          YYLeng() - 2 );
                                     gsub ( tmp, "\\'", "'" );
				     yylval.word = new Word( tmp, false );
				     return WORD;
                                }
	{
    string *rule = new string();
    int c;

    while( ( c = yyinput() ) && c != EOF && (char)c != (char)'') {
        if (c == '\n')
            line_no++;
        *rule += (char) c;
    }
    yylval.str = rule;
    return STRING;
}
\"[^\"]*\"			{
				     yylval.word =
					new Word( string( YYText() ).substr( 1,
						         YYLeng() - 2 ),
                                                  true);
				     return WORD;
				}
[0-9]+				{
				     yylval.val = atoi( YYText() );
				     return VALUE;
				}


