
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */

// $Id: server.h,v 1.9 1997/06/27 16:19:19 roemer Exp $


#ifndef __SERVER_H__
#define __SERVER_H__

#include "key.h"
#include <fstream.h>
#include "interface.h"
#include "dict.h"
#include "match.h"
#include "kb.h"
#include "services.h"
#include "profiles.h"


class Server : public AITraderInterface
{
private:
    Dict         dict;
    MatchingRule rules;
    KB           kb;
    Services     services;
    Profiles	 profiles;
    bool         verbose;

    // Last error message
    ErrorString         errorString;

    // Holds complete answer in one string
    string              answerString;

    // Answer variables
    bool                boolAnswer;
    string              strAnswer;
    KeyCG               keyAnswer;
    ConceptualGraph    *cgAnswer;
    SetOfMatchResult    matchesAnswer;
    SetOfServiceEntry   serviceEntriesAnswer;
    SetOfWord           wordSetAnswer;
    long		intAnswer;

public:
    Server();
    virtual ~Server();
    ErrorString GetVersion( string &version );
    ErrorString SetVerbose( bool flag );
    ErrorString SaveTraderDB( string *name );
    ErrorString DefineMatchingRule( string *name, string *rule );
    ErrorString DeleteMatchingRule( string *name );
    ErrorString GetMatchingRuleNames( string *name );
    ErrorString GetMatchingRule( string *name, string *rule );

    // Dictionary
    ErrorString DictQuickMode( bool flag );
    ErrorString RegisterRelation( string &rel, int prop );
    ErrorString GetAllRelations( SetOfString &rels );
    ErrorString GetRelationProperties (string &rel, int &prop);
    ErrorString DeleteWord( string &word );
    ErrorString AddWord( string &word );
    ErrorString ContainsWord( string &word, bool &yesno );
    ErrorString AddRelation( string &word1, string &rel, string &word2 );
    ErrorString GetRelationPred( string &word, string &rel, SetOfWord &pred );
    ErrorString GetRelationAllPred( string &word, string &rel, SetOfWord &pd );
    ErrorString GetRelationSucc( string &word, string &rel, SetOfWord &succ );
    ErrorString GetRelationAllSucc( string &word, string &rel, SetOfWord &sc );
    ErrorString AreRelated( string &word1, SetOfString &rels, 
                            string &word2, bool &yesno );

	// Knowledgebase
    ErrorString Add( ConceptualGraph *cg, KeyCG &newKey );
    ErrorString Replace( KeyCG aKey, ConceptualGraph *cg );
    ErrorString Join( KeyCG aKey, ConceptualGraph *cg );
    ErrorString Match( ConceptualGraph *cg,
                       SetOfString &selecRules,
                       SetOfMatchResult &matches,
                       KeyCG profile = -1);
    ErrorString GetGraph( KeyCG aKey, ConceptualGraph **cg );
    ErrorString GetNumberOfVariants( string &conType, int &num );
    ErrorString GetNthVariant( string &conType, int num, KeyCG &theKey );
    ErrorString GetNumberOfGraphs( int &num );
    ErrorString GetNthGraph( int num, KeyCG &theKey );

	// Service offer management
    ErrorString AddProvider( KeyCG aKey,
                             string &theMaintainer, string &anOID );
    ErrorString RemoveProvider( KeyCG aKey, string &anOID );
    ErrorString GetProviders( KeyCG aKey, SetOfServiceEntry &result );

	// Profiles
    ErrorString CreateProfile( KeyCG &aKey );
    ErrorString RemoveProfile( KeyCG aKey );
    ErrorString SetMinmatchval( KeyCG aKey, long minmatchval );
    ErrorString SetTimeout( KeyCG aKey, long timeout );
    ErrorString GetMinmatchval( KeyCG aKey, long &minmatchval );
    ErrorString GetTimeout( KeyCG aKey, long &timeout );

	// Answer mechanism
    void SetError( int err, string *msg );
    void SetAnswer();
    void SetAnswer( ConceptualGraph *cg );
    void SetAnswer( bool flag );
    void SetAnswer( string &str );
    void SetAnswer( SetOfMatchResult &result );
    void SetAnswer( SetOfServiceEntry &entries );
    void SetAnswer( SetOfWord &word_set );
    void SetAnswer( KeyCG aKey );
    void SetAnswer( long number );
    ErrorString GetAnswer();
    ErrorString GetAnswer( ConceptualGraph **cg );
    ErrorString GetAnswer( bool &flag );
    ErrorString GetAnswer( string &str );
    ErrorString GetAnswer( SetOfMatchResult &result );
    ErrorString GetAnswer( SetOfServiceEntry &entries );
    ErrorString GetAnswer( SetOfWord &word_set );
    ErrorString GetAnswer( KeyCG &aKey );
    ErrorString GetAnswer( long &number );
    string GetAnswerString();
    bool CheckForError();
};

#endif
