
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: services.cc,v 1.6 1997/02/19 15:06:45 roemer Exp $


#include "services.h"


ServiceEntry::ServiceEntry( string &theMaintainer, string &anOID )
	: maintainer( theMaintainer ), oid( anOID )
{
}


string ServiceEntry::GetMaintainer() const
{
	return maintainer;
}


string ServiceEntry::GetOID() const
{
	return oid;
}


void ServiceEntry::Print( ostream &ostr ) const
{
	ostr << "'";
	ostr << maintainer;
	ostr << "', '";
	ostr << oid;
	ostr << "'";
}
	

Services::Services()
{
}


Services::~Services()
{
}


void Services::RegisterNewProvider( KeyCG aKey,	string theMaintainer,
									string anOID )
{
	services[ aKey ].insert( ServiceEntry( theMaintainer, anOID ) );
}



void Services::DeleteProvider( KeyCG aKey, string anOID )
{
	if( services.count( aKey ) == 0 )
		return;
	SetOfServiceEntry &m = services[ aKey ];

	SetOfServiceEntry::iterator i;
	for( i = m.begin(); i != m.end(); ) {
		if( (*i).GetOID() == anOID ) {
            /*
             * you cannot earase an entry for iterator i and then
             * do an ++i without coredumping...
             */
			m.erase( i );
            i = m.begin();
        } else
            ++i;
	}
}



SetOfServiceEntry Services::GetAllProviders( KeyCG aKey )
{
	if( services.count( aKey ) == 0 )
		return SetOfServiceEntry();
	return services[ aKey ];
}


void Services::DeleteAllProviders (KeyCG aKey)
{
    services.erase (aKey);
}


void Services::Print( ostream &ostr )
{
	map<KeyCG, SetOfServiceEntry, less<KeyCG> >::iterator i1;
	for( i1 = services.begin(); i1 != services.end(); i1++ ) {
		SetOfServiceEntry::iterator i2;
		for( i2 = (*i1).second.begin(); i2 != (*i1).second.end(); i2++ ) {
			ostr << "@add-provider $";
			ostr << (*i1).first;
			ostr << ": ";
			(*i2).Print( ostr );
			ostr << endl;
		}
	}
}


	
