
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: services.h,v 1.3 1997/02/19 15:06:45 roemer Exp $
 
 
#ifndef __SERVICES_H__
#define __SERVICES_H__

#include "key.h"
#include <fstream.h>


class ServiceEntry 
{
private:
	string maintainer;
	string oid;
public:
	ServiceEntry()
	{
	}
	ServiceEntry( string &theMaintainer, string &anOID );
	ServiceEntry( const ServiceEntry &s )
	{
		maintainer = s.maintainer;
		oid = s.oid;
	}
	~ServiceEntry()
	{
	}
	ServiceEntry &operator=( const ServiceEntry &s )
	{
		maintainer = s.maintainer;
		oid = s.oid;
		return *this;
	}
	bool operator==( const ServiceEntry &s ) const
	{
		return oid == s.oid;
	}
	bool operator<( const ServiceEntry &s ) const
	{
		return oid < s.oid;
	}
	string GetMaintainer() const;
	string GetOID() const;
	void Print( ostream &ostr ) const;
};


typedef set<ServiceEntry, less<ServiceEntry> > SetOfServiceEntry;



class Services 
{
private:
	map<KeyCG, SetOfServiceEntry, less<KeyCG> > services;
public:
	Services();
	~Services();
	void RegisterNewProvider( KeyCG aKey, string theMaintainer, string anOID );
	void DeleteProvider( KeyCG aKey, string anOID );
	SetOfServiceEntry GetAllProviders( KeyCG aKey );
	void DeleteAllProviders (KeyCG aKey);
	void Print( ostream &ostr );
};





#endif
