
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


/* $Id: util.cc,v 1.4 1998/02/06 11:05:51 roemer Exp $ */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "stl.h"
#include "util.h"

string
LocateInitialKB (void)
{
    string s;

    char *homedir = getenv ("HOME");
    if (homedir) {
        s = string (homedir) + "/.kbrc";
        if (!access (s.c_str(), R_OK))
            return s;
    }
    char *path = getenv ("PATH");
    if (path && (path = strdup (path))) {
        char *cp = strtok (path, ":");
        for ( ; cp; cp = strtok (NULL, ":")) {
            s = string (cp) + "/initial.kb";
            if (!access (s.c_str(), R_OK))
                return s;
        }
    }
    return "initial.kb";
}

/*
 * substitutes for g++ String methods
 */

extern "C" {
    int strcasecmp (const char *, const char *);
}


string &
gsub (string &s, const string &pat, const string &subst)
{
    string::size_type pos = s.find (pat, 0);
    for ( ; pos < s.length(); pos = s.find (pat, pos)) {
        s.replace (pos, pat.length(), subst);
        pos += subst.length();
    }
    return s;
}


int
fcompare (const string &s1, const string &s2)
{
    return strcasecmp (s1.c_str(), s2.c_str());
}


int
contains (const string &s, const string &pat)
{
    return s.find (pat, 0) < s.length();
}


int
split (const string &s, string res[], int maxn, const string &sep)
{
    int i, lastpos = 0;
    string::size_type pos = s.find (sep, 0);
    for (i = 0; i < maxn && pos < s.length(); ++i) {
        res[i] = s.substr (lastpos, pos - lastpos);
        lastpos = pos + sep.length();
        pos = s.find (sep, lastpos);
    }
    if (i < maxn && lastpos < s.length())
        res[i++] = s.substr (lastpos, s.length() - lastpos);
    return i;
}

/*
 * return everything in 's' before the first occurence of 'pat' that
 * is after position 'pos'
 */
string
before (const string &s, const string &pat, int pos)
{
    string::size_type npos = s.find (pat, pos);
    return npos < s.length() ? s.substr (0, npos) : string("");
}

/*
 * return everything in 's' after the first occurence of 'pat' that
 * is after position 'pos'
 */
string
after (const string &s, const string &pat, int pos)
{
    string::size_type npos = s.find (pat, pos);
    return npos < s.length()
        ? s.substr (npos + pat.length(), s.length() - (npos + pat.length()))
        : string ("");
}

/*
 * delete the first occurence of 'pat' that starts after 'pos' in 's'
 */
void
del (string &s, const string &pat, int pos)
{
    string::size_type npos = s.find (pat, pos);
    if (npos < s.length())
        //AP s.remove (npos, pat.length());
        s.replace (npos, pat.length(), "");
}

void
prepend (string &s, const string &s2)
{
    s = s2 + s;
}

int
freq (const string &s, const string &pat)
{
    int cnt = 0;
    string::size_type pos = s.find (pat, 0);
    for ( ; pos < s.length(); pos = s.find (pat, pos)) {
        ++cnt;
        pos += pat.length();
    }
    return cnt;
}

void
cat (const string &s1, const string &s2, string &res)
{
    res = s1 + s2;
}
