
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


// $Id: word.h,v 1.6 1998/02/06 11:05:52 roemer Exp $


#ifndef __WORD_H__
#define __WORD_H__


#include <iostream.h>
#include "stl.h"


template<class T>
class Ptr
{
public:
    Ptr(T *p=0) : ptr(p) {}
    Ptr(const Ptr &p) : ptr(p.ptr) {}
    ~Ptr() {}
    Ptr &operator=(const Ptr &p) 
    {
        ptr=p.ptr;
        return *this;
    }
    T &operator * () 
    {
        return *ptr;
    }
    const T & operator * () const
    {
        return *ptr;
    }
    bool operator==(const Ptr &p) const 
    {
        return *ptr==*p.ptr;
    }
    bool operator<(const Ptr &p) const 
    {
        return *ptr<*p.ptr;
    }
    operator const T * () const
    {
        return ptr;
    }
private:
    T *ptr;
};


/*
 * XXX 
 * Normalisierung defaultmaessig ausgeschaltet
 */

class Word : public string
{
        
public:

    typedef Ptr<Word> pointer;
    typedef Ptr<const Word> const_pointer;

    Word() {}
    Word(const char *s, bool do_convert=false)
    {
        if (do_convert)
            *(string*)this=convert(s);
        else
            *(string*)this=(string)s;
    }
    Word(const string &s, bool do_convert=false)
    {
        if (do_convert)
            *(string*)this=convert(s.c_str());
        else
            *(string*)this=s;
    }
    Word(const Word &w) 
    {
        *this=w;
    }
    ~Word() {}
    Word &operator=(const Word &w)
    {
        *(string*)this=w;
        return *this;
    }
    bool operator==(const Word &w) const
    {
        return (*(string*)this==(const string &)w);
    }
    bool operator<(const Word &w) const
    {
        return (*(string*)this < (const string &)w);
    }
    bool operator!=(const Word &w) const
    {
        return !(*this==w);
    }
    operator const char *() const
    {
        return c_str();
    } 
    friend ostream &operator<<( ostream &o, const Word &w );

protected:

    string convert(const char *);
    
};


#endif
