
/*
 *  AI-Trader --- Service Trading using Conceptual Graphs
 *  Copyright (C) 1995 Arno Puder et al.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 aitrader@informatik.uni-frankfurt.de
 */


/* $Id: yacc.y,v 1.19 1997/06/27 16:19:20 roemer Exp $ */


%{

#include <string.h>
#include <iostream.h>

#include "cg.h"
#include "match.h"
#include "parser.h"
#include "kb.h"
#include "services.h"
#include "dict.h"


extern int yylex();
extern int line_no;
extern char *lexFile;


void yyerror( char *s )
{
    if (strlen (s) == 0)
        s = "parse error";
    theParser->SetError( theParser->GetLexfile(), s, line_no );
}


%}

%union {
	int				    val;
	KeyCG               key;
	Word               *word;
	SetOfWord		   *words;
	string             *str;
	SetOfString        *strs;
	Node			   *node;
	ConceptualGraph	   *cg;
	SetOfMatchResult   *matches;
	ServiceEntry	   *prov;
	SetOfServiceEntry  *provs;
}

%token			VERSION
%token			SAVE
%token			DEFINE_MATCHING_RULE
%token			DELETE_MATCHING_RULE
%token			SHOW_MATCHING_RULE_NAMES
%token			SHOW_MATCHING_RULE

%token			ADD
%token			REMOVE
%token			JOIN
%token			MATCH
%token			GET_GRAPH
%token			NUMBER_OF_VARIANTS
%token			NUMBER_OF_GRAPHS
%token			VERBOSE_ON
%token			VERBOSE_OFF
%token			DICT_QUICK_MODE_ON
%token			DICT_QUICK_MODE_OFF
%token			ADD_PROVIDER
%token			REMOVE_PROVIDER
%token			GET_PROVIDER
%token			ECHO

%token			CREATE_PROFILE
%token			REMOVE_PROFILE
%token			SET_MINMATCHVAL
%token			SET_TIMEOUT
%token			GET_MINMATCHVAL
%token			GET_TIMEOUT

%token			REGISTER_RELATION
%token			GET_RELATION_PROP
%token			SHOW_ALL_RELATIONS
%token			REL_SYMMETRIC
%token			REL_REFLEXIVE
%token			REL_TRANSITIVE
%token			REL_ACYCLIC
%token			DELETE_WORD
%token			ADD_WORD
%token			CONTAINS_WORD
%token			ADD_RELATION
%token			GET_RELATION_PRED
%token			GET_RELATION_ALL_PRED
%token			GET_RELATION_SUCC
%token			GET_RELATION_ALL_SUCC
%token			ARE_RELATED

%token			ERROR
%token			ANSWER_OK
%token			ANSWER_GRAPH
%token			ANSWER_FLAG
%token			ANSWER_YES
%token			ANSWER_NO
%token			ANSWER_STR
%token			ANSWER_MATCHES
%token			ANSWER_PROVIDERS
%token			ANSWER_WORDS
%token          	ANSWER_KEY
%token			ANSWER_PROPERTIES
%token			ANSWER_INT

%token			TOKEN_UNDEF
%token			CONCEPT_OPEN
%token			CONCEPT_CLOSE
%token			ID_LIST_OPEN
%token			ID_LIST_CLOSE
%token			COLON
%token			RELATION_OPEN
%token			RELATION_CLOSE
%token			CONNECTS
%token			COMMA
%token			SUBGRAPH_OPEN
%token			SUBGRAPH_CLOSE
%token			EQUAL

%token			BRACKET_OPEN
%token			BRACKET_CLOSE
%token			SEMICOLON
%token			NODES_ACCEPT
%token			NODES_REJECT
%token			MATCH_GRAPHS
%token <key>    KEY
%token <word>	WORD
%token <str>	STRING
%token <val>	VALUE

%type <matches>	matches
%type <provs>	providers
%type <prov>	provider
%type <words>	words
%type <words>   word_list

%type <str>     string
%type <strs>    string_list

%type <val>		simple_word_rel_prop
%type <val>		word_rel_prop
%type <val>		word_rel_prop2
%type <val>	property
%type <val>	properties

%type <node>	concept_node
%type <node>	relation_node
%type <cg>		cg
%type <cg>		con_connects_to
%type <cg>		rel_graph
%type <cg>		con_subgraph
%type <cg>		rel_subgraph

%%

grammar
	:
	| grammar simple_command
	;

simple_command
	: VERSION
            {
                string version;
                theParser->GetServer()->GetVersion( version );
                theParser->GetServer()->SetAnswer( version );
            }
	| VERBOSE_ON
	    {
                theParser->GetServer()->SetVerbose( true );
                theParser->GetServer()->SetAnswer();
	    }
	| VERBOSE_OFF
	    {
                theParser->GetServer()->SetVerbose( false );
                theParser->GetServer()->SetAnswer();
	    }
	| DICT_QUICK_MODE_ON
	    {
                theParser->GetServer()->DictQuickMode( true );
                theParser->GetServer()->SetAnswer();
            }
	| DICT_QUICK_MODE_OFF
	    {
                theParser->GetServer()->DictQuickMode( false );
                theParser->GetServer()->SetAnswer();
            }
	| SAVE string
	    {
                theParser->GetServer()->SaveTraderDB( $2 );
                theParser->GetServer()->SetAnswer();
                delete $2;
	    }
	| DEFINE_MATCHING_RULE string string
	    {
                theParser->GetServer()->DefineMatchingRule( $2, $3 );
                theParser->GetServer()->SetAnswer();
                delete $2;
	    }
	| DELETE_MATCHING_RULE string
	    {
    		theParser->GetServer()->DeleteMatchingRule( $2 );
                theParser->GetServer()->SetAnswer();
                delete $2;
	    }
	| SHOW_MATCHING_RULE_NAMES
	    {
                string names;
                theParser->GetServer()->GetMatchingRuleNames( &names );
                theParser->GetServer()->SetAnswer( names );
	    }
	| SHOW_MATCHING_RULE string
	    {
                string rule;
                theParser->GetServer()->GetMatchingRule( $2, &rule );
                theParser->GetServer()->SetAnswer( rule );
                delete $2;
            }
	| REGISTER_RELATION string word_rel_prop
	    {
                theParser->GetServer()->RegisterRelation( *$2, $3 );
                theParser->GetServer()->SetAnswer();
                delete $2;
	    }
	| GET_RELATION_PROP string
	    {
                int prop;
                theParser->GetServer()->GetRelationProperties (*$2, prop);
                // XXX
                theParser->GetServer()->SetAnswer();
                delete $2;
            }
	| SHOW_ALL_RELATIONS
	    {
                SetOfString rels;
                theParser->GetServer()->GetAllRelations( rels );
                string result;
                for( SetOfString::iterator i = rels.begin(); i != rels.end(); i++ ) {
                    result += *i;
                    result += '\n';
                }
                theParser->GetServer()->SetAnswer( result );
            }
	| DELETE_WORD WORD
	    {
                theParser->GetServer()->DeleteWord( *$2 );
                theParser->GetServer()->SetAnswer();
                delete $2;
	    }
	| ADD_WORD WORD
	    {
                theParser->GetServer()->AddWord( *$2 );
                theParser->GetServer()->SetAnswer();
                delete $2;
	    }
	| CONTAINS_WORD WORD
	    {
                bool yesno;
                theParser->GetServer()->ContainsWord( *$2, yesno );
                theParser->GetServer()->SetAnswer( yesno );
                delete $2;
	    }
	| ADD_RELATION WORD COLON string COLON WORD
	    {
                theParser->GetServer()->AddRelation( *$2, *$4, *$6 );
                theParser->GetServer()->SetAnswer();
                delete $2;
                delete $4;
                delete $6;
	    }
	| GET_RELATION_PRED WORD COLON string
	    {
                SetOfWord pred;
                theParser->GetServer()->GetRelationPred( *$2, *$4, pred );
                theParser->GetServer()->SetAnswer( pred );
                delete $2;
                delete $4;
	    }
	| GET_RELATION_ALL_PRED WORD COLON string
	    {
                SetOfWord pred;
                theParser->GetServer()->GetRelationAllPred( *$2, *$4, pred );
                theParser->GetServer()->SetAnswer( pred );
                delete $2;
                delete $4;
	    }
	| GET_RELATION_SUCC WORD COLON string
	    {
                SetOfWord succ;
                theParser->GetServer()->GetRelationSucc( *$2, *$4, succ );
                theParser->GetServer()->SetAnswer( succ );
                delete $2;
                delete $4;
	    }
	| GET_RELATION_ALL_SUCC WORD COLON string
	    {
                SetOfWord succ;
                theParser->GetServer()->GetRelationAllSucc( *$2, *$4, succ );
                theParser->GetServer()->SetAnswer( succ );
                delete $2;
                delete $4;
	    }
	| ARE_RELATED WORD COLON string_list COLON WORD
	    {
                bool yesno;
                theParser->GetServer()->AreRelated( *$2, *$4, *$6, yesno );
                theParser->GetServer()->SetAnswer( yesno );
                delete $2;
                delete $4;
                delete $6;
	    }
	| ADD cg
	    {
                KeyCG key;
                theParser->GetServer()->Add( $2, key );
                theParser->GetServer()->SetAnswer( key );
                delete $2;
	    }
	| ADD KEY COLON cg
	    {
                theParser->GetServer()->Replace( $2, $4 );
                theParser->GetServer()->SetAnswer();
                delete $4;
            }
	| REMOVE KEY
	    {
                theParser->GetServer()->Replace ($2, NULL);
                theParser->GetServer()->SetAnswer();
            }
	| JOIN KEY COLON cg
	    {
                theParser->GetServer()->Join( $2, $4 );
                theParser->GetServer()->SetAnswer();
                delete $4;
            }
	| MATCH string_list COLON cg
	    {
                SetOfMatchResult result;
                
                theParser->GetServer()->Match( $4, *$2, result );
                theParser->GetServer()->SetAnswer( result );
//		delete $2;
                delete $4;
	    }
	| MATCH KEY COLON string_list COLON cg
	    {
                SetOfMatchResult result;
                
                theParser->GetServer()->Match( $6, *$4, result, $2 );
                theParser->GetServer()->SetAnswer( result );
//		delete $4;
                delete $6;
	    }
	| GET_GRAPH KEY
	    {
                ConceptualGraph *cg;
                theParser->GetServer()->GetGraph( $2, &cg );
                theParser->GetServer()->SetAnswer( cg );
	    }
	| GET_GRAPH WORD COLON VALUE
	    {
                KeyCG key;
                theParser->GetServer()->GetNthVariant( *$2, $4, key );
                theParser->GetServer()->SetAnswer( key );
                delete $2;
	    }
	| NUMBER_OF_VARIANTS WORD
	    {
                char buf[ 20 ];
                int num;
                theParser->GetServer()->GetNumberOfVariants( *$2, num );
                delete $2;
                sprintf( buf, "%d", num );
                string tmp = buf;
                theParser->GetServer()->SetAnswer(tmp);
            }
	| NUMBER_OF_GRAPHS
	| ADD_PROVIDER KEY COLON string COMMA string
	    {
                theParser->GetServer()->AddProvider( $2, *$4, *$6 );
                theParser->GetServer()->SetAnswer();
                delete $4;
                delete $6;
            }
	| REMOVE_PROVIDER KEY COMMA string
	    {
                theParser->GetServer()->RemoveProvider( $2, *$4 );
                theParser->GetServer()->SetAnswer();
                delete $4;
            }
	| GET_PROVIDER KEY
	    {
                SetOfServiceEntry result;
                
                theParser->GetServer()->GetProviders( $2, result );
                theParser->GetServer()->SetAnswer( result );
//		delete result;
            }
	| CREATE_PROFILE
	    {
                KeyCG key;
                theParser->GetServer()->CreateProfile (key);
                theParser->GetServer()->SetAnswer (key);
            }
	| REMOVE_PROFILE KEY
	    {
                theParser->GetServer()->RemoveProfile ($2);
                theParser->GetServer()->SetAnswer (true);
            }
	| SET_MINMATCHVAL KEY COLON VALUE
	    {
                theParser->GetServer()->SetMinmatchval ($2, $4);
                theParser->GetServer()->SetAnswer (true);
            }
	| SET_TIMEOUT KEY COLON VALUE
	    {
                theParser->GetServer()->SetTimeout ($2, $4);
                theParser->GetServer()->SetAnswer (true);
            }
	| GET_MINMATCHVAL KEY
	    {
                long result;
                theParser->GetServer()->GetMinmatchval ($2, result);
                theParser->GetServer()->SetAnswer (result);
            }
	| GET_TIMEOUT KEY
	    {
                long result;
                theParser->GetServer()->GetTimeout ($2, result);
                theParser->GetServer()->SetAnswer (result);
            }
	| ANSWER_OK
	    {
                theParser->GetServer()->SetAnswer();
            }
	| ANSWER_GRAPH EQUAL
	    {
                theParser->GetServer()->SetAnswer( (ConceptualGraph *) NULL );
            }
	| ANSWER_GRAPH EQUAL cg
	    {
                theParser->GetServer()->SetAnswer( $3 );
            }
	| ANSWER_FLAG EQUAL ANSWER_YES
	    {
                theParser->GetServer()->SetAnswer( true );
            }
	| ANSWER_FLAG EQUAL ANSWER_NO
	    {
                theParser->GetServer()->SetAnswer( false );
            }
	| ANSWER_STR EQUAL string
	    {
                theParser->GetServer()->SetAnswer( *$3 );
                delete $3;
            }
	| ANSWER_MATCHES EQUAL matches
	    {
                theParser->GetServer()->SetAnswer( *$3 );
                delete $3;
            }
	| ANSWER_PROVIDERS EQUAL providers
	    {
                theParser->GetServer()->SetAnswer( *$3 );
                delete $3;
            }
	| ANSWER_WORDS EQUAL words
	    {
                theParser->GetServer()->SetAnswer( *$3 );
                delete $3;
            }
	| ANSWER_KEY EQUAL KEY
            {
                theParser->GetServer()->SetAnswer( $3 );
            }
	| ANSWER_PROPERTIES properties
	    {
            }
	| ANSWER_INT EQUAL VALUE
	    {
                theParser->GetServer()->SetAnswer ((long)$3);
            }
	| ERROR VALUE COLON string
	    {
                theParser->GetServer()->SetError( $2, $4 );
                delete $4;
            }
	| ECHO string
	    {
                cout << *$2 << endl;
                theParser->GetServer()->SetAnswer( *$2 );
                delete $2;
	    }
	;

matches
	:
	    {
                $$ = new SetOfMatchResult();
            }
	| KEY COMMA VALUE
	    {
                $$ = new SetOfMatchResult();
                $$->insert( MatchResult( $1, NULL, $3 ) );
            }
	| matches COLON KEY COMMA VALUE
	    {
                $1->insert( MatchResult( $3, NULL, $5 ) );
                $$ = $1;
            }
	;

providers
	:
	    {
                $$ = new SetOfServiceEntry();
            }
	| provider
	    {
                $$ = new SetOfServiceEntry();
                $$->insert( *$1 );
                delete $1;
            }
	| providers COLON provider
	    {
                $1->insert( *$3 );
                $$ = $1;
                delete $3;
	    }
	;

provider
	: string COMMA string
	    {
                $$ = new ServiceEntry( *$1, *$3 );
                delete $1;
                delete $3;
            }
	;

string
	: WORD
	    {
                $$ = new string( *$1 );
                delete $1;
            }
	| STRING
	    {
                $$ = $1;
            }
	;

string_list
	: string
	    {
                SetOfString *sl = new SetOfString();
                sl->insert( *$1 );
                delete $1;
                $$ = sl;
	    }
	| string_list COMMA string
	    {
                $1->insert( *$3 );
                delete $3;
                $$ = $1;
	    }
	;


word_list
	: WORD
	    {
                SetOfWord *wl = new SetOfWord();
                wl->insert( *$1 );
                $$ = wl;
	    }
	| word_list COMMA WORD
	    {
                $1->insert( *$3 );
                $$ = $1;
	    }
	;

words
	:
	    {
                $$ = new SetOfWord();
            }
	| WORD
	    {
                $$ = new SetOfWord();
                $$->insert( *$1 );
            }
	| words COMMA WORD
	    {
                $1->insert( *$3 );
                $$ = $1;
            }
	;

word_rel_prop
	:
	    {
                $$ = Dict::RelNone;
	    }
	| COLON word_rel_prop2
	    {
                $$ = $2;
	    }
	;

word_rel_prop2
	: simple_word_rel_prop
	    {
                $$ = $1;
	    }
	| word_rel_prop2 COMMA simple_word_rel_prop
	    {
                $$ = $1 | $3;
	    }
	;

simple_word_rel_prop
	: REL_SYMMETRIC
	    {
                $$ = Dict::RelSymmetric;
	    }
	| REL_REFLEXIVE
	    {
                $$ = Dict::RelReflexive;
	    }
	| REL_TRANSITIVE
	    {
                $$ = Dict::RelTransitive;
	    }
	| REL_ACYCLIC
	    {
                $$ = Dict::RelAcyclic;
	    }
	;

cg
	: concept_node con_connects_to
	    {
                $2->SetNode( $1 );
                $$ = $2;
	    }
	;

con_connects_to
	:
	    {
                $$ = new ConceptualGraph( NULL );
	    }
	| CONNECTS rel_graph
	    {
                $$ = new ConceptualGraph( NULL );
                $$->SetWidth( $2 );
	    }
	| SUBGRAPH_OPEN rel_subgraph SUBGRAPH_CLOSE
	    {
                $$ = new ConceptualGraph( NULL );
                $$->SetWidth( $2 );
	    }
	;

rel_graph
	: relation_node CONNECTS cg
	    {
                $$ = new ConceptualGraph( $1, $3 );
	    }
	| relation_node SUBGRAPH_OPEN con_subgraph SUBGRAPH_CLOSE
	    {
                $$ = new ConceptualGraph( $1, $3 );
	    }
	;

con_subgraph
	: CONNECTS cg
	    {
                $$ = $2;
	    }
	| CONNECTS cg COMMA con_subgraph
	    {
                $2->SetDepth( $4 );
                $$ = $2;
	    }
	;

rel_subgraph
	: CONNECTS rel_graph
	    {
                $$ = $2;
	    }
	| CONNECTS rel_graph COMMA rel_subgraph
	    {
                $2->SetDepth( $4 );
                $$ = $2;
	    }
	;

concept_node
	: CONCEPT_OPEN WORD CONCEPT_CLOSE
	    {
                $$ = new ConceptNode( *$2 );
                delete $2;
	    }
	| CONCEPT_OPEN WORD COLON VALUE CONCEPT_CLOSE
	    {
                ConceptNode *con = new ConceptNode( *$2 );
                con->SetValue( (long) $4 );
                $$ = con;
                delete $2;
            }
	| CONCEPT_OPEN WORD COLON ID_LIST_OPEN word_list ID_LIST_CLOSE CONCEPT_CLOSE
	    {
                $$ = new ConceptNode( *$2, *$5 );
                delete $2;
                delete $5;
	    }
	;

relation_node
	: RELATION_OPEN WORD RELATION_CLOSE
	    {
                $$ = new RelationNode( *$2 );
                delete $2;
	    }
	;

properties
	:
	    {
                $$ = 0;
            }
	| properties property
	    {
                $$ = $1 | $2;
            }
	;
            
property
	: REL_SYMMETRIC
	    {
                $$ = Dict::RelSymmetric;
            }
	| REL_REFLEXIVE
	    {
                $$ = Dict::RelReflexive;
            }
	| REL_TRANSITIVE
	    {
                $$ = Dict::RelTransitive;
            }
	| REL_ACYCLIC
	    {
                $$ = Dict::RelAcyclic;
            }
	;
